@@ -1,14 +1,14 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: checkpc.c,v 5.27 2000/12/25 01:51:04 papowell Exp papowell $";
+"$Id: checkpc.c,v 1.30 2002/05/06 01:06:40 papowell Exp $";
 
 
 
@@ -25,6 +25,7 @@
 #include "stty.h"
 #include "proctitle.h"
 #include "lpd_remove.h"
+#include "linksupport.h"
 #include "gethostinfo.h"
 
 /**** ENDINCLUDE ****/
@@ -50,7 +51,7 @@ int main( int argc, char *argv[], char *envp[] )
 	int ruid, euid, rgid, egid;
 	char *printcap;
 	char *serial_line = 0;
-	struct line_list raw;
+	struct line_list raw, spooldirs;
 	char *s, *t;
 	struct stat statb;
 
@@ -59,8 +60,9 @@ int main( int argc, char *argv[], char *envp[] )
 
 	s = t = printcap = 0;
 	Init_line_list(&raw);
+	Init_line_list(&spooldirs);
 	/* set up the uid state */
-	To_root();
+	To_euid_root();
 	time(&Current_time);
 
 	Verbose = 0;
@@ -110,7 +112,6 @@ int main( int argc, char *argv[], char *envp[] )
 			case 'p': ++Printcap; break;
 			case 'P': User_specified_printer = Optarg; break;
 			case 'T':
-				To_user();
 				initsetproctitle( argc, argv, envp );
 				Test_port( getuid(), geteuid(), serial_line );
 				exit(0);
@@ -125,6 +126,8 @@ int main( int argc, char *argv[], char *envp[] )
 	Initialize(argc, argv, envp, 'D' );
 	Setup_configuration();
 
+	To_daemon();
+
 	/* we have a user specified printcap we can check as well */
 	Free_line_list(&raw);
 	for( i = Optind; i < argc; ++i ){
@@ -136,7 +139,9 @@ int main( int argc, char *argv[], char *envp[] )
 		&PC_info_line_list, &raw, &Host_IP );
 	Free_line_list( &raw );
 
-	To_root();
+	if( Fix && geteuid() && getuid() ){
+		WARNMSG("Fix option (-f) requires root permissions\n" );
+	}
 
 	if(Verbose)MESSAGE("Checking for configuration files '%s'", Config_file_DYN);
 	found_pc = Check_path_list( Config_file_DYN, 0 );
@@ -145,10 +150,11 @@ int main( int argc, char *argv[], char *envp[] )
 	}
 
 	if(Verbose)MESSAGE("Checking for printcap files '%s'", Printcap_path_DYN);
-	found_pc = Check_path_list( Printcap_path_DYN, 0 );
-	if( Lpd_printcap_path_DYN ){
+	if( Is_server && Lpd_printcap_path_DYN ){
 		if(Verbose)MESSAGE("Checking for lpd only printcap files '%s'", Lpd_printcap_path_DYN);
 		found_pc += Check_path_list( Lpd_printcap_path_DYN, 1 );
+	} else {
+		found_pc += Check_path_list( Printcap_path_DYN, 0 );
 	}
 	if( found_pc == 0 ){
 		WARNMSG("No printcap files!!!" );
@@ -190,12 +196,10 @@ int main( int argc, char *argv[], char *envp[] )
 				if( Fix ){
 					mkdir_path( path );
 				}
-			} else {
-				*s = '/';
-				To_root();
-				Make_write_file( path, 0 );
 			}
+			*s = '/';
 		}
+		if( path ) free( path ); path = 0;
 		Spool_file_perms_DYN = oldfile;
 	}
 
@@ -217,44 +221,13 @@ int main( int argc, char *argv[], char *envp[] )
 		DEBUG1("checkpc: for SERVER %s is really %s", User_specified_printer, s );
 		if( s ){
 			Set_DYN(&Printer_DYN,s);
-			Scan_printer();
+			Scan_printer(&spooldirs);
 		}
 	} else {
 		if( DEBUGL1 ) Dump_line_list("checkpc: all", &All_line_list );
 		for( i = 0; i < All_line_list.count; ++i ){
 			Set_DYN(&Printer_DYN,All_line_list.list[i]);
-			Scan_printer();
-		}
-	}
-
-	if(Verbose){
-		MESSAGE("*** Checking for client info ****");
-	}
-	Is_server = 0;
-	Initialize(argc, argv, envp, 'D' );
-	Setup_configuration();
-	Free_line_list(&raw);
-	for( i = Optind; i < argc; ++i ){
-		Getprintcap_pathlist( Require_configfiles_DYN,
-			&raw, &PC_filters_line_list,
-			argv[i] );
-	}
-	Build_printcap_info( &PC_names_line_list, &PC_order_line_list,
-		&PC_info_line_list, &raw, &Host_IP );
-	Free_line_list( &raw );
-	if(Verbose)Show_all_printcap_entries();
-	if( User_specified_printer ){
-		if( DEBUGL1 ) Dump_line_list("checkpc: names", &PC_names_line_list );
-		s = Find_str_value( &PC_names_line_list, User_specified_printer, Value_sep );
-		DEBUG1("checkpc: for CLIENT %s is really %s", User_specified_printer, s );
-		if( s ){
-			Set_DYN(&Printer_DYN,s);
-			Scan_printer();
-		}
-	} else {
-		for( i = 0; i < All_line_list.count; ++i ){
-			Set_DYN(&Printer_DYN,All_line_list.list[i]);
-			Scan_printer();
+			Scan_printer(&spooldirs);
 		}
 	}
 
@@ -267,6 +240,8 @@ int main( int argc, char *argv[], char *envp[] )
             }
         }
     }
+	Free_line_list(&raw);
+	Free_line_list(&spooldirs);
 	return(0);
 }
 
@@ -308,7 +283,7 @@ void mkdir_path( char *path )
 	"filter", "bp", "bs", "be", 0
 	};
 
-void Scan_printer(void)
+void Scan_printer(struct line_list *spooldirs)
 {
 	DIR *dir;
 	struct dirent *d;
@@ -386,6 +361,13 @@ void Scan_printer(void)
 			Printer_DYN);
 		return;
 	}
+	if( (s =  Find_str_value(spooldirs,Spool_dir_DYN,Value_sep)) ){
+		WARNMSG("%s: CATASTOPHIC ERROR! queue '%s' also has spool directory '%s'",
+			Printer_DYN, s, Spool_dir_DYN);
+		return;
+	}
+	Set_str_value(spooldirs,Spool_dir_DYN,Printer_DYN);
+	
 
 	/*
 	 * check the permissions of files and directories
@@ -401,6 +383,9 @@ void Scan_printer(void)
 			Printer_DYN, Spool_dir_DYN, Errormsg(errno) );
 		return;
 	}
+	if( (Fix || Remove) && Lpq_status_file_DYN ){
+		unlink(Lpq_status_file_DYN);
+	}
 	while( (d = readdir(dir)) ){
 		cf_name = d->d_name;
 		if( safestrcmp( cf_name, "." ) == 0
@@ -411,6 +396,10 @@ void Scan_printer(void)
 				cf_name, Errormsg(errno) );
 			continue;
 		}
+		/* do not touch symbolic links */
+		if( S_ISLNK( statb.st_mode ) ){
+			continue;
+		}
 		delta = Current_time - statb.st_mtime;
 
 		/*
@@ -423,22 +412,42 @@ void Scan_printer(void)
 				&& isalpha(cval(cf_name+1))
 				&& isalpha(cval(cf_name+2))
 				&& isdigit(cval(cf_name+3)) );
-		if( !jobfile ) jobfile = ( !safestrncmp( cf_name,"temp",4 ) );
 
-		if( jobfile && delta > Age && Remove ){
+		if( jobfile && Age && delta > Age ){
+			float n = (delta)/60.0 ;
+			float a = (Age)/60.0 ;
+			char *remove = Remove?" (removing)":"";
+			char *range = "mins";
+			if( a/60 > 2 ){
+				a = a/60;
+				n = n/60;
+				range = "hours";
+				if( a/24 > 2 ){
+					a = a/24;
+					n = n/24;
+					range = "days";
+				}
+			}
             if( (statb.st_size == 0) ){
-				if(Verbose)MESSAGE( "  file '%s', zero length file %ld hours old",
-					cf_name, (long)((delta+3599)/3600) );
-				unlink(cf_name);
+				if( Remove || Verbose)MESSAGE( " %s:  file '%s', zero length file > %3.2f %s old%s",
+					Printer_DYN, cf_name, n, range, remove );
+				if( Remove ){
+					unlink(cf_name);
+				}
 				continue;
 			} else {
-				if(Verbose)MESSAGE( "  file '%s', %ld hours old",
-					cf_name, (long)((delta+3599)/3600) );
-				unlink(cf_name);
+				if( Remove || Verbose)MESSAGE( " %s:  file '%s', age %3.2f %s > %3.2f %s maximum%s",
+					Printer_DYN, cf_name, n, range, a, range, remove );
+				if( Remove ){
+					unlink(cf_name);
+				}
 				continue;
 			}
 		}
-		Check_file( cf_name, Fix, 0, 0 );
+		/* we update all real files in this directory */
+		if( jobfile ){
+			Check_file( cf_name, Fix, 0, 0 );
+		}
 	}
 	closedir(dir);
 
@@ -453,10 +462,11 @@ void Scan_printer(void)
 	 */
 	if( Fix ){
 		if( Lpq_status_file_DYN ) unlink(Lpq_status_file_DYN );
-		if( Queue_db_file_DYN ) unlink(Queue_db_file_DYN );
 	}
 	Free_line_list( &Sort_order );
-	Scan_queue( &Spool_control, &Sort_order,0,0,0,0, 1, 0 );
+	{ int fdx = open("/dev/null",O_RDWR); DEBUG1("Scan_printer: Scan_queue before maxfd %d", fdx); close(fdx); }
+	Scan_queue( &Spool_control, &Sort_order,0,0,0,0, 0, 0 );
+	{ int fdx = open("/dev/null",O_RDWR); DEBUG1("Scan_printer: Scan_queue after maxfd %d", fdx); close(fdx); }
 
 	/*
 	 * check to see if we have a local or remote printer
@@ -478,7 +488,7 @@ void Scan_printer(void)
 		} else if( stat(s,&statb) < 0 ){
 			WARNMSG( "%s: cannot stat lp device '%s' - %s",
 				Printer_DYN, s, Errormsg(errno) );
-		} else if( (fd = Checkwrite(s,&statb,0,0,0)) < 0 ){
+		} else if( (fd = Checkwrite(s,&statb,0,0,1)) < 0 ){
 			WARNMSG( "%s: cannot open lp device '%s' - %s",
 				Printer_DYN, s, Errormsg(errno) );
 		}
@@ -490,25 +500,39 @@ void Scan_printer(void)
 	for( i = 'a'; i <= 'z'; ++i ){
 		if( safestrchr("afls",i) ) continue;
 		error[0] = i;
-		Check_executable_filter( error );
+		Check_executable_filter( error, 0 );
 	}
 
 	for( names = filter_names; (s = *names); ++names ){
-		Check_executable_filter( s );
+		Check_executable_filter( s, 0 );
+	}
+
+	/* check the Lpd_port_DYN */
+	n = 0;
+	if( (s = safestrchr( Lpd_port_DYN, '%')) ){
+		n = Link_dest_port_num(s+1);
+	} else if( Lpd_port_DYN ){
+		n = Link_dest_port_num(Lpd_port_DYN);
+	}
+	if( n == 0 ){
+		WARNMSG( "%s: bad lpd_port value '%s'",
+			Printer_DYN, Lpd_port_DYN );
 	}
 }
 
-void Check_executable_filter( char *id )
+void Check_executable_filter( char *id, char *filter_str )
 {
 	struct line_list files;
-	char *s, *t, *filter_str;
+	char *s, *t;
 	struct stat statb;
 	int c, j, n;
 	
 	Init_line_list(&files);
-	filter_str = Find_str_value(&PC_entry_line_list,id,Value_sep);
-	if(!filter_str) filter_str = Find_str_value(&Config_line_list,id,Value_sep);
-	Split(&files,filter_str,Whitespace,0,0,0,0,0);
+	if( !filter_str ){
+		filter_str = Find_str_value(&PC_entry_line_list,id,Value_sep);
+		if(!filter_str) filter_str = Find_str_value(&Config_line_list,id,Value_sep);
+	}
+	Split(&files,filter_str,Whitespace,0,0,0,0,0,0);
 	if( files.count ){
 		if(Verbose)MESSAGE("  '%s' filter '%s'", id, filter_str );
 		s = 0;
@@ -527,7 +551,7 @@ void Check_executable_filter( char *id )
 		c = cval(s);
 		if( c == '(' || strpbrk( s, "<>|;") ){
 			if(Verbose)MESSAGE("    shell script '%s'", filter_str );
-			t = filter_str + strlen(filter_str) - 1;
+			t = filter_str + safestrlen(filter_str) - 1;
 			while( isspace(cval(t)) ) --t;
 			if( cval(t) != ')' ){
 				WARNMSG("filter needs ')' at end - '%s'", filter_str );
@@ -579,14 +603,20 @@ void Make_write_file( char *file, char *printer )
 		if(Verbose)MESSAGE( "  checking '%s' file", s );
 	}
 
-	if( (fd = Checkread( s, &statb )) < 0 ){
+	if( (fd = Checkwrite( s, &statb, O_RDWR, 1, 1 )) < 0 ){
+		WARNMSG( " ** cannot open '%s' - '%s'", s, Errormsg(errno) );
 		if( Fix ){
-			fd = Checkwrite( s, &statb, O_RDWR, 1, 0 );
+			int euid = geteuid();
+			To_euid_root();
+			fd = open( s, O_RDWR|O_CREAT, Spool_file_perms_DYN  );
+			To_euid(euid);
+			if( fd < 0 ){
+				WARNMSG( " ** cannot create '%s' - '%s'", s, Errormsg(errno) );
+			}
+			Fix_owner( s );
 		}
 	}
-	if( fd < 0 ){
-		WARNMSG( " ** cannot open '%s' - '%s'", s, Errormsg(errno) );
-	} else if( Check_file( s, Fix, 0, 0 ) ){
+	if( Check_file( s, Fix, 0, 0 ) ){
 		WARNMSG("  ** ownership or permissions problem with '%s'", s );
 	}
 	if( s ) free(s); s = 0;
@@ -619,6 +649,8 @@ void usage(void)
 	for( s = usemsg; *s; ++s ){
 		FPRINTF( STDERR, "%s\n", *s );
 	}
+	Parse_debug("=",-1);
+	FPRINTF( STDOUT, "%s\n", Version );
 	exit(1);
 }
 
@@ -692,6 +724,7 @@ int Check_file( char  *path, int fix, int age, int rmflag )
 		WARNMSG( "'%s' not a regular file - unusual", path );
 		return(2) ;
 	}
+
 	if( statb.st_uid != DaemonUID || statb.st_gid != DaemonGID ){
 		WARNMSG( "owner/group of '%s' are %d/%d, not %d/%d", path,
 			(int)(statb.st_uid), (int)(statb.st_gid), DaemonUID, DaemonGID );
@@ -729,7 +762,7 @@ int Fix_create_dir( char  *path, struct stat *statb )
 	char *s;
 	int err = 0;
 
-	s = path+strlen(path)-1;
+	s = path+safestrlen(path)-1;
 	if( *s == '/' ) *s = 0;
 	if( stat( path, statb ) == 0 ){
 		if( !S_ISDIR( statb->st_mode ) ){
@@ -745,14 +778,15 @@ int Fix_create_dir( char  *path, struct stat *statb )
 	}
 	/* we don't have a directory */
 	if( stat( path, statb ) ){
-		To_root();
+		int euid = geteuid();
+		To_euid_root();
 		if( mkdir( path, Spool_dir_perms_DYN ) ){
 			WARNMSG( "mkdir '%s' failed, %s", path, Errormsg(errno) );
 			err = 1;
 		} else {
 			err = Fix_owner( path );
 		}
-		To_user();
+		To_euid(euid);
 	}
 	return( err );
 }
@@ -761,11 +795,12 @@ int Fix_owner( char *path )
 {
 	int status = 0;
 	int err;
+	int euid = geteuid();
 
-	To_root();
+	To_euid_root();
 	WARNMSG( "  changing ownership '%s' to %d/%d", path, DaemonUID, DaemonGID );
 	chown( path, DaemonUID, DaemonGID );
-	if( geteuid() == 0 ){
+	if( geteuid() == ROOTUID ){
 		WARNMSG( "  changing ownership '%s' to %d/%d", path, DaemonUID, DaemonGID );
 		status = chown( path, DaemonUID, DaemonGID );
 		err = errno;
@@ -774,7 +809,7 @@ int Fix_owner( char *path )
 		}
 		errno = err;
 	}
-	To_user();
+	To_euid(euid);
 	return( status != 0 );
 }
 
@@ -782,11 +817,12 @@ int Fix_perms( char *path, int perms )
 {
 	int status;
 	int err;
+	int euid = geteuid();
 
-	To_root();
+	To_euid_root();
 	status = chmod( path, perms );
-	To_daemon();
 	err = errno;
+	To_euid( euid );
 
 	if( status ){
 		WARNMSG( "chmod '%s' to 0%o failed, %s", path, perms,
@@ -811,7 +847,7 @@ int Check_spool_dir( char *path, int owner )
 	/* get the required group and user ids */
 	
 	if(Verbose)MESSAGE(" Checking directory: '%s'", path );
-	pathname = path+strlen(path)-1;
+	pathname = path+safestrlen(path)-1;
 	if( pathname[0] == '/' ) *pathname = 0;
 	Init_line_list(&parts);
 	if( path == 0 || path[0] != '/' || strstr(path,"/../") ){
@@ -820,7 +856,7 @@ int Check_spool_dir( char *path, int owner )
 	}
 
 	pathname = 0;
-	Split(&parts,path,"/",0,0,0,0,0);
+	Split(&parts,path,"/",0,0,0,0,0,0);
 
 	for( i = 0; i < parts.count; ++i ){
 		pathname = safeextend3(pathname,"/",parts.list[i],__FILE__,__LINE__);
@@ -835,7 +871,6 @@ int Check_spool_dir( char *path, int owner )
 				return( 2 );
 			}
 		}
-		To_daemon();
 		if( stat( pathname, &statb ) == 0 && S_ISDIR( statb.st_mode )
 			&& chdir( pathname ) == -1 ){
 			if( !Fix ){
@@ -862,11 +897,14 @@ int Check_spool_dir( char *path, int owner )
 	if( !owner ) return(err);
 	if( Fix ){
 		char cmd[SMALLBUFFER];
+		int euid = geteuid();
+
+		To_euid_root();
 		SNPRINTF( cmd, sizeof(cmd)) "%s -R %d %s", CHOWN, DaemonUID, path );
 		system( cmd );
 		SNPRINTF( cmd, sizeof(cmd)) "%s -R %d %s", CHGRP, DaemonGID, path );
 		system( cmd );
-		To_daemon();
+		To_euid(euid);
 	}
 	if( stat( path, &statb ) ){
 		WARNMSG( "stat of '%s' failed - %s", path, Errormsg(errno) );
@@ -935,25 +973,25 @@ void Test_port(int ruid, int euid, char *serial_line )
 
 	Spool_file_perms_DYN = 000600;
 	Spool_dir_perms_DYN =  042700;
-	if( ( ruid == 0 && euid == 0 ) || (ruid != 0 && euid != 0 ) ){
+	if( ( ruid == ROOTUID && euid == ROOTUID ) || (ruid != ROOTUID && euid != ROOTUID ) ){
 			FPRINTF( STDERR,
 				"*******************************************************\n" );
 			FPRINTF( STDERR, "***** not SETUID, skipping setuid checks\n" );
 			FPRINTF( STDERR,
 				"*******************************************************\n" );
 			goto freespace;
-	} else if( ( ruid == 0 || euid == 0 ) ){
+	} else if( ( ruid == ROOTUID || euid == ROOTUID ) ){
 		if( UID_root == 0 ){
 		FPRINTF( STDERR,
 			"checkpc: setuid code failed!! Portability problems\n" );
 			exit(1);
 		}
-		if( To_uid(1) ){
+		if( To_euid(1) ){
 			FPRINTF( STDERR,
-			"checkpc: To_uid() seteuid code failed!! Portability problems\n" );
+			"checkpc: To_euid() seteuid code failed!! Portability problems\n" );
 			exit(1);
 		}
-		if( To_user() ){
+		if( To_daemon() ){
 			FPRINTF( STDERR,
 			"checkpc: To_usr() seteuid code failed!! Portability problems\n" );
 			exit(1);
@@ -1404,20 +1442,18 @@ void Fix_clean( char *s, int no )
 int Check_path_list( char *plist, int allow_missing )
 {
 	struct line_list values;
-	char *path, *s;
+	char *path;
 	int found_pc = 0, i, fd;
 	struct stat statb;
 
 	Init_line_list(&values);
-	Split(&values,plist,File_sep,0,0,0,0,0);
+	Split(&values,plist,File_sep,0,0,0,0,0,0);
 	for( i = 0; i < values.count; ++i ){
 		path = values.list[i];
 		if( path[0] == '|' ){
 			++path;
-			while( isspace(cval(path)) ) ++path;
-			if( (s = strpbrk(path,Whitespace)) ) *s = 0;
-		}
-		if( path[0] == '/' ){
+			Check_executable_filter( path, 0 );
+		} else if( path[0] == '/' ){
 			if( (fd = Checkread( path,&statb ) ) < 0 ){
 				if( stat( path, &statb ) ){
 					if( ! allow_missing ) WARNMSG(" '%s' not present", path );
@@ -1440,12 +1476,3 @@ int Check_path_list( char *plist, int allow_missing )
 	Free_line_list(&values);
 	return(found_pc);
 }
- void Dispatch_input(int *talk, char *input ){}
-
-/*
- * Calls[] = list of dispatch functions 
- */
-
- struct call_list Calls[] = {
-	{0,0}
-};
