@@ -1,14 +1,14 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: copyright.c,v 5.6 2000/12/25 01:51:05 papowell Exp papowell $";
+"$Id: copyright.c,v 1.30 2002/05/06 01:06:40 papowell Exp $";
 
 
 #include "lp.h"
@@ -23,7 +23,7 @@ char *Copyright[] = {
 #if defined(MIT_KERBEROS4)
  ", MIT Kerberos4"
 #endif
-", Copyright 1988-2000 Patrick Powell, <papowell@lprng.com>",
+", Copyright 1988-2002 Patrick Powell, <papowell@lprng.com>",
 
 "",
 "locking uses: "
@@ -48,6 +48,12 @@ char *Copyright[] = {
             "termios"
 #endif
 ,
+#ifdef USE_GDBM
+"with GDBM"
+#else
+"without GDBM"
+#endif
+,
 "",
 #include "license.h"
 #include "copyright.h"
