@@ -1,14 +1,14 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: fileopen.c,v 5.9 2000/12/25 01:51:05 papowell Exp papowell $";
+"$Id: fileopen.c,v 1.30 2002/05/06 01:06:40 papowell Exp $";
 
 
 #include "lp.h"
@@ -18,14 +18,14 @@
 /**** ENDINCLUDE ****/
 
 /***************************************************************************
-Commentary:
-Patrick Powell Mon May  1 05:37:02 PDT 1995
+  Commentary:
+  Patrick Powell Mon May  1 05:37:02 PDT 1995
+   
+  These routines were created in order to centralize all file open
+  and checking.  Hopefully,  if there are portability problems, these
+  routines will be the only ones to change.
  
-These routines were created in order to centralize all file open
-and checking.  Hopefully,  if there are portability problems, these
-routines will be the only ones to change.
- 
- ***************************************************************************/
+***************************************************************************/
 
 /***************************************************************************
  * int Checkread( char *file, struct stat *statb )
@@ -126,18 +126,20 @@ int Checkwrite( const char *file, struct stat *statb, int rw, int create,
 		if( mask == -1 ){
 			LOGERR(LOG_ERR) "Checkwrite: fcntl F_GETFL of '%s' failed", file);
 			status = -1;
-		} else {
+		} else if( mask & NONBLOCK ){
 			DEBUG3( "Checkwrite: F_GETFL value '0x%x', BLOCK 0x%x",
 				mask, NONBLOCK );
 			mask &= ~NONBLOCK;
 			mask = fcntl( fd, F_SETFL, mask );
-			if( mask == -1 ){
+			err = errno;
+			DEBUG3( "Checkwrite: after F_SETFL value now '0x%x'",
+				fcntl( fd, F_GETFL, 0 ) );
+			if( mask == -1 && err != ENODEV ){
+				errno = err;
 				LOGERR(LOG_ERR) "Checkwrite: fcntl F_SETFL of '%s' failed",
 					file );
 				status = -1;
 			}
-			DEBUG3( "Checkwrite: after F_SETFL value now '0x%x'",
-				fcntl( fd, F_GETFL, 0 ) );
 		}
 	}
 
