@@ -1,14 +1,14 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: gethostinfo.c,v 5.15 2000/12/25 01:51:05 papowell Exp papowell $";
+"$Id: gethostinfo.c,v 1.30 2002/05/06 01:06:40 papowell Exp $";
 
 /********************************************************************
  * char *get_fqdn (char *shorthost)
@@ -98,7 +98,7 @@ char *Find_fqdn( struct host_information *info, const char *shorthost )
 			shorthost );
 		return(0);
 	}
-	if( strlen(shorthost) > 64 ){
+	if( safestrlen(shorthost) > 64 ){
 		FATAL(LOG_ALERT) "Find_fqdn: hostname too long, HACKER ALERT '%s'",
 			shorthost );
 	}
@@ -150,7 +150,7 @@ char *Fixup_fqdn( const char *shorthost, struct host_information *info,
 	 */
 	fqdn = 0;
 	if( safestrchr( host_ent->h_name, '.' ) ){
-		fqdn = host_ent->h_name;
+		fqdn = (char *)host_ent->h_name;
 	} else if( (list = host_ent->h_aliases) ){
 		for( ; *list && !safestrchr(*list,'.'); ++list );
 		fqdn = *list;
@@ -191,22 +191,22 @@ char *Fixup_fqdn( const char *shorthost, struct host_information *info,
 
 	if( fqdn == 0 ){
 		if( safestrchr( host_ent->h_name, '.' ) ){
-			fqdn = host_ent->h_name;
+			fqdn = (char *)host_ent->h_name;
 		} else if( (list = host_ent->h_aliases) ){
 			for( ; *list && !safestrchr(*list,'.'); ++list );
 			fqdn = *list;
 		}
-		if( fqdn == 0 ) fqdn = host_ent->h_name;
+		if( fqdn == 0 ) fqdn = (char *)host_ent->h_name;
 	}
 
 	info->h_addrtype = host_ent->h_addrtype;
 	info->h_length = host_ent->h_length;
 	/* put the names in the database */
-	info->fqdn = safestrdup(fqdn,__FILE__,__LINE__);
+	fqdn = info->fqdn = safestrdup(fqdn,__FILE__,__LINE__);
 	info->shorthost = safestrdup(fqdn,__FILE__,__LINE__);
 	if( (s = safestrchr(info->shorthost,'.')) ) *s = 0;
 
-	Add_line_list(&info->host_names,host_ent->h_name,0,0,0 );
+	Add_line_list(&info->host_names,(char *)host_ent->h_name,0,0,0 );
 	for( list = host_ent->h_aliases; list && (s = *list); ++list ){
 		Add_line_list(&info->host_names,s,0,0,0 );
 	}
@@ -364,18 +364,18 @@ char *Get_hostinfo_byaddr( struct host_information *info,
 		info->h_addr_list.list[info->h_addr_list.count] = 0;
 
 		const_s = inet_ntop_sockaddr( sinaddr, buffer, sizeof(buffer) );
-		info->fqdn = safestrdup(const_s,__FILE__,__LINE__);
+		fqdn = info->fqdn = safestrdup(const_s,__FILE__,__LINE__);
+		info->shorthost = safestrdup(fqdn,__FILE__,__LINE__);
 		Add_line_list( &info->host_names,info->fqdn,0,0,0);
-		info->shorthost = safestrdup(s,__FILE__,__LINE__);
 	}
 	return( fqdn );
 }
 
 char *Get_remote_hostbyaddr( struct host_information *info,
-	struct sockaddr *sinaddr )
+	struct sockaddr *sinaddr, int force_ip_addr_use )
 {
 	char *fqdn;
-	fqdn = Get_hostinfo_byaddr( info, sinaddr, 0 );
+	fqdn = Get_hostinfo_byaddr( info, sinaddr, force_ip_addr_use );
 	DEBUG3("Get_remote_hostbyaddr: %s", fqdn );
 	Set_DYN( &FQDNRemote_FQDN, info->fqdn );
 	Set_DYN( &ShortRemote_FQDN, info->shorthost );
@@ -415,10 +415,10 @@ int Same_host( struct host_information *host,
 						int n;
 						ls[0] = 0; rs[0] = 0;
 						for( n = 0; n < l1; ++n ){
-							SNPRINTF( ls + strlen(ls), 3) "%02x", h1[n] );
+							SNPRINTF( ls + safestrlen(ls), 3) "%02x", h1[n] );
 						}
 						for( n = 0; n < l1; ++n ){
-							SNPRINTF( rs + strlen(rs), 3) "%02x", h2[n] );
+							SNPRINTF( rs + safestrlen(rs), 3) "%02x", h2[n] );
 						}
 						LOGDEBUG("Same_host: comparing %s to %s, result %d",
 							ls, rs, result );
@@ -456,7 +456,7 @@ void Dump_host_information( char *title,  struct host_information *info )
 			SNPRINTF( msg, sizeof(msg)) "    [%d] 0x", i );
 			s = info->h_addr_list.list[i];
 			for( j = 0; j < info->h_length; ++j ){
-				len = strlen( msg );
+				len = safestrlen( msg );
 				SNPRINTF( msg+len, sizeof(msg)-len) "%02x",((unsigned char *)s)[j] );
 			}
 			LOGDEBUG( "%s", msg );
@@ -481,7 +481,7 @@ void form_addr_and_mask(char *v, char *addr,char *mask,
 	if( v == 0 ) return;
 	
 	DEBUG5("form_addr_and_mask: '%s'", v );
-	if( 4*addrlen+1 >= sizeof(buffer) ){
+	if( 4*addrlen+1 >= (int)(sizeof(buffer)) ){
 		FATAL(LOG_ERR)"form_addr_and_mask: addrlen too large - hacker attack?");
 	}
 	memset( addr, 0, addrlen );
@@ -509,20 +509,18 @@ void form_addr_and_mask(char *v, char *addr,char *mask,
 			for( i = 0; i < bytecount; ++i ){
 				if( buffer[0] ) *t++ = '.';
 				strcpy(t,"255");
-				t += strlen(t);
-				*t = 0;
+				t += safestrlen(t);
 			}
-			if( bitcount ){
+			if( bitcount && i < addrlen ){
 				if( buffer[0] ) *t++ = '.';
 				SNPRINTF(t,6) "%d", (~((1<<(8-bitcount))-1))&0xFF);
-				t += strlen(t);
-				*t = 0;
+				t += safestrlen(t);
+				++i;
 			}
 			for( ; i < addrlen; ++i ){
 				if( buffer[0] ) *t++ = '.';
 				strcpy(t,"0");
-				t += strlen(t);
-				*t = 0;
+				t += safestrlen(t);
 			}
 			DEBUG6("form_addr_and_mask: len %d '%s'", m, buffer );
 			result = inet_pton(family, buffer, mask );
@@ -608,6 +606,8 @@ int Match_ipaddr_value( struct line_list *list, struct host_information *host )
 				result = Globmatch( str, host->host_names.list[j] );
 			}
 			if( result ){
+				DEBUGF(DDB2)("Match_ipaddr_value: mask '%s'",
+					str );
 				form_addr_and_mask(str,addr,mask,host->h_length,
 					host->h_addrtype );
 				for( j = 0; result && j < host->h_addr_list.count; ++j ){
