@@ -1,26 +1,30 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: getopt.c,v 5.5 2000/12/25 01:51:06 papowell Exp papowell $";
+"$Id: getopt.c,v 1.30 2002/05/06 01:06:40 papowell Exp $";
 
 
 #include "lp.h"
 /**** ENDINCLUDE ****/
 
-int Optind;                 /* next argv to process */
-int Opterr = 1;                 /* Zero disables errors msgs */
-char *Optarg;               /* Pointer to option argument */
-char *next_opt;			    /* pointer to next option char */
-char *Name;					/* Name of program */
-char **Argv_p;
-int Argc_p;
+
+# if 0
+   --------- now in lp.h ---------
+ int Optind;                 /* next argv to process */
+ int Opterr = 1;                 /* Zero disables errors msgs */
+ char *Optarg;               /* Pointer to option argument */
+ char *Name;					/* Name of program */
+#endif
+ static char *next_opt;			    /* pointer to next option char */
+ static char **Argv_p;
+ static int Argc_p;
 
 int Getopt (int argc, char *argv[], char *optstring)
 {
