@@ -1,14 +1,14 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: getqueue.c,v 5.36 2000/12/28 01:32:56 papowell Exp papowell $";
+"$Id: getqueue.c,v 1.30 2002/05/06 01:06:40 papowell Exp $";
 
 
 /***************************************************************************
@@ -30,15 +30,14 @@
 #include "child.h"
 #include "errorcodes.h"
 #include "fileopen.h"
+#include "linelist.h"
 #include "getprinter.h"
+#include "gethostinfo.h"
 #include "getqueue.h"
 #include "globmatch.h"
 #include "permission.h"
 #include "lockfile.h"
 #include "merge.h"
-#ifdef HAVE_GDBM_H
-#include <gdbm.h>
-#endif
 
 #if defined(USER_INCLUDE)
 # include USER_INCLUDE
@@ -57,33 +56,25 @@
  *   and leave it to the checkpc code to clean them up.
  */
 
-#ifdef HAVE_GDBM_H
- GDBM_FILE gdbm_file = 0;
- int gdbm_lock = -1;
-#endif
-
 int Scan_queue( struct line_list *spool_control,
 	struct line_list *sort_order, int *pprintable, int *pheld, int *pmove,
-		int only_pr_and_move, int create_db, int write_db )
+		int only_queue_process, int *perr, int *pdone )
 {
 	DIR *dir;						/* directory */
 	struct dirent *d;				/* directory entry */
 	struct line_list directory_files;
-	char *hf_name, *sort_key, *outstr;
-	int c, printable, held, move, p, h, m;
+	char *hf_name;
+	int c, printable, held, move, error, done, p, h, m, e, dn;
 	struct job job;
-#ifdef HAVE_GDBM_H
-	datum key_data, data_data;
-	plp_block_mask oblock;
-#endif
 
-	sort_key = outstr = 0;
-	c = printable = held = move = 0;
+	c = printable = held = move = error = done = 0;
 	Init_job( &job );
 	Init_line_list(&directory_files);
-	if( pprintable ) *pprintable = printable;
-	if( pheld ) *pheld = held;
-	if( pmove ) *pmove = move;
+	if( pprintable ) *pprintable = 0;
+	if( pheld ) *pheld = 0;
+	if( pmove ) *pmove = 0;
+	if( perr ) *perr = 0;
+	if( pdone ) *pdone = 0;
 
 	Free_line_list(sort_order);
 
@@ -91,45 +82,21 @@ int Scan_queue( struct line_list *spool_control,
 		return(1);
 	}
 
-#ifdef HAVE_GDBM_H
-	memset(&key_data,0,sizeof(key_data));
-	memset(&data_data,0,sizeof(data_data));
-	if( Queue_db_file_DYN ){
-		create_db = Open_gdbm( create_db );
-	}
-#else
-	create_db = 1;	/* we always need to read files */
-#endif
-
 	hf_name = 0;
 	while(1){
-		if( create_db ){
-			hf_name = 0;
-			while( hf_name == 0 && (d = readdir(dir)) ){
-				hf_name = d->d_name;
-				DEBUG5("Scan_queue: found file '%s'", hf_name );
-				if(    (cval(hf_name+0) == 'h')
-					&& (cval(hf_name+1) == 'f')
-					&& isalpha(cval(hf_name+2))
-					&& isdigit(cval(hf_name+3))
-					){
-					break;
-				} else {
-					hf_name = 0;
-				}
-			}
-#ifdef HAVE_GDBM_H
-		} else if( gdbm_file ){
-			if( hf_name == 0 ){
-				key_data = gdbm_firstkey( gdbm_file );
+		hf_name = 0;
+		while( hf_name == 0 && (d = readdir(dir)) ){
+			hf_name = d->d_name;
+			DEBUG5("Scan_queue: found file '%s'", hf_name );
+			if(    (cval(hf_name+0) == 'h')
+				&& (cval(hf_name+1) == 'f')
+				&& isalpha(cval(hf_name+2))
+				&& isdigit(cval(hf_name+3))
+				){
+				break;
 			} else {
-				/* we now get the keys */
-				data_data = gdbm_nextkey( gdbm_file, key_data );
-				if( key_data.dptr ) free( key_data.dptr );
-				key_data = data_data;
+				hf_name = 0;
 			}
-			hf_name = key_data.dptr;
-#endif
 		}
 		/* found them all */
 		if( hf_name == 0 ){
@@ -141,25 +108,8 @@ int Scan_queue( struct line_list *spool_control,
 		DEBUG2("Scan_queue: processing file '%s'", hf_name );
 
 		/* read the hf file and get the information */
-		if( create_db ){
-			Get_file_image_and_split( hf_name, 0, 0,
-				&job.info, Line_ends, 1, Value_sep,1,1,1,0);
-#ifdef HAVE_GDBM_H
-		} else if( gdbm_file ){
-			struct stat statb;
-			if( stat( hf_name, &statb ) ){
-				/* we now delete the record */
-				plp_block_all_signals( &oblock );
-				gdbm_delete( gdbm_file, key_data );
-				plp_set_signal_mask( &oblock, 0 );
-			} else {
-				data_data = gdbm_fetch( gdbm_file, key_data );
-				Split( &job.info, data_data.dptr,
-					Line_ends, 1, Value_sep,1,1,1);
-				if( data_data.dptr ) free( data_data.dptr ); data_data.dptr = 0;
-			}
-#endif
-		}
+		Get_file_image_and_split( hf_name, 0, 0,
+			&job.info, Line_ends, 1, Value_sep,1,1,1,0);
 		if(DEBUGL5)Dump_line_list("Scan_queue: hf", &job.info );
 		if( job.info.count == 0 ){
 			continue;
@@ -167,66 +117,30 @@ int Scan_queue( struct line_list *spool_control,
 
 		/* get the data file from the control file */
 		Setup_cf_info( &job, 1 );
-		Job_printable(&job,spool_control, &p,&h,&m);
+		Job_printable(&job,spool_control, &p,&h,&m,&e,&dn);
 		if( p ) ++printable;
 		if( h ) ++held;
 		if( m ) ++move;
-
-		if( outstr ) free(outstr); outstr = 0;
-		outstr = Make_hf_image( &job );
+		if( e ) ++error;
+		if( dn ) ++done;
 
 		/* now generate the sort key */
-		if(DEBUGL4)Dump_job("Scan_queue - before Make_sort_key",&job);
-		sort_key = Make_sort_key( &job );
-		DEBUG5("Scan_queue: sort key '%s'",sort_key);
-		DEBUG4("Scan_queue: p %d, m %d, only_pr_and_move %d",
-			p, m, only_pr_and_move );
+		DEBUG4("Scan_queue: p %d, m %d, e %d, dn %d, only_queue_process %d",
+			p, m, e, dn, only_queue_process );
 		if( sort_order ){
-			if( only_pr_and_move ){
-				if( p || m ){
-					Set_str_value(sort_order,sort_key,hf_name);
-				}
-			} else {
-				Set_str_value(sort_order,sort_key,hf_name);
+			if( !only_queue_process || (p || m || e || dn) ){
+				if(DEBUGL4)Dump_job("Scan_queue - before Make_sort_key",&job);
+				Make_sort_key( &job );
+				DEBUG5("Scan_queue: sort key '%s'",job.sort_key);
+				Set_str_value(sort_order,job.sort_key,hf_name);
 			}
 		}
-
-#ifdef HAVE_GDBM_H
-		/* we now set the DB entry */
-		if( gdbm_file ){
-			key_data.dptr = hf_name;
-			key_data.dsize = strlen( hf_name ) + 1;
-			data_data.dptr = outstr;
-			data_data.dsize = strlen( outstr ) + 1;
-			plp_block_all_signals( &oblock );
-			m = gdbm_store( gdbm_file, key_data, data_data, GDBM_REPLACE );
-			plp_set_signal_mask( &oblock, 0 );
-			if( m ){
-				Close_gdbm();
-				unlink( Queue_db_file_DYN );
-				Errorcode = JABORT;
-				FATAL(LOG_ERR)"Scan_queue: DB '%s' not updated '%s'",
-					Queue_db_file_DYN, gdbm_strerror(gdbm_errno) );
-			}
-		}
-#endif
-		if(sort_key) free(sort_key); sort_key = 0;
-		if(outstr) free(outstr); outstr = 0;
 	}
+	closedir(dir);
 
-	if(sort_key) free(sort_key); sort_key = 0;
-	if(outstr) free(outstr); outstr = 0;
 	Free_job(&job);
 	Free_line_list(&directory_files);
 
-#ifdef HAVE_GDBM_H
-	if( !write_db ){
-		Close_gdbm();
-	}
-	DEBUG1("Scan_queue: gdbm_file 0x%x, gdbm file fd %d, lock fd %d",
-		Cast_ptr_to_int(gdbm_file), gdbm_file?gdbm_fdesc(gdbm_file):-1,  gdbm_lock );
-#endif
-
 	if(DEBUGL5){
 		LOGDEBUG("Scan_queue: final values" );
 		Dump_line_list_sub(SORT_KEY,sort_order);
@@ -234,8 +148,10 @@ int Scan_queue( struct line_list *spool_control,
 	if( pprintable ) *pprintable = printable;
 	if( pheld ) *pheld = held;
 	if( pmove ) *pmove = move;
-	DEBUG3("Scan_queue: printable %d, held %d, move %d", printable,
-		held, move );
+	if( perr ) *perr = error;
+	if( pdone ) *pdone = done;
+	DEBUG3("Scan_queue: final printable %d, held %d, move %d, error %d, done %d",
+		printable, held, move, error, done );
 	return(0);
 }
 
@@ -268,7 +184,7 @@ char *Get_fd_image( int fd, int maxsize )
 	}
 	if(DEBUGL3){
 		SNPRINTF(buffer,32)"%s",s);
-		logDebug("Get_fd_image: len %d '%s'", s?strlen(s):0, buffer );
+		logDebug("Get_fd_image: len %d '%s'", s?safestrlen(s):0, buffer );
 	}
 	return(s);
 }
@@ -309,7 +225,7 @@ int Get_fd_image_and_split( int fd,
 	s = Get_fd_image( fd, maxsize );
 	if( s == 0 ) return 1;
 	if( clean ) Clean_meta(s);
-	Split( l, s, sep, sort, keysep, uniq, trim, nocomments );
+	Split( l, s, sep, sort, keysep, uniq, trim, nocomments ,0);
 	if( return_image ){
 		*return_image = s;
 	} else {
@@ -337,7 +253,7 @@ int Get_file_image_and_split( const char *file,
 	}
 	if( s == 0 ) return 1;
 	if( clean ) Clean_meta(s);
-	Split( l, s, sep, sort, keysep, uniq, trim, nocomments );
+	Split( l, s, sep, sort, keysep, uniq, trim, nocomments ,0);
 	if( return_image ){
 		*return_image = s;
 	} else {
@@ -382,7 +298,7 @@ void Check_for_hold( struct job *job, struct line_list *spool_control )
  *  read the control file and hold file.
  */
 
-void Setup_job( struct job *job, struct line_list *spool_control, const char *dir,
+void Setup_job( struct job *job, struct line_list *spool_control,
 	const char *cf_name, const char *hf_name, int check_for_existence )
 {
 	struct stat statb;
@@ -442,6 +358,7 @@ void Setup_job( struct job *job, struct line_list *spool_control, const char *di
 		Set_decimal_value(&job->info,SIZE,size);
 	}
 
+	Make_identifier( job );
 	Check_for_hold( job, spool_control );
 
 	if(DEBUGL3)Dump_job("Setup_job",job);
@@ -465,7 +382,7 @@ int Get_hold_class( struct line_list *info, struct line_list *sq )
 		&& (t = Find_str_value(info,CLASS,Value_sep)) ){
 		held = 1;
 		Free_line_list(&l);
-		Split(&l,s,File_sep,0,0,0,0,0);
+		Split(&l,s,File_sep,0,0,0,0,0,0);
 		for( i = 0; held && i < l.count; ++i ){
 			held = Globmatch( l.list[i], t );
 		}
@@ -576,7 +493,7 @@ int Setup_cf_info( struct job *job, int check_for_existence )
 
 	hpformat = Find_flag_value(&job->info,HPFORMAT,Value_sep);
 	if( (s = Find_str_value(&job->info,DATAFILES,Value_sep)) ){
-		Split(&cf_line_list,s,"\001",0,0,0,0,0);
+		Split(&cf_line_list,s,"\001",0,0,0,0,0,0);
 	} else {
 		s = Find_str_value(&job->info,OPENNAME,Value_sep);
 		if( !s ) s = Find_str_value(&job->info,TRANSFERNAME,Value_sep);
@@ -587,11 +504,14 @@ int Setup_cf_info( struct job *job, int check_for_existence )
 			SNPRINTF(buffer,sizeof(buffer))
 				"no control file %s - %s", s, Errormsg(errno) );
 			Set_str_value(&job->info,ERROR,buffer);
+			Set_flag_value(&job->info,ERROR_TIME,time(0));
 			returnstatus = 1;
 			goto done;
 		}
 	}
 
+	Free_listof_line_list( &job->datafiles );
+
 	file_found = 0;
 	datafile = malloc_or_die(sizeof(datafile[0]),__FILE__,__LINE__);
 	memset(datafile,0,sizeof(datafile[0]));
@@ -637,6 +557,7 @@ int Setup_cf_info( struct job *job, int check_for_existence )
 					SNPRINTF(buffer,sizeof(buffer))
 						"missing data file %s - %s", file_found, Errormsg(errno) );
 					Set_str_value(&job->info,ERROR,buffer);
+					Set_flag_value(&job->info,ERROR_TIME,time(0));
 					returnstatus = 1;
 					goto done;
 				}
@@ -651,7 +572,7 @@ int Setup_cf_info( struct job *job, int check_for_existence )
 					if( s[3] ) Set_str_value(&job->info,"T",s+3);
 				} else if( c == 'O' ){
 					s = s+3;
-					if( strlen(s) ){
+					if( safestrlen(s) ){
 						for( t = s; (t = strpbrk(t," ")); ++t ){
 							*t = ',';
 						}
@@ -660,6 +581,7 @@ int Setup_cf_info( struct job *job, int check_for_existence )
 				}
 				continue;
 			}
+			/* if we have a file name AND an 'N' for it, then set up a new file */
 			if( file_found && (t = Find_str_value(datafile,"N",Value_sep))
 				/* && safestrcmp(t,s+1) */ ){
 				Check_max(&job->datafiles,1);
@@ -694,19 +616,21 @@ int Setup_cf_info( struct job *job, int check_for_existence )
 				buffer[0] = c; buffer[1] = 0;
 				DEBUG4("Setup_cf_info: control '%s'='%s'", buffer, s+1 );
 				Set_str_value(&job->info,buffer,s+1);
-			} else if( c == '_' ){
-				Set_str_value(&job->info,AUTHINFO,s+1); break;
 			}
 		}
 	}
-	if( Find_str_value(datafile,TRANSFERNAME,Value_sep) ){
+	if( file_found ){
 		Check_max(&job->datafiles,1);
 		job->datafiles.list[job->datafiles.count++] = (void *)datafile;
 	} else {
-		free(datafile); datafile = 0;
+		free(datafile);
 	}
+	datafile = 0;
 	Set_str_value(&job->info,FILENAMES,names);
+
  done:
+
+	if( datafile )	free(datafile); datafile=0;
 	if( names )	free(names); names=0;
 	Free_line_list( &cf_line_list );
 	if(DEBUGL4)Dump_job("Setup_cf_info - final",job);
@@ -717,7 +641,7 @@ char *Make_hf_image( struct job *job )
 {
 	char *outstr, *s;
 	int i;
-	int len = strlen(OPENNAME);
+	int len = safestrlen(OPENNAME);
 
 	outstr = 0;
 	for( i = 0; i < job->info.count; ++i ){
@@ -734,10 +658,10 @@ char *Make_hf_image( struct job *job )
  * Write a hold file
  */
 
-int Set_hold_file( struct job *job, struct line_list *perm_check )
+int Set_hold_file( struct job *job, struct line_list *perm_check, int fd )
 {
 	char *hf_name, *tempfile, *outstr;
-	int fd, status;
+	int status;
 
 	status = 0;
 	outstr = 0;
@@ -751,63 +675,31 @@ int Set_hold_file( struct job *job, struct line_list *perm_check )
 	Set_str_value(&job->info,UPDATE_TIME,Time_str(0,0));
 	outstr = Make_hf_image( job );
 
-	fd = Make_temp_fd( &tempfile );
-	if( Write_fd_str(fd, outstr) < 0 ){
-		LOGERR(LOG_INFO)"Set_hold_file: write to '%s' failed", tempfile );
-		status = 1;
+	if( !fd ){
+		fd = Make_temp_fd( &tempfile );
+		if( Write_fd_str(fd, outstr) < 0 ){
+			LOGERR(LOG_INFO)"Set_hold_file: write to '%s' failed", tempfile );
+			status = 1;
+		}
+		close(fd);
+		if( status == 0 && rename( tempfile, hf_name ) == -1 ){
+			LOGERR(LOG_INFO)"Set_hold_file: rename '%s' to '%s' failed",
+				tempfile, hf_name );
+			status = 1;
+		}
+	} else {
+		if( lseek( fd, 0, SEEK_SET ) == -1 ){
+			LOGERR_DIE(LOG_ERR) "Set_hold_file: lseek failed" );
+		}
+		if( ftruncate( fd, 0 ) ){
+			LOGERR_DIE(LOG_ERR) "Set_hold_file: ftruncate failed" );
+		}
+		if( Write_fd_str(fd, outstr) < 0 ){
+			LOGERR(LOG_INFO)"Set_hold_file: write to '%s' failed", hf_name );
+			status = 1;
+		}
+		/* close(fd); */
 	}
-	close(fd);
-	if( status == 0 && rename( tempfile, hf_name ) == -1 ){
-		LOGERR(LOG_INFO)"Set_hold_file: rename '%s' to '%s' failed",
-			tempfile, hf_name );
-		status = 1;
-	}
-
-#ifdef HAVE_GDBM_H
-	if( status == 0 && Queue_db_file_DYN ){
-		datum key_data, data_data;
-		plp_block_mask oblock;
-		int n, write_db = 0;
-
-		if( gdbm_file == 0 ){
-			if( Open_gdbm( 0 ) ){
-				Scan_queue( 0, 0, 0, 0, 0, 0, 1, 1 );
-			}
-		} else {
-			write_db = 1;
-		}
-		if( gdbm_file == 0 ){
-			Errorcode = JABORT;
-			unlink( Queue_db_file_DYN );
-			FATAL(LOG_ERR)"Set_hold_file: DB '%s' not updated '%s'",
-				Queue_db_file_DYN, gdbm_strerror(gdbm_errno) );
-		}
-		/* set the file information */
-		memset(&key_data,0,sizeof(key_data));
-		memset(&data_data,0,sizeof(data_data));
-		key_data.dptr = hf_name;
-		key_data.dsize = strlen( hf_name ) + 1;
-		data_data.dptr = outstr;
-		data_data.dsize = strlen( outstr ) + 1;
-		plp_set_signal_mask( &oblock, 0 );
-		n = gdbm_store( gdbm_file, key_data, data_data, GDBM_REPLACE );
-		plp_set_signal_mask( &oblock, 0 );
-		if( n ){
-			/*
-			 * we can remove the database file and force it to be
-			 * rebuilt
-			 */
-			Close_gdbm();
-			unlink( Queue_db_file_DYN );
-			Errorcode = JABORT;
-			FATAL(LOG_ERR)"Set_hold_file: DB '%s' not updated '%s'",
-				Queue_db_file_DYN, gdbm_strerror(gdbm_errno) );
-		}
-		if( !write_db ){
-			Close_gdbm();
-		}
-	}
-#endif
 
 	if( Lpq_status_file_DYN ){
 		unlink(Lpq_status_file_DYN );
@@ -846,89 +738,13 @@ void Get_hold_file( struct job *job, char *hf_name )
 
 	if( ISNULL(hf_name) ) return;
 
-#ifdef HAVE_GDBM_H
-	if( Queue_db_file_DYN ) {
-		datum key_data, data_data;
-		int write_db = 0;
-
-		if( gdbm_file == 0 ){
-			if( Open_gdbm( 0 ) ){
-				Scan_queue( 0, 0, 0, 0, 0, 0, 1, 1 );
-			}
-		} else {
-			write_db = 1;
-		}
-		if( gdbm_file == 0 ){
-			Errorcode = JABORT;
-			unlink( Queue_db_file_DYN );
-			FATAL(LOG_ERR)"Get_hold_file: open of DB file %s failed - %s",
-				Queue_db_file_DYN, gdbm_strerror(gdbm_errno) );
-		}
-		memset(&key_data,0,sizeof(key_data));
-		memset(&data_data,0,sizeof(data_data));
-
-		DEBUG1("Get_hold_file: getting gdbm entry '%s'", hf_name );
-		/* set the file information */
-		key_data.dptr = hf_name;
-		key_data.dsize = strlen( hf_name ) + 1;
-		data_data = gdbm_fetch( gdbm_file, key_data );
-		if( data_data.dptr ){
-			DEBUG1("Get_hold_file: db has '%s'", hf_name );
-			Split( &job->info, data_data.dptr,
-				Line_ends, 1, Value_sep,1,1,1);
-			free( data_data.dptr); data_data.dptr = 0;
-			status = 1;
-		}
-		if( !write_db ){
-			Close_gdbm();
-		}
-	}
-#endif
 	if( !status ) {
 		Get_file_image_and_split( hf_name, 0, 0,
 			&job->info, Line_ends, 1, Value_sep,1,1,1,0);
 	}
-}
-
-/*
- * remove a db entry for the job
- */
-
-void Remove_from_db( char *openname )
-{
-#ifdef HAVE_GDBM_H
-	if( Queue_db_file_DYN ){
-		datum key_data;
-		plp_block_mask oblock;
-		int write_db = 0;
-
-		DEBUG1("Remove_from_db: '%s' in DB file '%s'",
-			openname, Queue_db_file_DYN );
-		if( gdbm_file == 0 ){
-			if( Open_gdbm( 0 ) ){
-				Scan_queue( 0, 0, 0, 0, 0, 0, 1, 1 );
-			}
-		} else {
-			write_db = 1;
-		}
-		if( gdbm_file == 0 ){
-			Errorcode = JABORT;
-			unlink( Queue_db_file_DYN );
-			FATAL(LOG_ERR)"Remove_from_db: open of DB file %s failed - %s",
-				Queue_db_file_DYN, gdbm_strerror(gdbm_errno) );
-		}
-		memset(&key_data,0,sizeof(key_data));
-		key_data.dptr = openname;
-		key_data.dsize = strlen( openname ) + 1;
-		/* we now delete the record */
-		plp_block_all_signals( &oblock );
-		gdbm_delete( gdbm_file, key_data );
-		plp_set_signal_mask( &oblock, 0 );
-		if( !write_db ){
-			Close_gdbm();
-		}
+	if( !Find_str_value(&job->info,HF_NAME,Value_sep) ){
+		Set_str_value(&job->info,HF_NAME,hf_name);
 	}
-#endif
 }
 
 /*
@@ -1006,55 +822,55 @@ void Set_spool_control( struct line_list *perm_check, const char *file,
 	if(t) free(t); t = 0;
 }
 
-char *intval( const char *key, struct line_list *list, char *cmpstr )
+void intval( const char *key, struct line_list *list, struct job *job )
 {
-	char buffer[SMALLBUFFER];
 	int i = Find_flag_value(list,key,Value_sep);
-	SNPRINTF(buffer,sizeof(buffer))"%s.0x%08x",key,i&0xffffffff);
-	DEBUG5("intval: '%s'", buffer );
-	return( safeextend3(cmpstr,"|",buffer,__FILE__,__LINE__) );
+	int len = safestrlen(job->sort_key);
+	SNPRINTF(job->sort_key+len,sizeof(job->sort_key)-len)
+    "|%s.0x%08x",key,i&0xffffffff);
+	DEBUG5("intval: '%s'", job->sort_key );
 }
 
-char * revintval( const char *key, struct line_list *list, char *cmpstr )
+void revintval( const char *key, struct line_list *list, struct job *job )
 {
-	char buffer[SMALLBUFFER];
 	int i = Find_flag_value(list,key,Value_sep);
-	SNPRINTF(buffer,sizeof(buffer))"%s.0x%08x",key,(~i)&0xffffffff);
-	DEBUG5("intval: '%s'", buffer );
-	return( safeextend3(cmpstr,"|",buffer,__FILE__,__LINE__) );
+	int len = safestrlen(job->sort_key);
+	SNPRINTF(job->sort_key+len,sizeof(job->sort_key)-len)
+	"|%s.0x%08x",key,(~i)&0xffffffff);
+	DEBUG5("revintval: '%s'", job->sort_key );
 }
 
-char * strzval( const char *key, struct line_list *list, char *cmpstr )
+void strzval( const char *key, struct line_list *list, struct job *job )
 {
-	char buffer[SMALLBUFFER];
 	char *s = Find_str_value(list,key,Value_sep);
-	SNPRINTF(buffer,sizeof(buffer))"%s.%d",key,s!=0);
-	DEBUG5("strzval: '%s'", buffer );
-	return( safeextend3(cmpstr,"|",buffer,__FILE__,__LINE__) );
+	int len = safestrlen(job->sort_key);
+	SNPRINTF(job->sort_key+len,sizeof(job->sort_key)-len)
+	"|%s.%d",key,s!=0);
+	DEBUG5("strzval: '%s'", job->sort_key );
 }
 
-char * strnzval( const char *key, struct line_list *list, char *cmpstr )
+void strnzval( const char *key, struct line_list *list, struct job *job )
 {
-	char buffer[SMALLBUFFER];
 	char *s = Find_str_value(list,key,Value_sep);
-	SNPRINTF(buffer,sizeof(buffer))"%s.%d",key,(s==0 || *s == 0));
-	DEBUG5("strzval: '%s'", buffer );
-	return( safeextend3(cmpstr,"|",buffer,__FILE__,__LINE__) );
+	int len = safestrlen(job->sort_key);
+	SNPRINTF(job->sort_key+len,sizeof(job->sort_key)-len)
+	"|%s.%d",key,(s==0 || *s == 0));
+	DEBUG5("strnzval: '%s'", job->sort_key );
 }
 
-char * strval( const char *key, struct line_list *list, char *cmpstr,
+void strval( const char *key, struct line_list *list, struct job *job,
 	int reverse )
 {
-	char buffer[SMALLBUFFER];
 	char *s = Find_str_value(list,key,Value_sep);
+	int len = safestrlen(job->sort_key);
 	int c = 0;
 
 	if(s) c = cval(s);
 	if( reverse ) c = -c;
 	c = 0xFF & (-c);
-	SNPRINTF(buffer,sizeof(buffer))"%s.%02x",key,c);
-	DEBUG5("strval: '%s'", buffer );
-	return( safeextend3(cmpstr,"|",buffer,__FILE__,__LINE__) );
+	SNPRINTF(job->sort_key+len,sizeof(job->sort_key)-len)
+	"|%s.%02x",key,c);
+	DEBUG5("strval: '%s'", job->sort_key );
 }
 
 
@@ -1062,45 +878,42 @@ char * strval( const char *key, struct line_list *list, char *cmpstr,
  * Make_sort_key
  *   Make a sort key from the image information
  */
-char *Make_sort_key( struct job *job )
+void Make_sort_key( struct job *job )
 {
-	char *cmpstr = 0;
+	job->sort_key[0] = 0;
 	if( Order_routine_DYN ){
 #if defined(ORDER_ROUTINE)
 		extern char *ORDER_ROUTINE( struct job *job );
-		cmpstr = ORDER_ROUTINE( &job );
+		ORDER_ROUTINE( &job );
 #else
 		Errorcode = JABORT;
-		FATAL(LOG_ERR)"Scan_queue: order_routine requested and ORDER_ROUTINE undefined");
+		FATAL(LOG_ERR)"Make_sort_key: order_routine requested and ORDER_ROUTINE undefined");
 #endif
 	} else {
 		/* first key is DONE_TIME - done jobs come last */
-		cmpstr = intval(DONE_TIME,&job->info,cmpstr);
+		intval(DONE_TIME,&job->info,job);
+		intval(INCOMING_TIME,&job->info,job);
 		/* next key is REMOVE_TIME - removed jobs come before last */
-		cmpstr = intval(REMOVE_TIME,&job->info,cmpstr);
+		intval(REMOVE_TIME,&job->info,job);
 		/* next key is ERROR - error jobs jobs come before removed */
-		cmpstr = strzval(ERROR,&job->info,cmpstr);
+		intval(ERROR_TIME,&job->info,job);
 		/* next key is HOLD - before the error jobs  */
-		cmpstr = intval(HOLD_CLASS,&job->info,cmpstr);
-		cmpstr = intval(HOLD_TIME,&job->info,cmpstr);
+		intval(HOLD_CLASS,&job->info,job);
+		intval(HOLD_TIME,&job->info,job);
 		/* next key is MOVE - before the held jobs  */
-		cmpstr = strnzval(MOVE,&job->info,cmpstr);
+		strnzval(MOVE,&job->info,job);
 		/* now by priority */
 		if( Ignore_requested_user_priority_DYN == 0 ){
-			cmpstr = strval(PRIORITY,&job->info,cmpstr,Reverse_priority_order_DYN);
+			strval(PRIORITY,&job->info,job,Reverse_priority_order_DYN);
 		}
 		/* now we do TOPQ */
-		cmpstr = revintval(PRIORITY_TIME,&job->info,cmpstr);
+		revintval(PRIORITY_TIME,&job->info,job);
 		/* now we do FirstIn, FirstOut */
-		cmpstr = intval(JOB_TIME,&job->info,cmpstr);
-		cmpstr = intval(JOB_TIME_USEC,&job->info,cmpstr);
+		intval(JOB_TIME,&job->info,job);
+		intval(JOB_TIME_USEC,&job->info,job);
 		/* now we do by job number if two at same time (very unlikely) */
-		cmpstr = intval(NUMBER,&job->info,cmpstr);
+		intval(NUMBER,&job->info,job);
 	}
-
-	DEBUG4("Make_sort_key: cmpstr '%s'", cmpstr );
-
-	return(cmpstr);
 }
 
 /*
@@ -1214,15 +1027,17 @@ int Read_pid( int fd, char *str, int len )
  * Write_pid( int fd )
  *   - Write the pid to a file
  **************************************************************************/
-void Write_pid( int fd, int pid, char *str )
+int Write_pid( int fd, int pid, char *str )
 {
 	char line[LINEBUFFER];
 
-	if( ftruncate( fd, 0 ) ){
-		LOGERR_DIE(LOG_ERR) "Write_pid: ftruncate failed" );
-	}
 	if( lseek( fd, 0, SEEK_SET ) == -1 ){
-		LOGERR_DIE(LOG_ERR) "Write_pid: lseek failed" );
+		LOGERR(LOG_ERR) "Write_pid: lseek failed" );
+		return -1;
+	}
+	if( ftruncate( fd, 0 ) ){
+		LOGERR(LOG_ERR) "Write_pid: ftruncate failed" );
+		return -1;
 	}
 
 	if( str == 0 ){
@@ -1232,8 +1047,10 @@ void Write_pid( int fd, int pid, char *str )
 	}
 	DEBUG3( "Write_pid: pid %d, str '%s'", pid, str );
 	if( Write_fd_str( fd, line ) < 0 ){
-		LOGERR_DIE(LOG_ERR) "Write_pid: write failed" );
+		LOGERR(LOG_ERR) "Write_pid: write failed" );
+		return -1;
 	}
+	return 0;
 }
 
 /***************************************************************************
@@ -1451,7 +1268,7 @@ int Check_format( int type, const char *name, struct job *job )
 
 char *Find_start(char *str, const char *key )
 {
-	int n = strlen(key);
+	int n = safestrlen(key);
 	while( (str = strstr(str,key)) && str[n] != '=' );
 	if( str ) str += (n+1);
 	return( str );
@@ -1586,12 +1403,12 @@ void Dump_job( char *title, struct job *job )
 
 
 void Job_printable( struct job *job, struct line_list *spool_control,
-	int *pprintable, int *pheld, int *pmove )
+	int *pprintable, int *pheld, int *pmove, int *perr, int *pdone )
 {
 	char *s;
 	char buffer[SMALLBUFFER];
 	char destbuffer[SMALLBUFFER];
-	int n, printable = 0, held = 0, move = 0, destination, destinations;
+	int n, printable = 0, held = 0, move = 0, error = 0, done = 0,destination, destinations;
 
 	if(DEBUGL4)Dump_job("Job_printable - job info",job);
 	if(DEBUGL4)Dump_line_list("Job_printable - spool control",spool_control);
@@ -1599,14 +1416,16 @@ void Job_printable( struct job *job, struct line_list *spool_control,
 	buffer[0] = 0;
 	if( job->info.count == 0 ){
 		SNPRINTF(buffer,sizeof(buffer)) "removed" );
-	} else if( Find_str_value(&job->info,ERROR,Value_sep) ){
+	} else if( (error = Find_flag_value(&job->info,ERROR_TIME,Value_sep)) ){
 		SNPRINTF(buffer,sizeof(buffer)) "error" );
 	} else if( Find_flag_value(&job->info,HOLD_TIME,Value_sep) ){
 		SNPRINTF(buffer,sizeof(buffer)) "hold" );
 		held = 1;
 	} else if( Find_flag_value(&job->info,REMOVE_TIME,Value_sep) ){
 		SNPRINTF(buffer,sizeof(buffer)) "remove" );
-	} else if( Find_flag_value(&job->info,DONE_TIME,Value_sep) ){
+	} else if( Find_flag_value(&job->info,INCOMING_TIME,Value_sep) ){
+		SNPRINTF(buffer,sizeof(buffer)) "incoming" );
+	} else if( (done = Find_flag_value(&job->info,DONE_TIME,Value_sep)) ){
 		SNPRINTF(buffer,sizeof(buffer)) "done" );
 	} else if( (n = Find_flag_value(&job->info,SERVER,Value_sep))
 		&& kill( n, 0 ) == 0 ){
@@ -1640,7 +1459,7 @@ void Job_printable( struct job *job, struct line_list *spool_control,
 			Get_destination(job,destination);
 			if(DEBUGL4)Dump_job("Job_destination_printable - job",job);
 			destbuffer[0] = 0;
-			if( Find_str_value(&job->destination,ERROR,Value_sep) ){
+			if( Find_flag_value(&job->destination,ERROR_TIME,Value_sep) ){
 				SNPRINTF(destbuffer,sizeof(destbuffer)) "error" );
 			} else if( Find_flag_value(&job->destination,HOLD_TIME,Value_sep) ){
 				SNPRINTF(destbuffer,sizeof(destbuffer)) "hold" );
@@ -1688,9 +1507,10 @@ void Job_printable( struct job *job, struct line_list *spool_control,
 	if( pprintable ) *pprintable = printable;
 	if( pheld ) *pheld = held;
 	if( pmove ) *pmove = move;
-	DEBUG3("Job_printable: printable %d, held %d, move '%d', status '%s'",
-		printable, held, move, buffer );
-
+	if( perr ) *perr = error;
+	if( pdone ) *pdone = done;
+	DEBUG3("Job_printable: printable %d, held %d, move '%d', error '%d', done '%d', status '%s'",
+		printable, held, move, error, done, buffer );
 }
 
 int Server_active( char *file )
@@ -1756,7 +1576,7 @@ int Get_destination( struct job *job, int n )
 	if( (s = Find_str_value(&job->info,buffer,Value_sep)) ){
 		s = safestrdup(s,__FILE__,__LINE__);
 		Unescape(s);
-		Split(&job->destination,s,Line_ends,1,Value_sep,1,1,1);
+		Split(&job->destination,s,Line_ends,1,Value_sep,1,1,1,0);
 		if(s) free( s ); s = 0;
 		result = 0;
 	}
@@ -1778,7 +1598,7 @@ int Get_destination_by_name( struct job *job, char *name )
 	if( name && (s = Find_str_value(&job->info,name,Value_sep)) ){
 		s = safestrdup(s,__FILE__,__LINE__);
 		Unescape(s);
-		Split(&job->destination,s,Line_ends,1,Value_sep,1,1,1);
+		Split(&job->destination,s,Line_ends,1,Value_sep,1,1,1,0);
 		if(s) free( s ); s = 0;
 		result = 0;
 	}
@@ -1797,19 +1617,19 @@ int Trim_status_file( int status_fd, char *file, int max, int min )
 	char *tempfile, *s;
 	int count;
 
-	status_fd = tempfd = status = -1;
+	tempfd = status = -1;
 
 	DEBUG1("Trim_status_file: file '%s' max %d, min %d", file, max, min);
 
 	/* if the file does not exist, do not create it */
-	if( file == 0 ) return( status_fd );
+	if( ISNULL(file) ) return( status_fd );
 	if( stat( file, &statb ) == 0 ){
 		DEBUG1("Trim_status_file: '%s' max %d, min %d, size %ld", file, max, min, 
 			(long)(statb.st_size) );
 		if( max > 0 && statb.st_size/1024 > max ){
 			status = Checkwrite( file, &statb,O_RDWR,0,0);
 			tempfd = Make_temp_fd(&tempfile);
-			if( min > max ){
+			if( min > max || min == 0 ){
 				min = max/4;
 			}
 			if( min == 0 ) min = 1;
@@ -1843,7 +1663,7 @@ int Trim_status_file( int status_fd, char *file, int max, int min )
 			close(status);
 		}
 		close( tempfd );
-		close( status_fd );
+		if( status_fd > 0 ) close( status_fd );
 		status_fd = Checkwrite( file, &statb,0,0,0);
 	}
 	return( status_fd );
@@ -1903,6 +1723,7 @@ char *Fix_datafile_info( struct job *job, char *number, char *suffix,
 	for( linecount = 0; linecount < job->datafiles.count; ++linecount ){
 		lp = (void *)job->datafiles.list[linecount];
 		transfername = Find_str_value(lp,TRANSFERNAME,Value_sep);
+		Set_str_value(lp,OTRANSFERNAME,transfername);
 		if( !(s = Find_casekey_str_value(&outfiles,transfername,Value_sep)) ){
 			/* we add the entry */
 			offset = count % 52;
@@ -1943,7 +1764,7 @@ char *Fix_datafile_info( struct job *job, char *number, char *suffix,
 				fmt[0] = *s;
 			}
 			if( xlate_format ){
-				int l = strlen(xlate_format);
+				int l = safestrlen(xlate_format);
 				for( i = 0; i+1 < l; i+= 2 ){
 					if( (xlate_format[i] == fmt[0])
 						|| (xlate_format[i] == '*') ){
@@ -2001,17 +1822,17 @@ int ordercomp(  const void *left, const void *right, const void *orderp)
 
 	/* blank lines always come last */
 	if( (wildcard = safestrchr( order, '*' )) ){
-		wildcard = order + strlen(order);
+		wildcard = order + safestrlen(order);
 	}
 	lpos = *((const char **)left);
 	if( lpos == 0 || *lpos == 0 ){
-		lpos = order+strlen(order);
+		lpos = order+safestrlen(order);
 	} else if( !(lpos = safestrchr( order, *lpos )) ){
 		lpos = wildcard;
 	}
 	rpos = *((const char **)right);
 	if( rpos == 0 || *rpos == 0 ){
-		rpos = order+strlen(order);
+		rpos = order+safestrlen(order);
 	} else if( !(rpos = safestrchr( order, *rpos )) ){
 		rpos = wildcard;
 	}
@@ -2099,7 +1920,6 @@ void Fix_control( struct job *job, char *filter, char *xlate_format )
 
 	if(DEBUGL3) Dump_job( "Fix_control: before fixing", job );
 
-
 	/* fix control file name */
 
 	s = safestrdup4("cf",pr,number,file_hostname,__FILE__,__LINE__);
@@ -2108,34 +1928,24 @@ void Fix_control( struct job *job, char *filter, char *xlate_format )
 
 	/* fix control file contents */
 
-	if( Use_identifier_DYN ){
-		if( job->destination.count == 0 ){
-			s = Find_str_value(&job->info,IDENTIFIER,Value_sep);
-			if( !s ){
-				Make_identifier( job );
-				s = Find_str_value(&job->info,IDENTIFIER,Value_sep);
-			}
+	s = Make_identifier( job );
+
+	if( job->destination.count == 0 ){
+		Set_str_value(&controlfile,IDENTIFIER,s);
+	} else {
+		s = Find_str_value(&job->destination,IDENTIFIER,Value_sep);
+		cccc = Find_flag_value(&job->destination,COPIES,Value_sep);
+		n = Find_flag_value(&job->destination,COPY_DONE,Value_sep);
+		if( cccc > 1 ){
+			SNPRINTF(buffer,sizeof(buffer))"C%d",n+1);
+			s = safestrdup2(s,buffer,__FILE__,__LINE__);
 			Set_str_value(&controlfile,IDENTIFIER,s);
+			if(s) free(s); s = 0;
 		} else {
-			s = Find_str_value(&job->destination,IDENTIFIER,Value_sep);
-			cccc = Find_flag_value(&job->destination,COPIES,Value_sep);
-			n = Find_flag_value(&job->destination,COPY_DONE,Value_sep);
-			if( cccc > 1 ){
-				SNPRINTF(buffer,sizeof(buffer))"C%d",n+1);
-				s = safestrdup2(s,buffer,__FILE__,__LINE__);
-				Set_str_value(&controlfile,IDENTIFIER,s);
-				if(s) free(s); s = 0;
-			} else {
-				Set_str_value(&controlfile,IDENTIFIER,s);
-			}
+			Set_str_value(&controlfile,IDENTIFIER,s);
 		}
 	}
-	if( (Is_server && Auth_forward_DYN == 0) ){
-		/* clobber the authentication information */
-		Set_str_value(&controlfile,AUTHINFO,0);
-	}
-	if( Use_date_DYN &&
-		!Find_str_value(&controlfile,DATE,Value_sep) ){
+	if( !Find_str_value(&controlfile,DATE,Value_sep) ){
 		Set_str_value(&controlfile,DATE, Time_str( 0, 0 ) );
 	}
 	if( (Use_queuename_DYN || Force_queuename_DYN) &&
@@ -2181,7 +1991,7 @@ void Fix_control( struct job *job, char *filter, char *xlate_format )
 		} else {
 			if( Backwards_compatible_DYN ){
 				for( j = 0; maxclen[j].c && cccc != maxclen[j].c ; ++j );
-				if( (len = maxclen[j].len) && strlen(s+1) > len ){
+				if( (len = maxclen[j].len) && safestrlen(s+1) > len ){
 					s[len+1] = 0;
 				}
 			}
@@ -2205,7 +2015,7 @@ void Fix_control( struct job *job, char *filter, char *xlate_format )
 	if(DEBUGL3) Dump_job( "Fix_control: after sorting", job );
 	for( i = 0; i < controlfile.count; ++i ){
 		s = controlfile.list[i];
-		memmove(s+1,s+2,strlen(s+2)+1);
+		memmove(s+1,s+2,safestrlen(s+2)+1);
 	}
 	s = Join_line_list(&controlfile,"\n");
 	DEBUG3( "Fix_control: control info '%s'", s );
@@ -2266,7 +2076,7 @@ void Fix_control( struct job *job, char *filter, char *xlate_format )
  *
  ************************************************************************/
 
-int Create_control( struct job *job, char *error, int errlen, char *auth_id,
+int Create_control( struct job *job, char *error, int errlen,
 	char *xlate_format )
 {
 	char *s, *t, *file_hostname, *number, *priority, *datalines, *openname;
@@ -2278,13 +2088,25 @@ int Create_control( struct job *job, char *error, int errlen, char *auth_id,
 	if(DEBUGL3) Dump_job( "Create_control: before fixing", job );
 
 	/* deal with authentication */
-	if( auth_id ){
-		Set_str_value(&job->info,AUTHINFO,auth_id);
-	}
+
+	Make_identifier( job );
+
 	if( !(s = Find_str_value(&job->info,FROMHOST,Value_sep)) || Is_clean_name(s) ){
 		Set_str_value(&job->info,FROMHOST,FQDNRemote_FQDN);
 		s = Find_str_value(&job->info,FROMHOST,Value_sep);
 	}
+	if( Force_IPADDR_hostname_DYN ){
+		char buffer[SMALLBUFFER];
+		const char *const_s;
+		int family;
+		/* We will need to create a dummy record. - no host */
+		family = RemoteHost_IP.h_addrtype;
+		const_s = (char *)inet_ntop( family, RemoteHost_IP.h_addr_list.list[0],
+			buffer, sizeof(buffer) );
+		DEBUG1("Create_control: remotehost '%s'", const_s );
+		Set_str_value(&job->info,FROMHOST,const_s);
+		s = Find_str_value(&job->info,FROMHOST,Value_sep);
+	}
 	if( Force_FQDN_hostname_DYN && !safestrchr(s,'.')
 		&& (t = safestrchr(FQDNRemote_FQDN,'.')) ){
 		s = safestrdup2(s, t, __FILE__,__LINE__ );
@@ -2292,12 +2114,9 @@ int Create_control( struct job *job, char *error, int errlen, char *auth_id,
 		if( s ) free(s); s = 0;
 		s = Find_str_value(&job->info,FROMHOST,Value_sep);
 	}
-	if( Use_identifier_DYN &&
-		!Find_str_value(&job->info,IDENTIFIER,Value_sep) ){
-		Make_identifier( job );
-	}
 
-	if( Use_date_DYN && Find_str_value(&job->info,DATE,Value_sep) ){
+
+	if( !Find_str_value(&job->info,DATE,Value_sep) ){
 		s = Time_str(0,0);
 		Set_str_value(&job->info,DATE,s);
 	}
@@ -2419,7 +2238,7 @@ void Put_buf_len( const char *s, int cnt, char **buf, int *max, int *len )
 
 void Put_buf_str( const char *s, char **buf, int *max, int *len )
 {
-	if( s && *s ) Put_buf_len( s, strlen(s), buf, max, len );
+	if( s && *s ) Put_buf_len( s, safestrlen(s), buf, max, len );
 }
 
 void Free_buf(char **buf, int *max, int *len)
@@ -2428,76 +2247,3 @@ void Free_buf(char **buf, int *max, int *len)
 	*len = 0;
 	*max = 0;
 }
-
-int Open_gdbm( int db_create )
-{
-#ifdef HAVE_GDBM_H
-	struct stat statb;
-	int flags = GDBM_SYNC|GDBM_NOLOCK|GDBM_WRITER;
-
-	if( gdbm_file ){
-		Close_gdbm();
-	}
-	if( Queue_db_file_DYN ){
-		gdbm_lock = Checkwrite( Queue_db_file_DYN, &statb,O_RDWR,1,0);
-		DEBUG1("Open_gdbm: gdbm_lock fd %d", (int)gdbm_lock );
-		if( gdbm_lock == -1 ){
-			Errorcode = JABORT;
-			LOGERR_DIE(LOG_ERR)"Open_gdbm: cannot open '%s'",
-				Queue_db_file_DYN );
-		}
-		DEBUG1("Open_gdbm: locking %d", (int)gdbm_lock );
-		while(1){
-			errno  = 0;
-			if( Do_lock( gdbm_lock, 1 ) < 0 ){
-				if( errno == EINTR ){
-					LOGERR(LOG_ERR)"Open_gdbm: cannot lock '%s'",
-						Queue_db_file_DYN );
-					plp_usleep(1000);
-					continue;
-				} else {
-					Errorcode = JABORT;
-					LOGERR_DIE(LOG_ERR)"Open_gdbm: cannot lock '%s'",
-						Queue_db_file_DYN );
-				}
-			} else {
-				break;
-			}
-		}
-		if( fstat( gdbm_lock, &statb ) ){
-			Errorcode = JABORT;
-			LOGERR_DIE(LOG_ERR)"Open_gdbm: cannot stat '%s'",
-				Queue_db_file_DYN );
-		}
-		if( statb.st_size == 0 ){
-			db_create = 1;
-		}
-		if( db_create ) flags |= GDBM_NEWDB;
-		gdbm_file = gdbm_open( Queue_db_file_DYN, 0,
-			flags, Spool_file_perms_DYN, NULL );
-		if( gdbm_file == 0 ){
-			Errorcode = JABORT;
-			FATAL(LOG_ERR)"Open_dgbm: open of DB file '%s' failed, %s",
-				Queue_db_file_DYN, gdbm_strerror(gdbm_errno) );
-		}
-		DEBUG1("Open_gdbm: gdbm_file 0x%x, gdbm file fd %d, lock fd %d",
-			Cast_ptr_to_int(gdbm_file), gdbm_file?gdbm_fdesc(gdbm_file):-1,  gdbm_lock );
-	}
-#endif
-	return( 1 );
-}
-
-
-void Close_gdbm(void)
-{
-#ifdef HAVE_GDBM_H
-	DEBUG1("Close_gdbm: gdbm_file 0x%x, gdbm file fd %d, lock fd %d",
-		Cast_ptr_to_int(gdbm_file), gdbm_file?gdbm_fdesc(gdbm_file):-1,  gdbm_lock );
-	if( gdbm_file ){
-		gdbm_close( gdbm_file );
-		close( gdbm_lock );
-	}
-	gdbm_lock = -1;
-	gdbm_file = 0;
-#endif
-}
