@@ -1,14 +1,14 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: linksupport.c,v 5.20 2000/12/25 01:51:07 papowell Exp papowell $";
+"$Id: linksupport.c,v 1.30 2002/05/06 01:06:40 papowell Exp $";
 
 
 /***************************************************************************
@@ -21,7 +21,7 @@
  *
  * int Link_open(char *host, int port, int int timeout )
  *    opens a link to the remote host
- *    if timeout == 0, wait indefinately
+ *    if timeout == 0, wait indefinitely
  *    returns socket fd (>= 0); LINK errorcode (negative) if error
  *
  * int Link_listen()
@@ -37,7 +37,7 @@
  *    sends 'ch'line'lf' to the remote host
  *    if write/read does not complete within timeout seconds,
  *      terminate action with error.
- *    if timeout == 0, wait indefinately
+ *    if timeout == 0, wait indefinitely
  *    if timeout > 0, i.e.- local, set signal handler
  *    if ch != 0, send ch at start of line
  *    if lf != 0, send LF at end of line
@@ -48,7 +48,7 @@
  *	char *src, int fd, double count)
  *    copies count bytes from fd to the socket
  *    do a timeout on both reading from fd and writing to socket;
- *    if timeout == 0, wait indefinately
+ *    if timeout == 0, wait indefinitely
  *      returns 0 if successful, LINK errorcode if failure
  *
  * int Link_line_read(char *host, int *socket, int timeout,
@@ -59,7 +59,7 @@
  *      updated with actual value read (less 1 for '\n' )
  *    if read does not complete within timeout seconds,
  *      terminate action with error.
- *    if timeout == 0, wait indefinately
+ *    if timeout == 0, wait indefinitely
  *    if *count read and not '\n',  then error set; last character
  *       will be discarded.
  *    returns 0 if '\n' read and read <= *count characters
@@ -74,7 +74,7 @@
  *      updated with actual value read
  *    if read does not complete within timeout seconds,
  *      terminate action with error.
- *    if timeout == 0, wait indefinately
+ *    if timeout == 0, wait indefinitely
  *    returns 0 *count not read
  *            LINK errorcode otherwise
  *
@@ -87,7 +87,7 @@
  *       returns value in *ack
  *    if read does not complete within timeout seconds,
  *      terminate action with error.
- *    if timeout == 0, wait indefinately
+ *    if timeout == 0, wait indefinitely
  *    returns 0 *count not read
  *
  ***************************************************************************/
@@ -217,14 +217,13 @@ int connect_timeout( int timeout,
 }
 
 int getconnection ( char *hostname, char *dest_port,
-	int timeout, int connection_type, struct sockaddr *bindto )
+	int timeout, int connection_type, struct sockaddr *bindto, char *unix_socket_path )
 {
 	int sock;	         /* socket */
 	int i, err;            /* ACME Generic Integers */
 	struct sockaddr_in dest_sin;     /* inet socket address */
 	struct sockaddr_in src_sin;     /* inet socket address */
 	int maxportno, minportno;
-	int port = 0;			/* max and minimum port numbers */
 	int euid;
 	int status = -1;			/* status of operation */
 	plp_block_mask oblock;
@@ -235,6 +234,7 @@ int getconnection ( char *hostname, char *dest_port,
 	char *use_host;
 	int address_count = 0;
 	int incoming_port;
+	static int last_port_used;
 
 	/*
 	 * find the address
@@ -253,7 +253,7 @@ int getconnection ( char *hostname, char *dest_port,
 		DEBUGF(DNW1)("getconnection: fqdn found %s, h_addr_list count %d",
 			LookupHost_IP.fqdn, LookupHost_IP.h_addr_list.count );
 		dest_sin.sin_family = LookupHost_IP.h_addrtype;
-		if( LookupHost_IP.h_length > sizeof( dest_sin.sin_addr ) ){
+		if( LookupHost_IP.h_length > (int)sizeof( dest_sin.sin_addr ) ){
 			FATAL(LOG_ALERT) "getconnection: addresslength outsize value");
 		}
 		memcpy( &dest_sin.sin_addr,
@@ -263,13 +263,78 @@ int getconnection ( char *hostname, char *dest_port,
 		DEBUGF(DNW2)("getconnection: cannot get address for '%s'", hostname );
 		return( LINK_OPEN_FAIL );
 	}
+
+	/* UNIX socket connection for localhost support */
+	DEBUGF(DNW1)("getconnection: unix_socket_path %s", unix_socket_path );
+	/* check to see if the flag is set and the destination
+		is one of the localhost addresses */
+	if( !ISNULL(unix_socket_path) && safestrcasecmp( "off", unix_socket_path ) &&
+			( !Same_host(&LookupHost_IP,&Host_IP)
+			|| !Same_host(&LookupHost_IP,&Localhost_IP) ) ){
+		/* taken from Unix Network Programming, Volume 1, 2nd Edition
+		 * by W. Richard Stevens.  With great thanks to his memory
+		 * and his amazingly detailed reference monographs.
+		 */
+		struct sockaddr_un dest_un;     /* unix socket address */
+		bzero( (char *)&dest_un, sizeof(dest_un) );
+
+		DEBUGF(DNW1)("getconnection: using unix socket");
+		plp_block_all_signals( &oblock );
+		if( UID_root ) (void)To_euid_root();
+		safestrncpy( dest_un.sun_path, unix_socket_path );
+#ifdef AF_LOCAL
+		dest_un.sun_family = AF_LOCAL;
+#else
+		dest_un.sun_family = AF_UNIX;
+#endif
+		sock = socket(dest_un.sun_family, SOCK_STREAM, 0);
+		err = errno;
+		if( UID_root ) (void)To_euid( euid );
+		plp_set_signal_mask( &oblock, 0 );
+		Max_open(sock);
+		if( sock < 0 ){
+			errno = err;
+			LOGERR_DIE(LOG_DEBUG) "getconnection: UNIX domain socket call failed");
+		}
+		DEBUGF(DNW2) ("getconnection: unix domain socket %d", sock);
+		/*
+		 * set up timeout and then make connect call
+		 */
+		errno = 0;
+		status = -1;
+		Alarm_timed_out = 0;
+		use_host = Unix_socket_path_DYN;
+		DEBUGF(DNW2)("getconnection: trying connect to UNIX domain socket '%s', timeout %d",
+			use_host, timeout );
+		status = connect_timeout(timeout,sock,
+			(struct sockaddr *) &dest_un, sizeof(dest_un));
+		err = errno;
+
+		DEBUGF(DNW2)(
+			"getconnection: connect sock %d, status %d, err '%s', timedout %d",
+			sock, status, Errormsg(err), Alarm_timed_out );
+		if( status < 0 || Alarm_timed_out ){
+			(void) close (sock);
+			sock = LINK_OPEN_FAIL;
+			if( Alarm_timed_out ) {
+				DEBUGF(DNW1)("getconnection: connection to '%s' timed out",
+					use_host);
+				err = errno = ETIMEDOUT;
+			} else {
+				DEBUGF(DNW1)("getconnection: connection to '%s' failed '%s'",
+					use_host, Errormsg(err) );
+			}
+		}
+		errno = err;
+		return (sock);
+	}
+	if( ISNULL(dest_port) ) dest_port = Lpd_port_DYN;
 	dest_sin.sin_port = Link_dest_port_num(dest_port);
 	if( dest_sin.sin_port == 0 ){
-		LOGERR(LOG_INFO)
-		"getconnection: bad port number for LPD connection!\n"
-		"check 'lpd-port' in configuration file, or the\n"
-		"/etc/services file for a missing 'printer 515/tcp' entry" );
-		return( LINK_OPEN_FAIL );
+		LOGMSG(LOG_INFO)
+		"getconnection: using illegal port '%s' for connection to '%s'!\n",
+			dest_port, hostname );
+		return( JABORT );
 	}
 
 	/* handle multi-homed hosts with bad ideas about
@@ -317,13 +382,25 @@ int getconnection ( char *hostname, char *dest_port,
 		}
 	}
 	range = maxportno - minportno;
-	if( minportno && range ){
-		/* ugly way to get a not so random number */ 
-		port = getpid()^time((void *)0);
-		if(port<0) port = -port;
-		port = port % range;
-	} else {
-		port = 0;
+
+	port_count = 0;			/* numbers of ports tried */
+	port_number = 0;
+	connect_count = 0;		/* number of connections tried */
+	port_number = 0;
+
+	if( minportno ){
+		port_number = minportno;
+		if( range ){
+			if( last_port_used && last_port_used >= minportno ){
+				port_number = ++last_port_used;
+			} else {
+				srand(getpid()); 
+				port_number = minportno + (abs(rand()>>8) % range);
+			}
+		}
+		if( port_number > maxportno){
+			port_number = minportno;
+		}
 	}
 
 	/* we now have a range of ports and a starting position
@@ -344,38 +421,32 @@ int getconnection ( char *hostname, char *dest_port,
 	 * very odd system implementations.  Note that you can
 	 * read and write to the socket as a user.
 	 */
-	DEBUGF(DNW2)("getconnection: RESTART ");
-	port_count = 0;			/* numbers of ports tried */
-	port_number = 0;
-	connect_count = 0;		/* number of connections tried */
-	if( minportno ){
-		port_number = minportno + port;
-	}
 	DEBUGF(DNW2)("getconnection: minportno %d, maxportno %d, range %d, port_number %d",
 		minportno, maxportno, range, port_number );
 
  again:
-	DEBUGF(DNW2)("getconnection: AGAIN port %d, count %d, connects %d",
-		port_number, port_count, connect_count );
+	DEBUGF(DNW1)("getconnection: AGAIN port %d, min %d, max %d, count %d, connects %d",
+		port_number, minportno, maxportno, port_count, connect_count );
 	DEBUGF(DNW2)("getconnection: protocol %d, connection_type %d",
 		AF_Protocol(), connection_type );
 	plp_block_all_signals( &oblock );
-	if( UID_root ) (void)To_root();
+	if( UID_root ) (void)To_euid_root();
 	sock = socket(AF_Protocol(), connection_type, 0);
-	Max_open(sock);
 	err = errno;
-	if( UID_root ) (void)To_uid( euid );
+	if( UID_root ) (void)To_euid( euid );
 	plp_set_signal_mask( &oblock, 0 );
+	Max_open(sock);
 	if( sock < 0 ){
 		errno = err;
 		LOGERR_DIE(LOG_DEBUG) "getconnection: socket call failed");
 	}
-	DEBUGF(DNW4) ("getconnection: socket %d", sock);
+	DEBUGF(DNW2) ("getconnection: socket %d", sock);
 
 	/* bind to an outgoing port if you need to */
 	if( minportno || bindto ){
 		incoming_port = ntohs(Link_dest_port_num(0));
 		do{
+			last_port_used = port_number;
 			status = -1;
 			if( bindto == 0 ){
 				src_sin.sin_family = AF_Protocol();
@@ -400,10 +471,10 @@ int getconnection ( char *hostname, char *dest_port,
 					able to reuse a port for up to 10 minutes */
 				/* we do the next without interrupts and as root */
 				plp_block_all_signals( &oblock );
-				if( UID_root ) (void)To_root();
+				if( UID_root ) (void)To_euid_root();
 				status = Link_setreuse( sock );
 				err = errno;
-				if( UID_root ) (void)To_uid( euid );
+				if( UID_root ) (void)To_euid( euid );
 				plp_set_signal_mask( &oblock, 0 );
 				DEBUGF(DNW2) ("getconnection: sock %d, reuse status %d",
 					sock, status );
@@ -414,10 +485,10 @@ int getconnection ( char *hostname, char *dest_port,
 			if( status >= 0 ){
 				/* we do the next without interrupts */
 				plp_block_all_signals( &oblock );
-				if( UID_root ) (void)To_root();
+				if( UID_root ) (void)To_euid_root();
 				status = bind(sock, (struct sockaddr *)&src_sin, sizeof(src_sin));
 				err = errno;
-				if( UID_root ) (void)To_uid( euid );
+				if( UID_root ) (void)To_euid( euid );
 				plp_set_signal_mask( &oblock, 0 );
 				DEBUGF(DNW2) ("getconnection: bind returns %d, sock %d, port %d, src '%s'",
 					status, sock, ntohs(src_sin.sin_port), inet_ntoa(src_sin.sin_addr) );
@@ -425,10 +496,10 @@ int getconnection ( char *hostname, char *dest_port,
 			if( status >= 0 && Keepalive_DYN ){
 				/* we do the next without interrupts */
 				plp_block_all_signals( &oblock );
-				if( UID_root ) (void)To_root();
+				if( UID_root ) (void)To_euid_root();
 				status = Link_setkeepalive( sock );
 				err = errno;
-				if( UID_root ) (void)To_uid( euid );
+				if( UID_root ) (void)To_euid( euid );
 				plp_set_signal_mask( &oblock, 0 );
 				if( status < 0 ){
 					LOGERR(LOG_ERR) "getconnection: set SO_KEEPALIVE failed" );
@@ -540,6 +611,7 @@ void Set_linger( int sock, int n )
 	DEBUGF(DNW2) ("Set_linger: NO SO_LINGER, socket %d, value %d", sock, n);
 #endif
 }
+
 /*
  * int Link_listen(port)
  *  1. opens a socket on the current host
@@ -547,7 +619,7 @@ void Set_linger( int sock, int n )
  *  3. does a bind to port determined by Link_dest_port_num();
  */
 
-int Link_listen( void )
+int Link_listen( char *port_name )
 {
 	int sock;                   /* socket */
 	int status;                   /* socket */
@@ -564,50 +636,52 @@ int Link_listen( void )
 	/*
 	 * Get the destination host address and remote port number to connect to.
 	 */
-	if( Lpd_port_DYN == 0 ){
-		Errorcode = JABORT;
-		FATAL(LOG_ERR) "Link_listen: LOGIC ERROR- no Lpd_port_DYN value");
-	}
 	sinaddr.sin_family = AF_Protocol();
 	sinaddr.sin_addr.s_addr = INADDR_ANY;
-	sinaddr.sin_port = Link_dest_port_num(Lpd_port_DYN);
-	port = ntohs( sinaddr.sin_port );
-	if( (s = safestrchr( Lpd_port_DYN, '%')) ){
+	if( (s = safestrchr( port_name, '%')) ){
 		*s = 0;
-		if( Find_fqdn( &LookupHost_IP, Lpd_port_DYN ) ){
+		if( Find_fqdn( &LookupHost_IP, port_name ) ){
 			/*
 			 * Get the destination host address and remote port number to connect to.
 			 */
 			DEBUGF(DNW1)("Link_listen: fqdn found %s, h_addr_list count %d",
 				LookupHost_IP.fqdn, LookupHost_IP.h_addr_list.count );
 			sinaddr.sin_family = LookupHost_IP.h_addrtype;
-			if( LookupHost_IP.h_length > sizeof( sinaddr.sin_addr ) ){
+			if( LookupHost_IP.h_length > (int)sizeof( sinaddr.sin_addr ) ){
 				FATAL(LOG_ALERT) "getconnection: addresslength outsize value");
 			}
 			/* use the first address in the list */
 			memcpy( &sinaddr.sin_addr,
 				LookupHost_IP.h_addr_list.list[0],
 				LookupHost_IP.h_length );
-		} else if( inet_pton( AF_Protocol(), Lpd_port_DYN, &sinaddr.sin_addr ) != 1 ){
+		} else if( inet_pton( AF_Protocol(), port_name, &sinaddr.sin_addr ) != 1 ){
 			*s = '%';
 			Errorcode = JABORT;
 			FATAL(LOG_ERR) "Link_listen: bad lpd_port value, cannot resolve IP address '%s'",
-				Lpd_port_DYN );
+				port_name );
 		}
+		sinaddr.sin_port = Link_dest_port_num(s+1);
 		*s = '%';
+	} else if( port_name ){
+		sinaddr.sin_port = Link_dest_port_num(port_name);
 	}
+	port = ntohs( sinaddr.sin_port );
 	DEBUGF(DNW2)("Link_listen: bind to IP '%s' port %d",
 		inet_ntoa( sinaddr.sin_addr ), ntohs( sinaddr.sin_port ) );
+	if( port == 0 ){
+		errno = 0;
+		return( 0 );
+	}
 
 	euid = geteuid();
-	if( UID_root ) (void)To_root();
+	if( UID_root ) (void)To_euid_root();
 	errno = 0;
 	status = (sock = socket (AF_Protocol(), SOCK_STREAM, 0)) < 0
 		|| Link_setreuse( sock ) < 0
 		|| (Keepalive_DYN && Link_setkeepalive( sock ) < 0)
 		|| bind(sock, (struct sockaddr *)&sinaddr, sizeof(sinaddr)) < 0;
 	err = errno;
-	if( UID_root ) (void)To_uid( euid );
+	if( UID_root ) (void)To_euid( euid );
 	if( status ){
 		DEBUGF(DNW4)("Link_listen: bind to lpd port %d failed '%s'",
 			port, Errormsg(err));
@@ -632,30 +706,101 @@ int Link_listen( void )
 	return (sock);
 }
 
-int Link_open(char *host, char *port, int timeout, struct sockaddr *bindto )
+int Unix_link_listen( char *unix_socket_path )
+{
+	int sock;                   /* socket */
+	int status;                 /* socket */
+	struct sockaddr_un sunaddr;     /* inet socket address */
+	int euid;					/* euid at time of call*/
+	int err;
+	int omask;
+
+	euid = geteuid();
+
+	DEBUGF(DNW2)("Unix_link_listen: path %s", unix_socket_path );
+	
+	/*
+	 * Zero out the sunaddr struct
+	 */
+	memset(&sunaddr, 0, sizeof (sunaddr));
+	/*
+	 * Get the destination host address and remote port number to connect to.
+	 */
+	DEBUGF(DNW1)("Unix_link_listen: using unix socket");
+	safestrncpy( sunaddr.sun_path, Unix_socket_path_DYN );
+#ifdef AF_LOCAL
+	sunaddr.sun_family = AF_LOCAL;
+#else
+	sunaddr.sun_family = AF_UNIX;
+#endif
+	if( UID_root ) (void)To_euid_root();
+	unlink( sunaddr.sun_path );
+	status = ((sock = socket (sunaddr.sun_family, SOCK_STREAM, 0)) < 0);
+	err = errno;
+	if( UID_root ) (void)To_euid( euid );
+	Max_open(sock);
+	if( sock < 0 ){
+		errno = err;
+		LOGERR_DIE(LOG_DEBUG) "Unix_link_listen: UNIX domain socket call failed");
+	}
+
+	omask = umask(0);
+	if( UID_root ) (void)To_euid_root();
+	status = bind(sock, (struct sockaddr *)&sunaddr, sizeof(sunaddr)) < 0;
+	err = errno;
+	if( UID_root ) (void)To_euid( euid );
+	umask(omask);
+	if( status ){
+		DEBUGF(DNW4)("Unix_link_listen: bind to unix port %s failed '%s'",
+			Unix_socket_path_DYN, Errormsg(err));
+		if( sock >= 0 ){
+			(void)close( sock );
+			sock = -1;
+		}
+		errno = err;
+		return( LINK_BIND_FAIL );
+	}
+	if( UID_root ) (void)To_euid_root();
+	status = listen(sock, 64 );	/* backlog of 10 is inadequate */
+	err = errno;
+	if( UID_root ) (void)To_euid( euid );
+	if( status ){
+		LOGERR_DIE(LOG_ERR) "Unix_link_listen: listen failed");
+		(void)close( sock );
+		sock = -1;
+		err = errno;
+		return( LINK_OPEN_FAIL );
+	}
+	DEBUGF(DNW4)("Unix_link_listen: socket %d", sock);
+	errno = err;
+	return (sock);
+}
+
+int Link_open(char *host, char *port, int timeout, struct sockaddr *bindto,
+	char *unix_socket_path )
 {
 	int sock;
 	DEBUGF(DNW4) ("Link_open: host '%s', port '%s', timeout %d",
 		host,port,timeout);
-	sock = Link_open_type( host, port, timeout, SOCK_STREAM, bindto );
+	sock = Link_open_type( host, port, timeout, SOCK_STREAM, bindto, unix_socket_path );
 	DEBUGF(DNW4) ("Link_open: socket %d", sock );
 	return(sock);
 }
 
 int Link_open_type(char *host, char *port, int timeout, int connection_type,
-	struct sockaddr *bindto )
+	struct sockaddr *bindto, char * unix_socket_path )
 {
 	int sock = -1;
 	DEBUGF(DNW4)(
 		"Link_open_type: host '%s', port '%s', timeout %d, type %d",
 		host,port, timeout, connection_type );
-	sock = getconnection( host, port, timeout, connection_type, bindto );
+	sock = getconnection( host, port, timeout, connection_type, bindto, unix_socket_path );
 	DEBUGF(DNW4) ("Link_open_type: socket %d", sock );
 	return( sock );
 }
 
 int Link_open_list( char *hostlist, char **result,
-	char *port, int timeout, struct sockaddr *bindto )
+	char *port, int timeout, struct sockaddr *bindto, char *unix_socket_path )
 {
 	int sock = -1, i, err = 0;
 	struct line_list list;
@@ -667,11 +812,11 @@ int Link_open_list( char *hostlist, char **result,
 		hostlist, port, timeout, Cast_ptr_to_long(bindto) );
 	}
 	if( result ) *result = 0;
-	Split(&list,hostlist,Host_sep,0,0,0,0,0);
+	Split(&list,hostlist,Host_sep,0,0,0,0,0,0);
 	err = errno = 0;
 	for( i = 0; sock < 0 && i < list.count; ++i ){
 		DEBUGF(DNW4) ("Link_open_list: host trying '%s'", list.list[i] );
-		sock = getconnection( list.list[i], port, timeout, SOCK_STREAM, bindto );
+		sock = getconnection( list.list[i], port, timeout, SOCK_STREAM, bindto, unix_socket_path );
 		err = errno;
 		DEBUGF(DNW4) ("Link_open_list: result host '%s' socket %d", list.list[i], sock );
 		if( sock >= 0 ){
@@ -710,7 +855,7 @@ void Link_close( int *sock )
  *    sends 'ch'str to the remote host
  *    if write/read does not complete within timeout seconds,
  *      terminate action with error.
- *    if timeout == 0, wait indefinately
+ *    if timeout == 0, wait indefinitely
  *    if ch != 0, send ch at start of line
  *    if lf != 0, send LF at end of line
  *    if ack != 0, wait for ack, and report it
@@ -829,7 +974,7 @@ int Link_send( char *host, int *sock, int timeout,
  *    copies count bytes from fd to the socket
  *    if write does not complete within timeout seconds,
  *      terminate action with error.
- *    if timeout == 0, wait indefinately
+ *    if timeout == 0, wait indefinitely
  *    if count < 0, will read until end of file
  *      returns 0 if successful, LINK errorcode if failure
  ***************************************************************************/
@@ -957,12 +1102,12 @@ int Link_dest_port_num( char *port )
  * int Link_line_read(char *host, int *sock, int timeout,
  *	  char *str, int *count )
  *    reads and copies characters from socket to str until '\n' read,
- *      '\n' NOT copied
+ *      '\n' NOT copied. \r\n -> \n
  *    *count points to maximum number of bytes to read;
  *      updated with actual value read (less 1 for '\n' )
  *    if read does not complete within timeout seconds,
  *      terminate action with error.
- *    if timeout == 0, wait indefinately
+ *    if timeout == 0, wait indefinitely
  *    if *count read and not '\n',  then error set; last character
  *       will be discarded.
  *    returns 0 if '\n' read at or before *count characters
@@ -975,7 +1120,7 @@ int Link_line_read(char *host, int *sock, int timeout,
 {
 	int i, len, max, err = 0;	/* ACME Integer, Inc. */
 	int status;				/* status of operation */
-	int lf;					/* lf found */
+	int cr, lf;					/* lf found */
 
 	len = i = status = 0;	/* shut up GCC */
 	max = *count;
@@ -992,12 +1137,18 @@ int Link_line_read(char *host, int *sock, int timeout,
 	/*
 	 * set up timeout and then do operation
 	 */
-	lf = len = 0;
+	cr = lf = len = 0;
 	errno = 0;
 	while( len < max-1
 		&&(i = Read_fd_len_timeout(timeout, *sock, &buf[len], 1 )) > 0
-		&& !Alarm_timed_out && !(lf = (buf[len] == '\n')) ){
-		len += i;
+		&& !Alarm_timed_out ){
+		if( (lf = (buf[len] == '\n')) ){
+			break;
+		} if( (cr = (buf[len] == '\r')) ){
+			cr = 1;
+		} else {
+			++len;
+		}
 	}
 	err = errno;
 	buf[len] = 0;
@@ -1038,7 +1189,7 @@ int Link_line_read(char *host, int *sock, int timeout,
  *      updated with actual value read
  *    if read does not complete within timeout seconds,
  *      terminate action with error.
- *    if timeout == 0, wait indefinately
+ *    if timeout == 0, wait indefinitely
  *    returns 0 *count not read
  *        LINK errorcode otherwise
  ***************************************************************************/
@@ -1104,7 +1255,7 @@ int Link_read(char *host, int *sock, int timeout,
  *       returns value in *ack
  *    if read does not complete within timeout seconds,
  *      terminate action with error.
- *    if timeout == 0, wait indefinately
+ *    if timeout == 0, wait indefinitely
  *    returns 0 *count not read
  *
  ***************************************************************************/
@@ -1300,7 +1451,7 @@ int AF_Protocol(void)
  * addrprt = destination
  ***************************************************************************/
 
-int inet_pton( int family, const char *strptr, void *addr )
+ int inet_pton( int family, const char *strptr, void *addr )
 {
 	if( family != AF_INET ){
 		FATAL(LOG_ERR) "inet_pton: bad family '%d'", family );
@@ -1309,10 +1460,10 @@ int inet_pton( int family, const char *strptr, void *addr )
 	return( inet_aton( strptr, addr ) );
 #else
 #if !defined(INADDR_NONE)
-#define INADDR_NONE (-1)
+#define INADDR_NONE 0xffffffff
 #endif
 	if( inet_addr( strptr ) != INADDR_NONE ){
-		((unsigned long *)addr)[0] = inet_addr( strptr );
+		((struct in_addr *)addr)->s_addr = inet_addr(strptr);
 		return(1);
 	}
 	return(0);
@@ -1367,6 +1518,16 @@ const char *inet_ntop_sockaddr( struct sockaddr *addr,
 	} else if( addr->sa_family == AF_INET6 ){
 		a = &((struct sockaddr_in6 *)addr)->sin6_addr;
 #endif
+	} else if( addr->sa_family == 0 
+#if defined(AF_LOCAL)
+		|| addr->sa_family == AF_LOCAL 
+#endif
+#if defined(AF_UNIX)
+		|| addr->sa_family == AF_UNIX 
+#endif
+		){
+		SNPRINTF (str, len) "%s", Unix_socket_path_DYN );
+		return( str );
 	} else {
 		FATAL(LOG_ERR) "inet_ntop_sockaddr: bad family '%d'",
 			addr->sa_family );
