@@ -1,14 +1,14 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: lpd_control.c,v 5.18 2000/12/28 01:32:59 papowell Exp papowell $";
+"$Id: lpd_control.c,v 1.30 2002/05/06 01:06:40 papowell Exp $";
 
 
 #include "lp.h"
@@ -22,8 +22,6 @@
 #include "globmatch.h"
 #include "permission.h"
 #include "gethostinfo.h"
-
-#include "lpd.h"
 #include "lpd_control.h"
 
 /**** ENDINCLUDE ****/
@@ -64,7 +62,7 @@ int Job_control( int *sock, char *input )
 	DEBUGF(DCTRL1)("Job_control: socket %d, doing '%s'", *sock, input );
 
 	/* check printername for characters, underscore, digits */
-	Split(&tokens,input,Whitespace,0,0,0,0,0);
+	Split(&tokens,input,Whitespace,0,0,0,0,0,0);
 	DEBUGFC(DCTRL2)Dump_line_list("Job_control - input", &tokens);
 
 	tokencount = tokens.count;
@@ -315,18 +313,17 @@ void Do_queue_control( char *user, int action, int *sock,
 
 	switch( action ){
 	case OP_LPQ:
-		if( Do_control_lpq( user, action, sock,
-			tokens, error, errorlen ) ){
+		if( Do_control_lpq( user, action,
+			tokens ) ){
 			goto error;
 		}
 		goto done;
 	case OP_PRINTCAP:
-		Do_control_printcap( user, action, sock,
-			tokens, error, errorlen );
+		Do_control_printcap( sock );
 		goto done;
 	case OP_STATUS:
-		if( Do_control_status( user, action, sock,
-			tokens, error, errorlen ) ){
+		if( Do_control_status( sock,
+			error, errorlen ) ){
 			goto error;
 		}
 		goto done;
@@ -364,7 +361,7 @@ void Do_queue_control( char *user, int action, int *sock,
 		Set_flag_value(&Spool_control,PRINTING_ABORTED, 0);
 		Set_flag_value(&Spool_control,PRINTING_DISABLED, 0);
 	case OP_HOLD:
-		if( Do_control_file( user, action, sock,
+		if( Do_control_file( action, sock,
 			tokens, error, errorlen, 0 ) ){
 			goto error;
 		}
@@ -376,44 +373,46 @@ void Do_queue_control( char *user, int action, int *sock,
 		Remove_line_list(tokens,0); /* 'msg' */
 		start = Join_line_list(tokens," ");
 		if( start ){
-			end = start+strlen(start)-1;
+			end = start+safestrlen(start)-1;
 			*end = 0;
 		}
 		DEBUGF(DCTRL1)("Do_queue_control: msg '%s'", start );
+		if( Lpq_status_file_DYN ) unlink(Lpq_status_file_DYN );
 		Set_str_value(&Spool_control,MSG,start);
 		break;
+
 	case OP_MOVE:
 		--tokens->count;
 		start = tokens->list[tokens->count];
-		status = Do_control_file( user, action, sock,
+		status = Do_control_file( action, sock,
 			tokens, error, errorlen, start );
 		++tokens->count;
 		if( status ) goto error;
 		break;
 
 	case OP_LPRM:
-		if( Do_control_lpq( user, action, sock,
-			tokens, error, errorlen ) ){
+		if( Do_control_lpq( user, action,
+			tokens ) ){
 			goto error;
 		}
 		break;
 		
 	case OP_REDIRECT:
-		if( Do_control_redirect( user, action, sock,
+		if( Do_control_redirect( sock,
 			tokens, error, errorlen ) ){
 			goto error;
 		}
 		break;
 
 	case OP_CLASS:
-		if( Do_control_class( user, action, sock,
+		if( Do_control_class( sock,
 			tokens, error, errorlen ) ){
 			goto error;
 		}
 		break;
 
 	case OP_DEBUG:
-		if( Do_control_debug( user, action, sock,
+		if( Do_control_debug( sock,
 			tokens, error, errorlen ) ){
 			goto error;
 		}
@@ -421,7 +420,21 @@ void Do_queue_control( char *user, int action, int *sock,
 
 	case OP_FLUSH:
 		if( Lpq_status_file_DYN ) unlink(Lpq_status_file_DYN );
-		if( Queue_db_file_DYN ) unlink(Queue_db_file_DYN );
+		{
+			char *file;
+			int fd = -1;
+			if( (file = Queue_status_file_DYN) &&
+				(fd = Checkwrite( file, &statb,O_RDWR,0,0)) > 0 ){
+				ftruncate( fd, 0);
+				close(fd);
+			}
+			if( (file = Status_file_DYN) &&
+				(fd = Checkwrite( file, &statb,O_RDWR,0,0)) > 0 ){
+				ftruncate( fd, 0);
+				close(fd);
+			}
+		}
+		signal_server = 0;
 		break;
 		
 	default:
@@ -454,11 +467,12 @@ void Do_queue_control( char *user, int action, int *sock,
 				kill( serverpid, SIGQUIT );
 			}
 			if( kill( serverpid, signal_server ) ){
-				serverpid = 0;
+				SNPRINTF(msg,sizeof(msg))_("server process PID %d exited\n"),
+					serverpid );
+			} else {
+				SNPRINTF(msg,sizeof(msg))_("kill server process PID %d with %s\n"),
+					serverpid, Sigstr(signal_server) );
 			}
-		} else {
-			SNPRINTF(msg,sizeof(msg))_("kill server PID %d with %s\n"),
-				serverpid, Sigstr(signal_server) );
 			Write_fd_str(*sock,msg);
 		}
 	}
@@ -499,8 +513,11 @@ void Do_queue_control( char *user, int action, int *sock,
 			if( serverpid == 0 || kill( serverpid, signal_server ) ){
 				serverpid = 0;
 			} else {
-				SNPRINTF(msg,sizeof(msg))_("kill load balance server PID %d with %s\n"),
-					serverpid, Sigstr(signal_server) );
+				SNPRINTF(msg,sizeof(msg))_(
+				"WARNING: the main load balance server may have exited before\n"
+				"it could be informed that there were new jobs.\n"
+                "Please use 'lpc start %s' to start the server\n"),
+					Server_queue_name_DYN );
 				Write_fd_str(*sock,msg);
 			}
 		}
@@ -541,7 +558,7 @@ void Do_queue_control( char *user, int action, int *sock,
 	SNPRINTF( error, sizeof(error)) "no permission");
  error:
 	DEBUGF(DCTRL2)("Do_queue_control: error msg '%s'", error );
-	if( (i = strlen(error)) ){
+	if( (i = safestrlen(error)) ){
 		SNPRINTF( msg, sizeof(msg)) "%s@%s: %s\n",
 			Printer_DYN, ShortHost_FQDN, error );
 		if( Write_fd_str( *sock, msg ) < 0 ) cleanup(0);
@@ -561,7 +578,7 @@ void Do_queue_control( char *user, int action, int *sock,
  * 3. update the hold file for the control file
  ***************************************************************************/
 
-int Do_control_file( char *user, int action, int *sock,
+int Do_control_file( int action, int *sock,
 	struct line_list *tokens, char *error, int errorlen, char *option )
 {
 	int i, permission, err;		/* ACME! Nothing but the best */
@@ -577,9 +594,8 @@ int Do_control_file( char *user, int action, int *sock,
 	Init_job(&job);
 	Free_line_list(&Sort_order);
 	if( Scan_queue( &Spool_control, &Sort_order,
-			0,0,0,0,0,1) ){
+			0,0,0,0,0,0) ){
 		err = errno;
-		Close_gdbm();
 		SNPRINTF(error, errorlen)
 			"Do_control_file: cannot read '%s' - '%s'",
 			Spool_dir_DYN, Errormsg(err) );
@@ -701,9 +717,10 @@ int Do_control_file( char *user, int action, int *sock,
 			Printer_DYN, identifier );
 		if( Write_fd_str( *sock, msg ) < 0 ) cleanup(0);
 		Set_str_value(&job.info,ERROR,0 );
+		Set_flag_value(&job.info,ERROR_TIME,0);
 		/* record the last update person */
 		Perm_check_to_list(&l, &Perm_check );
-		if( Set_hold_file(&job,&l) ){
+		if( Set_hold_file(&job,&l,0) ){
 			setmessage( &job, TRACE, "LPC failed" );
 			SNPRINTF( msg, sizeof(msg))
 				_("%s: cannot set hold file '%s'\n"),
@@ -715,7 +732,6 @@ int Do_control_file( char *user, int action, int *sock,
 			goto next_dest;
 		}
 	}
-	Close_gdbm();
 	Free_job(&job);
 	Free_line_list(&Sort_order);
 	Free_line_list(&l);
@@ -729,8 +745,8 @@ int Do_control_file( char *user, int action, int *sock,
  *  forward an OP_LPQ or OP_LPRM
  ***************************************************************************/
 
-int Do_control_lpq( char *user, int action, int *sock,
-	struct line_list *tokens, char *error, int errorlen )
+int Do_control_lpq( char *user, int action,
+	struct line_list *tokens )
 {
 	char msg[LINEBUFFER];			/* message field */
 	int i = 0;
@@ -770,8 +786,8 @@ int Do_control_lpq( char *user, int action, int *sock,
  *  report current status
  ***************************************************************************/
 
-int Do_control_status( char *user, int action, int *sock,
-	struct line_list *tokens, char *error, int errorlen )
+int Do_control_status( int *sock,
+	char *error, int errorlen )
 {
 	char msg[SMALLBUFFER];			/* message field */
 	char pr[LINEBUFFER];
@@ -784,13 +800,13 @@ int Do_control_status( char *user, int action, int *sock,
 	int serverpid, unspoolerpid;	/* server and unspooler */
 	int len;
 	char *s;
-	int printable, held, move;
+	int printable, held, move, err, done;
 
 	/* get the job files */
 	Free_line_list(&Spool_control);
 	Get_spool_control( Queue_control_file_DYN, &Spool_control );
 	if( Scan_queue( &Spool_control, &Sort_order, &printable,
-			&held, &move,1,0,0) ){
+			&held, &move,1,&err,&done) ){
 		SNPRINTF( error, errorlen)
 			"Do_control_status: cannot read '%s' - '%s'",
 			Spool_dir_DYN, Errormsg(errno) );
@@ -798,8 +814,8 @@ int Do_control_status( char *user, int action, int *sock,
 	}
 	Free_line_list(&Sort_order);
 
-	DEBUGF(DCTRL1)( "Do_control_status: printable %d, held %d, move",
-		printable, held, move );
+	DEBUGF(DCTRL1)( "Do_control_status: printable %d, held %d, move %d, err %d, done %d",
+		printable, held, move, error, done );
 
 	/* now check to see if there is a server and unspooler process active */
 	serverpid = Server_active( Printer_DYN );
@@ -812,19 +828,19 @@ int Do_control_status( char *user, int action, int *sock,
 		Report_server_as_DYN?Report_server_as_DYN:ShortHost_FQDN );
 	pr_status[0] = 0;
 	if( Hld_all(&Spool_control) ){
-		len = strlen(pr_status);
+		len = safestrlen(pr_status);
 		SNPRINTF( pr_status+len, sizeof(pr_status)-len) _(" holdall") );
 	}
 	if( (s = Clsses(&Spool_control)) ){
-		len = strlen(pr_status);
+		len = safestrlen(pr_status);
 		SNPRINTF( pr_status+len, sizeof(pr_status)-len) _(" class=%s"),s );
 	}
 	if( Auto_hold_DYN ){
-		len = strlen(pr_status);
+		len = safestrlen(pr_status);
 		SNPRINTF( pr_status+len, sizeof(pr_status)-len) _(" autohold") );
 	}
 	if( pr_status[0] ){
-		len = strlen(pr_status);
+		len = safestrlen(pr_status);
 		SNPRINTF( pr_status+len, sizeof(pr_status)-len) ")" );
 		pr_status[0] = '(';
 		
@@ -867,7 +883,7 @@ int Do_control_status( char *user, int action, int *sock,
  * 4. if option = printer@host, specify name
  ***************************************************************************/
 
-int Do_control_redirect( char *user, int action, int *sock,
+int Do_control_redirect( int *sock,
 	struct line_list *tokens, char *error, int errorlen )
 {
 	char *s;
@@ -920,7 +936,7 @@ int Do_control_redirect( char *user, int action, int *sock,
  * 4. if option = printer@host, specify name
  ***************************************************************************/
 
-int Do_control_class( char *user, int action, int *sock,
+int Do_control_class( int *sock,
 	struct line_list *tokens, char *error, int errorlen )
 {
 	char forward[LINEBUFFER];
@@ -976,7 +992,7 @@ int Do_control_class( char *user, int action, int *sock,
  * 4. if option = printer@host, specify name
  ***************************************************************************/
 
-int Do_control_debug( char *user, int action, int *sock,
+int Do_control_debug( int *sock,
 	struct line_list *tokens, char *error, int errorlen )
 {
 	char debugging[LINEBUFFER];
@@ -1032,8 +1048,7 @@ int Do_control_debug( char *user, int action, int *sock,
  * 3. if option = HUP, send signal
  ***************************************************************************/
 
-int Do_control_printcap( char *user, int action, int *sock,
-	struct line_list *tokens, char *error, int errorlen )
+int Do_control_printcap( int *sock )
 {
 	char *printcap = 0, *s, *t, *w;
 
