@@ -1,14 +1,14 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: lpd_rcvjob.c,v 5.22 2000/12/25 01:51:10 papowell Exp papowell $";
+"$Id: lpd_rcvjob.c,v 1.30 2002/05/06 01:06:41 papowell Exp $";
 
 
 #include "lp.h"
@@ -24,7 +24,7 @@
 #include "permission.h"
 #include "proctitle.h"
 
-#include "lpd.h"
+#include "lpd_remove.h"
 #include "lpd_rcvjob.h"
 #include "lpd_jobs.h"
 /**** ENDINCLUDE ****/
@@ -113,6 +113,7 @@ int Receive_job( int *sock, char *input )
 	int temp_fd = -1;				/* used for file opening and locking */
 	int filetype;				/* type of file - control or data */
 	int fd;						/* for log file */
+	int hold_fd = -1;				/* hold file */
 	int db, dbf, rlen;
 	struct line_list files, info, l;
 	struct job job;
@@ -127,7 +128,7 @@ int Receive_job( int *sock, char *input )
 
 	if( input && *input ) ++input;
 	Clean_meta(input);
-	Split(&info,input,Whitespace,0,0,0,0,0);
+	Split(&info,input,Whitespace,0,0,0,0,0,0);
 
 	DEBUGFC(DRECV1)Dump_line_list("Receive_job: input", &info );
 	if( info.count != 1 ){
@@ -225,6 +226,14 @@ int Receive_job( int *sock, char *input )
 		/* make sure we have length and filename */
 		filename = 0;
 		file_len = strtod(line+1,&filename);
+		if ((line+1) == filename){
+			/* Recover from Apple Desktop Printing stupidity.
+			   It occasionally resends the queue selection cmd.
+			   Darian Davis DD 03JUL2000 */
+			status = 0;
+			LOGERR(LOG_ERR)"Recovering from incorrect job submission");
+			continue;
+		}
 		if( filename ){
 			while( isspace(cval(filename)) ) ++filename;
 			Clean_meta(filename);
@@ -314,22 +323,25 @@ int Receive_job( int *sock, char *input )
 				job.info.count, files.count );
 			if( job.info.count ){
 				/* we received another control file */
-				if( Check_for_missing_files(&job, &files, error, errlen, 0) ){
+				if( Check_for_missing_files(&job, &files, error, errlen, 0, hold_fd) ){
 					goto error;
 				}
+				hold_fd = -1;
 				Free_line_list(&files);
 				jobsize = 0;
 				Free_job(&job);
 			}
 			Set_str_value(&job.info,OPENNAME,tempfile);
 			Set_str_value(&job.info,TRANSFERNAME,filename);
+			hold_fd = Set_up_temporary_control_file( &job, error, errlen );
 			if( files.count ){
 				/* we have datafiles, FOLLOWED by a control file,
 					followed (possibly) by another control file */
 				/* we receive another control file */
-				if( Check_for_missing_files(&job, &files, error, sizeof(error), 0) ){
+				if( Check_for_missing_files(&job, &files, error, errlen, 0, hold_fd) ){
 					goto error;
 				}
+				hold_fd = -1;
 				Free_line_list(&files);
 				jobsize = 0;
 				Free_job(&job);
@@ -344,9 +356,11 @@ int Receive_job( int *sock, char *input )
 	DEBUGF(DRECV2)("Receive_job: eof on transfer, job.info.count %d, files.count %d",
 		job.info.count, files.count );
 	if( job.info.count ){
-		if( Check_for_missing_files(&job, &files, error, sizeof(error), 0) ){
+		/* we receive another control file */
+		if( Check_for_missing_files(&job, &files, error, errlen, 0, hold_fd) ){
 			goto error;
 		}
+		hold_fd = -1;
 		Free_line_list(&files);
 		jobsize = 0;
 		Free_job(&job);
@@ -367,8 +381,12 @@ int Receive_job( int *sock, char *input )
 		/* LOG( LOG_INFO) "Receive_job: error '%s'", error ); */
 		DEBUGF(DRECV1)("Receive_job: sending ACK %d, msg '%s'", ack, error );
 		(void)Link_send( ShortRemote_FQDN, sock,
-			Send_job_rw_timeout_DYN, buffer, strlen(buffer), 0 );
+			Send_job_rw_timeout_DYN, buffer, safestrlen(buffer), 0 );
 		Link_close( sock );
+		if( hold_fd >= 0 ){
+			Remove_job( &job );
+			close(hold_fd); hold_fd = -1;
+		}
 	} else {
 		Link_close( sock );
 		Remove_tempfiles();
@@ -376,11 +394,16 @@ int Receive_job( int *sock, char *input )
 		if( Lpq_status_file_DYN ){
 			unlink( Lpq_status_file_DYN );
 		}
-		if( Server_queue_name_DYN ){
-			Do_queue_jobs( Server_queue_name_DYN, 0);
-		} else {
-			Do_queue_jobs( Printer_DYN, 0);
+		s = Server_queue_name_DYN;
+		if( !s ) s = Printer_DYN;
+
+		SNPRINTF( line, sizeof(line)) "\n" );
+		DEBUGF(DRECV1)("Receive_jobs: starting '%s'", s );
+		if( Write_fd_str( Lpd_request, line ) < 0 ){
+			LOGERR_DIE(LOG_ERR) _("Receive_jobs: write to fd '%d' failed"),
+				Lpd_request );
 		}
+		Do_queue_jobs( s, 0 );
 	}
 	cleanup( 0 );
 	return(0);
@@ -423,7 +446,7 @@ int Receive_block_job( int *sock, char *input )
 
 	if( *input ) ++input;
 	Clean_meta(input);
-	Split(&l,input,Whitespace,0,0,0,0,0);
+	Split(&l,input,Whitespace,0,0,0,0,0,0);
 	DEBUGFC(DRECV1)Dump_line_list("Receive_block_job: input", &l );
 
 	if( l.count != 2 ){
@@ -562,7 +585,7 @@ int Receive_block_job( int *sock, char *input )
 		/* LOG( LOG_INFO) "Receive_block_job: error '%s'", error ); */
 		DEBUGF(DRECV1)("Receive_block_job: sending ACK %d, msg '%s'", ack, error );
 		(void)Link_send( ShortRemote_FQDN, sock,
-			Send_job_rw_timeout_DYN, buffer, strlen(buffer), 0 );
+			Send_job_rw_timeout_DYN, buffer, safestrlen(buffer), 0 );
 		Link_close( sock );
 	} else {
 		Link_close( sock );
@@ -594,7 +617,7 @@ int Receive_block_job( int *sock, char *input )
  *          0 on success
  ***************************************************************************/
 
-int Scan_block_file( int fd, char *error, int errlen, char *auth_id )
+int Scan_block_file( int fd, char *error, int errlen, struct line_list *header_info )
 {
 	char line[LINEBUFFER];
 	char buffer[LARGEBUFFER];
@@ -604,6 +627,7 @@ int Scan_block_file( int fd, char *error, int errlen, char *auth_id )
 	char *tempfile;				/* name field */
 	int status;
 	int len, count, n;
+	int hold_fd = -1;
 	struct line_list l, info, files;
 	struct job job;
 	struct stat statb;
@@ -620,6 +644,7 @@ int Scan_block_file( int fd, char *error, int errlen, char *auth_id )
 	Init_job(&job);
 
 	/* first we find the file position */
+	
 	startpos = lseek( fd, 0, SEEK_CUR );
 	DEBUGF(DRECV2)("Scan_block_file: starting at %d", startpos );
 	while( (status = Read_one_line( fd, line, sizeof(line) )) > 0 ){
@@ -641,7 +666,7 @@ int Scan_block_file( int fd, char *error, int errlen, char *auth_id )
 			continue;
 		}
 		Clean_meta(line+1);
-		Split(&info,line+1,Whitespace,0,0,0,0,0);
+		Split(&info,line+1,Whitespace,0,0,0,0,0,0);
 		if( info.count != 2 ){
 			SNPRINTF( error, errlen)
 			_("bad length information '%s'"), line+1 );
@@ -682,26 +707,29 @@ int Scan_block_file( int fd, char *error, int errlen, char *auth_id )
 		tempfd = -1;
 
 		if( filetype == CONTROL_FILE ){
-			DEBUGF(DRECV2)("Scan_block_file- control file '%s'", filename );
+			DEBUGF(DRECV2)("Scan_block_file: control file '%s'", filename );
 			DEBUGF(DRECV2)("Scan_block_file: received control file, job.info.count %d, files.count %d",
 				job.info.count, files.count );
 			if( job.info.count ){
-				/* we received another control file */
-				if( Check_for_missing_files(&job, &files, error, errlen, 0) ){
+				if( Check_for_missing_files(&job, &files, error, errlen, 0, hold_fd) ){
 					goto error;
 				}
+				hold_fd = -1;
 				Free_line_list(&files);
 				Free_job(&job);
 			}
 			Set_str_value(&job.info,OPENNAME,tempfile);
 			Set_str_value(&job.info,TRANSFERNAME,filename);
+			hold_fd = Set_up_temporary_control_file( &job, error, errlen );
+			if( hold_fd < 0 ) goto error;
 			if( files.count ){
 				/* we have datafiles, FOLLOWED by a control file,
 					followed (possibly) by another control file */
 				/* we receive another control file */
-				if( Check_for_missing_files(&job, &files, error, sizeof(error), 0) ){
+				if( Check_for_missing_files(&job, &files, error, errlen, 0, hold_fd) ){
 					goto error;
 				}
+				hold_fd = -1;
 				Free_line_list(&files);
 				Free_job(&job);
 			}
@@ -711,16 +739,21 @@ int Scan_block_file( int fd, char *error, int errlen, char *auth_id )
 	}
 
 	if( files.count ){
-		if( Check_for_missing_files(&job, &files, error, sizeof(error), auth_id) ){
+		/* we receive another control file */
+		if( Check_for_missing_files(&job, &files, error, errlen, header_info, hold_fd) ){
 			goto error;
 		}
+		hold_fd = -1;
 		Free_line_list(&files);
 		Free_job(&job);
 	}
 
  error:
-	if( tempfd >= 0 ) close(tempfd);
-	tempfd = -1;
+	if( hold_fd >= 0 ){
+		Remove_job( &job );
+		close(hold_fd); hold_fd = -1;
+	}
+	if( tempfd >= 0 ) close(tempfd); tempfd = -1;
 	Free_line_list(&l);
 	Free_line_list(&info);
 	Free_line_list(&files);
@@ -810,28 +843,33 @@ int Do_perm_check( struct job *job, char *error, int errlen )
  *  spool_dir - the spool directory for this queue
  *  xlate_incoming_format - only valid for received files
  *  error, errlen - the error message information
- *  auth_id - authentication ID to put in the job
+ *  header_info - authentication ID to put in the job
  *   - if 0, do not update, this preserves copy
  *  returns: 0 - successful
  *          != 0 - error
  */
 
 int Check_for_missing_files( struct job *job, struct line_list *files,
-	char *error, int errlen, char *auth_id )
+	char *error, int errlen, struct line_list *header_info, int holdfile_fd )
 {
 	int count;
 	struct line_list *lp = 0, datafiles;
-	int fd = -1, status = 0;
+	int status = 0;
 	char *openname, *transfername;
 	double jobsize;
 	int copies;
 	struct stat statb;
 	struct timeval start_time;
 
+	if(DEBUGL1)Dump_job("Check_for_missing_files - start", job );
+	if(DEBUGL1)Dump_line_list("Check_for_missing_files- header_info", header_info );
+
 	if( gettimeofday( &start_time, 0 ) ){
 		Errorcode = JABORT;
 		LOGERR_DIE(LOG_INFO) "Check_for_missing_files: gettimeofday failed");
 	}
+	/* Setup_job must be called first */
+
 	DEBUG1("Check_for_missing_files: time 0x%x usec 0x%x",
 		(int)start_time.tv_sec, (int)start_time.tv_usec );
 
@@ -840,23 +878,20 @@ int Check_for_missing_files( struct job *job, struct line_list *files,
 
 	Init_line_list(&datafiles);
 
-	if( files ){
-		if(DEBUGL1)Dump_line_list("Check_for_missing_files - files", files );
-		Setup_job( job, &Spool_control, Spool_dir_DYN, 0, 0, 0);
-	} else {
-		Check_for_hold( job, &Spool_control );
-	}
-
-	if(DEBUGL1)Dump_job("Check_for_missing_files - job after setup", job );
-
 	/* we can get this as a new job or as a copy.
 	 * if we get a copy,  we do not need to check this stuff
 	 */
+	if( !Find_str_value(&job->info,REMOTEHOST,Value_sep) ){
+		Set_str_value(&job->info,REMOTEHOST,RemoteHost_IP.fqdn);
+		Set_flag_value(&job->info,UNIXSOCKET,Perm_check.unix_socket);
+		Set_flag_value(&job->info,REMOTEPORT,Perm_check.port);
+	}
 	if( files ){
 		if( Do_perm_check( job, error, errlen ) == P_REJECT ){
+			status = 1;
 			goto error;
 		}
-	
+
 		jobsize = 0;
 		error[0] = 0;
 		/* RedHat Linux 6.1 - sends a control file with NO data files */
@@ -881,17 +916,19 @@ int Check_for_missing_files( struct job *job, struct line_list *files,
 		}
 		for( count = 0; count < job->datafiles.count; ++count ){
 			lp = (void *)job->datafiles.list[count];
-			transfername = Find_str_value(lp,TRANSFERNAME,Value_sep);
+			transfername = Find_str_value(lp,OTRANSFERNAME,Value_sep);
+			if( ISNULL(transfername) ) transfername = Find_str_value(lp,TRANSFERNAME,Value_sep);
 			/* find the open name and replace it in the information */
 			if( (openname = Find_casekey_str_value(files,transfername,Value_sep)) ){
 				Set_str_value(lp,OPENNAME,openname);
 				Set_casekey_str_value(&datafiles,transfername,openname);
 			} else {
 				SNPRINTF(error,errlen)"missing data file '%s'",transfername);
+				status = 1;
 				goto error;
 			}
 			if( (status = stat( openname, &statb )) ){
-				SNPRINTF( error, errlen) "stat() '%s' error - %s",
+					SNPRINTF( error, errlen) "stat() '%s' error - %s",
 					openname, Errormsg(errno) );
 				goto error;
 			}
@@ -911,20 +948,31 @@ int Check_for_missing_files( struct job *job, struct line_list *files,
 	}
 
 	/* now we need to assign a control file number */
-	if( (fd = Find_non_colliding_job_number( job )) < 0 ){
+	if( holdfile_fd <= 0 && (holdfile_fd = Find_non_colliding_job_number( job )) < 0 ){
 		SNPRINTF(error,errlen)
 			"cannot allocate hold file");
 		status = 1;
 		goto error;
 	}
+	DEBUG1("Check_for_missing_files: hold file fd '%d'", holdfile_fd );
 
-	if( Create_control( job, error, errlen, auth_id, Xlate_incoming_format_DYN ) ){
+	if( Create_control( job, error, errlen, Xlate_incoming_format_DYN ) ){
 		DEBUG1("Check_for_missing_files: Create_control error '%s'", error );
 		status = 1;
 		goto error;
 	}
 	Set_str_value(&job->info,HPFORMAT,0);
+	Set_str_value(&job->info,INCOMING_TIME,0);
 
+	if( header_info ){
+		char *authfrom, *authuser, *authtype;
+		authfrom = Find_str_value(header_info,AUTHFROM,Value_sep);
+		authuser = Find_str_value(header_info,AUTHUSER,Value_sep);
+		authtype = Find_str_value(header_info,AUTHTYPE,Value_sep);
+		Set_str_value(&job->info,AUTHUSER,authuser);
+		Set_str_value(&job->info,AUTHFROM,authfrom);
+		Set_str_value(&job->info,AUTHTYPE,authtype);
+	}
 	/* now we do the renaming */
 	status = 0;
 	for( count = 0; status == 0 && count < job->datafiles.count; ++count ){
@@ -956,7 +1004,7 @@ int Check_for_missing_files( struct job *job, struct line_list *files,
 		if( transfername ) free( transfername ); transfername = 0;
 		goto error;
 	}
-	if( (status = Set_hold_file( job, 0 )) ){
+	if( (status = Set_hold_file( job, 0, holdfile_fd )) ){
 		SNPRINTF( error,errlen)
 			"error setting up hold file - %s",
 			Errormsg( errno ) );
@@ -989,11 +1037,54 @@ int Check_for_missing_files( struct job *job, struct line_list *files,
 		setmessage( job, "STATE", "CREATE" );
 	}
 
-	if( fd >= 0 ) close(fd);
+	if( holdfile_fd >= 0 ) close(holdfile_fd);
 	Free_line_list(&datafiles);
 	return( status );
 }
 
+/***************************************************************************
+ * int Set_up_temporary_control_file( struct job *job,
+ *	char *error, int errlen )
+ *  sets up a hold file and control file
+ ***************************************************************************/
+
+int Set_up_temporary_control_file( struct job *job,
+	char *error, int errlen  )
+{
+	int fd = -1;
+	/* now we need to assign a control file number */
+	DEBUG1("Set_up_temporary_control_file: starting" );
+
+	/* sets identifier and hold information */
+	Setup_job( job, &Spool_control, 0, 0, 0);
+	/* now we get collision resolution */
+	if( (fd = Find_non_colliding_job_number( job )) < 0 ){
+		SNPRINTF(error,errlen)
+			"cannot allocate hold file");
+		goto error;
+	}
+	DEBUG1("Set_up_temporary_control_file: hold file fd '%d'", fd );
+	
+	/* create the control file */
+	if( Create_control( job, error, errlen, Xlate_incoming_format_DYN ) ){
+		DEBUG1("Set_up_temporary_control_file: Create_control error '%s'", error );
+		close(fd); fd = -1;
+		goto error;
+	}
+	/* mark as incoming */
+	Set_flag_value(&job->info,INCOMING_TIME,time((void *)0) );
+	/* write status */
+	if( Set_hold_file( job, 0, fd ) ){
+		SNPRINTF( error,errlen)
+			"error setting up hold file - %s",
+			Errormsg( errno ) );
+		close(fd); fd = -1;
+		goto error;
+	}
+ error:
+	return( fd );
+}
+
 /***************************************************************************
  * int Find_non_colliding_job_number( struct job *job )
  *  Find a non-colliding job number for the new job
@@ -1041,11 +1132,11 @@ int Find_non_colliding_job_number( struct job *job )
 
 int Get_route( struct job *job, char *error, int errlen )
 {
-	int i, fd, tempfd, count;
+	int i, fd, tempfd, count, c;
 	char *tempfile, *openname, *s, *t, *id;
 	char buffer[SMALLBUFFER];
 	int errorcode = 0;
-	struct line_list info, dest, env, *lp;
+	struct line_list info, dest, env, *lp, cf_line_list;
 
 	DEBUG1("Get_route: routing filter '%s', control filter '%s'",
 		Routing_filter_DYN, Incoming_control_filter_DYN );
@@ -1056,6 +1147,7 @@ int Get_route( struct job *job, char *error, int errlen )
 	Init_line_list(&info);
 	Init_line_list(&dest);
 	Init_line_list(&env);
+	Init_line_list(&cf_line_list);
 
 	/* build up the list of files and initialize the DATAFILES
 	 * environment variable
@@ -1114,6 +1206,27 @@ int Get_route( struct job *job, char *error, int errlen )
 			goto error;
 		}
 		Max_open(fd);
+		if( Get_file_image_and_split(openname,0,0, &cf_line_list, Line_ends,0,0,0,0,0,0) ){
+            SNPRINTF(error,errlen)
+                "Get_route: open failed - modified control file  %s - %s", openname, Errormsg(errno) );
+			goto error;
+		}
+		for( i = 'A'; i <= 'Z'; ++i ){
+			buffer[1] = 0;
+			buffer[0] = i;
+			Set_str_value(&job->info,buffer,0);
+		}
+		for( i = 0; i < cf_line_list.count; ++i ){
+			s = cf_line_list.list[i];
+			Clean_meta(s);
+			c = cval(s);
+			DEBUG3("Get_route: doing line '%s'", s );
+			if( isupper(c) && c != 'U' && c != 'N' ){
+				buffer[0] = c; buffer[1] = 0;
+				DEBUG3("Get_route: control '%s'='%s'", buffer, s+1 );
+				Set_str_value(&job->info,buffer,s+1);
+			}
+		}
 		tempfd = Make_temp_fd(&tempfile);
 	}
 
@@ -1139,8 +1252,13 @@ int Get_route( struct job *job, char *error, int errlen )
 	Free_line_list( &env );
 	Get_file_image_and_split(tempfile,0,1,&env,Line_ends,0,0,0,1,0,0);
 	Free_line_list(&job->destination);
+
 	id = Find_str_value(&job->info,IDENTIFIER,Value_sep);
-	if(!id) id = Find_str_value(&job->info,TRANSFERNAME,Value_sep);
+	if(!id){
+		FATAL(LOG_ERR)
+			_("Get_route: no identifier for '%s'"),
+			Find_str_value(&job->info,HF_NAME,Value_sep) );
+	}
 	count = 0;
 	for(i = 0; i < env.count; ++i ){
 		s = env.list[i];
@@ -1192,5 +1310,6 @@ int Get_route( struct job *job, char *error, int errlen )
 	Free_line_list(&info);
 	Free_line_list(&dest);
 	Free_line_list(&env);
+	Free_line_list(&cf_line_list);
 	return( errorcode );
 }
