@@ -1,20 +1,20 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: lpq.c,v 5.18 2000/12/25 01:51:11 papowell Exp papowell $";
+"$Id: lpq.c,v 1.30 2002/05/06 01:06:41 papowell Exp $";
 
 
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
@@ -162,8 +162,6 @@ int main(int argc, char *argv[], char *envp[])
 	}
 
 	if(DEBUGL1)Dump_line_list("lpq- Config", &Config_line_list );
-	Get_all_printcap_entries();
-	if(DEBUGL1)Dump_line_list("lpq- All_line_list", &All_line_list );
 	/* we do the individual printers */
 	if( Displayformat == REQ_DLONG && Longformat && Status_line_count <= 0 ){
 		Status_line_count = (1 << (Longformat-1));
@@ -177,11 +175,15 @@ int main(int argc, char *argv[], char *envp[])
 		}
 		if( All_printers ){
 			DEBUG1("lpq: all printers");
+			Get_all_printcap_entries();
+			if(DEBUGL1)Dump_line_list("lpq- All_line_list", &All_line_list );
 			for( i = 0; i < All_line_list.count; ++i ){
 				Set_DYN(&Printer_DYN,All_line_list.list[i] );
 				Show_status(argv);
 			}
 		} else {
+			/* set up configuration */
+			Get_printer();
 			Show_status(argv);
 		}
 		DEBUG1("lpq: done");
@@ -206,10 +208,17 @@ void Show_status(char **argv)
 	char msg[LINEBUFFER];
 
 	DEBUG1("Show_status: start");
-	/* set up configuration */
-	Get_printer();
+
 	Fix_Rm_Rp_info(0,0);
 
+	if( ISNULL(RemotePrinter_DYN) ){
+		SNPRINTF( msg, sizeof(msg))
+			_("Printer: %s - cannot get status from device '%s'\n"),
+			Printer_DYN, Lp_device_DYN );
+		if(  Write_fd_str( 1, msg ) < 0 ) cleanup(0);
+		return;
+	}
+
 	if( Displayformat != REQ_DSHORT
 		&& safestrcasecmp(Printer_DYN, RemotePrinter_DYN) ){
 		SNPRINTF( msg, sizeof(msg)) _("Printer: %s is %s@%s\n"),
@@ -224,6 +233,13 @@ void Show_status(char **argv)
 		if(  Write_fd_str( 1, msg ) < 0 ) cleanup(0);
 		return;
 	}
+	if( Direct_DYN && Lp_device_DYN ){
+		SNPRINTF( msg, sizeof(msg))
+			_("Printer: %s - direct connection to device '%s'\n"),
+			Printer_DYN, Lp_device_DYN );
+		if(  Write_fd_str( 1, msg ) < 0 ) cleanup(0);
+		return;
+	}
 	if( Auth ){
 		Set_DYN(&Auth_DYN, getenv("AUTH") );
 	}
@@ -231,7 +247,7 @@ void Show_status(char **argv)
 		&argv[Optind], Connect_timeout_DYN,
 		Send_query_rw_timeout_DYN, 1 );
 	if( fd >= 0 ){
-		shutdown( fd, 1 );
+		/* shutdown( fd, 1 ); */
 		if( Read_status_info( RemoteHost_DYN, fd,
 			1, Send_query_rw_timeout_DYN, Displayformat,
 			Status_line_count ) ){
@@ -274,12 +290,13 @@ int Read_status_info( char *host, int sock,
 	DEBUG1("Read_status_info: output %d, timeout %d, dspfmt %d",
 		output, timeout, displayformat );
 	DEBUG1("Read_status_info: status_line_count %d", status_line_count );
+	DEBUG1("Read_status_info: displayformat %d, Show_all %d",displayformat, Show_all );
 
 	/*
 	 * Do not try to be fancy - the overhead is not high unless lots
 	 * of data and then something is wrong.
 	 */
-	if( displayformat == REQ_VERBOSE || displayformat == REQ_LPSTAT ){
+	if( displayformat == REQ_VERBOSE || displayformat == REQ_LPSTAT || Show_all ){
 		do{ 
 			if( (n = read( sock, buffer, sizeof(buffer)-1)) ){
 				buffer[n] = 0;
@@ -313,18 +330,37 @@ int Read_status_info( char *host, int sock,
 	for( line = 0; line < l.count; ){
 		/* we start by looking at the first line and seeing if it is
 		 * for a printer that we have already found
+		 * if look_for_pr is 1 then we have just started the search
+		 *    - we assume that any non-blank line not containing a Printer:
+		 *      line is a 'proper' entry.
+		 * if look_for_pr is 2 then we assume that we are looking for
+		 *    the end of a printer entry and we look for a line with
+		 *    Printer: in it that we have not seen 
 		 */
 		while( look_for_pr && line < l.count ){
 			s = l.list[line];
-			if( s == 0 || isspace(cval(s))
+			/* we do not want a line starting with a space or a blank line */
+			if( ISNULL(s) ){
+				look_for_pr = 1;
+			} else if( look_for_pr == 1 &&
+				!(strstr(s,"Printer:") || strstr(s,_("Printer:")) ) ){
+				look_for_pr = 0;
+			} else if( isspace(cval(s))
+				/* if line starts with a space */
+				/*  line does not contain Printer: */
 				|| !(strstr(s,"Printer:") || strstr(s,_("Printer:")) )
+				/* or that already is in the list */ 
 				|| Find_exists_value(&Printer_list,s,0) ){
-				;
+				look_for_pr = 2;
 			} else {
+				look_for_pr = 0;
+			}
+			if( look_for_pr == 0 ){
 				if( Write_fd_str( output, s ) < 0
 					|| Write_fd_str( output, "\n" ) < 0 ) return(1);
-				look_for_pr = 0;
-				Add_line_list(&Printer_list,s,0,1,0);
+				if( strstr(s,"Printer:") || strstr(s,_("Printer:")) ){
+					Add_line_list(&Printer_list,s,0,1,0);
+				}
 				DEBUG1("Read_status_info: pr [%d] '%s'", line, s );
 			}
 			++line;
@@ -388,7 +424,7 @@ int Read_status_info( char *host, int sock,
 	}
 	DEBUG1("Read_status_info: after checks look_for_pr %d, line %d, last_line %d",
 		look_for_pr, line, last_line);
-	if( !look_for_pr ){
+	if( !look_for_pr && last_line >= 0 ){
 		n = l.count - status_line_count;
 		if( n < last_line ) n = last_line;
 		for( ; n < l.count; ++n ){
@@ -505,21 +541,21 @@ void Get_parms(int argc, char *argv[] )
 }
 
  char *lpq_msg[] = {
-N_("usage: %s [-aAclV] [-Ddebuglevel] [-Pprinter] [-tsleeptime]\n"),
-N_("  -A           - use authentication specified by AUTH environment variable\n"),
-N_("  -a           - all printers\n"),
-N_("  -c           - clear screen before update\n"),
-N_("  -l           - increase (lengthen) detailed status information\n"),
-N_("                 additional l flags add more detail.\n"),
-N_("  -L           - maximum detailed status information\n"),
-N_("  -n linecount - linecount lines of detailed status information\n"),
-N_("  -Ddebuglevel - debug level\n"),
-N_("  -Pprinter    - specify printer\n"),
-N_("  -s           - short (summary) format\n"),
-N_("  -tsleeptime  - sleeptime between updates\n"),
-N_("  -V           - print version information\n"),
-0 };
-
+ N_("usage: %s [-aAclV] [-Ddebuglevel] [-Pprinter] [-tsleeptime]\n"),
+ N_("  -A           - use authentication specified by AUTH environment variable\n"),
+ N_("  -a           - all printers\n"),
+ N_("  -c           - clear screen before update\n"),
+ N_("  -l           - increase (lengthen) detailed status information\n"),
+ N_("                 additional l flags add more detail.\n"),
+ N_("  -L           - maximum detailed status information\n"),
+ N_("  -n linecount - linecount lines of detailed status information\n"),
+ N_("  -Ddebuglevel - debug level\n"),
+ N_("  -Pprinter    - specify printer\n"),
+ N_("  -s           - short (summary) format\n"),
+ N_("  -tsleeptime  - sleeptime between updates\n"),
+ N_("  -V           - print version information\n"),
+ 0 };
+ 
 void usage(void)
 {
 	char *s;
@@ -531,22 +567,14 @@ void usage(void)
 			FPRINTF( STDERR, "%s", _(s) );
 		}
 	}
+	Parse_debug("=",-1);
+	FPRINTF( STDOUT, "%s\n", Version );
 	exit(1);
 }
 
- void Dispatch_input(int *talk, char *input ){}
-/*
- * Calls[] = list of dispatch functions 
- */
-
- struct call_list Calls[] = {
-	{0,0}
-};
-
 #if 0
 
 #include "permission.h"
-#include "lpd.h"
 #include "lpd_status.h"
 /* int Send_request( */
 	int class,					/* 'Q'= LPQ, 'C'= LPC, M = lprm */
@@ -565,7 +593,7 @@ void usage(void)
 	cmd[1] = 0;
 	SNPRINTF(cmd+1, sizeof(cmd)-1, "%s", RemotePrinter_DYN);
 	for( i = 0; options[i]; ++i ){
-		n = strlen(cmd);
+		n = safestrlen(cmd);
 		SNPRINTF(cmd+n,sizeof(cmd)-n," %s",options[i] );
 	}
 	Perm_check.remoteuser = "papowell";
