@@ -1,14 +1,14 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: lpr.c,v 5.26 2000/12/25 01:51:11 papowell Exp papowell $";
+"$Id: lpr.c,v 1.30 2002/05/06 01:06:41 papowell Exp $";
 
 
 #include "lp.h"
@@ -57,7 +57,7 @@
 int main(int argc, char *argv[], char *envp[])
 {
 	off_t job_size;
-	char *s, *t, buffer[SMALLBUFFER];
+	char *s, *t, buffer[SMALLBUFFER], *send_to_pr = 0;
 	struct job prjob;
 	struct line_list opts, newargs;
 	int n;
@@ -133,6 +133,7 @@ int main(int argc, char *argv[], char *envp[])
 		newargs.list[newargs.count] = 0;
 		if(DEBUGL1)Dump_line_list("lpr - new options",&newargs );
 		Optind = 0;
+		Files.count = 0;
 		Getopt(0,0,0);
 		Get_parms(newargs.count, newargs.list);      /* scan input args */
 		newargs.count = 0;
@@ -149,6 +150,7 @@ int main(int argc, char *argv[], char *envp[])
             }
         }
     }
+
 	/*
 	 * Fix the rest of the control file
 	 */
@@ -173,7 +175,6 @@ int main(int argc, char *argv[], char *envp[])
 	/* we check to see if we need to do control file filtering */
 	/* we do not do any translation of formats */
 	s = 0;
-	Fix_control( &prjob, Control_filter_DYN, 0 );
 
 	n = Find_flag_value( &prjob.info,DATAFILE_COUNT,Value_sep);
 	if( Max_datafiles_DYN > 0 && n > Max_datafiles_DYN ){
@@ -182,25 +183,76 @@ int main(int argc, char *argv[], char *envp[])
 					n, Max_datafiles_DYN );
 	}
 
-	s = 0;
-	if( (Direct_JOB || Direct_DYN) ){
-		Lpr_bounce_DYN = Lpr_bounce_JOB = 0;
+	send_to_pr = 0;
+	if( Direct_JOB ){
 		/* check to see if we have a socket connection specified */
-		s = Printer_JOB;
-		if( !s ){ s = Find_str_value(&PC_entry_line_list,LP,Value_sep ); }
-		if( s ){
-			s = safestrdup(s,__FILE__,__LINE__);
-			Expand_percent(&s);
-		}
+		send_to_pr = Printer_JOB;
+	} else if( Direct_DYN ){
+		send_to_pr = Lp_device_DYN;
 	}
-	if( s && (strchr(s,'%') || (s[0] == '/') || strchr(s,'|')) ){
+	Force_localhost_DYN = 0;
+	if( send_to_pr ){
+		Force_localhost_DYN = 0;
+		Lpr_bounce_DYN = Lpr_bounce_JOB = 0;
+		send_to_pr = safestrdup(send_to_pr,__FILE__,__LINE__);
+		Expand_percent(&send_to_pr);
+	}
+
+	DEBUG1("lpr: send_to_pr '%s'", send_to_pr );
+
+	if( Lpr_bounce_DYN || Lpr_bounce_JOB ){
+		int tempfd;
+		struct stat statb;
+		char *tempfile, *old_lp_value;
+		struct line_list *lp;
+
+		if(DEBUGL2) Dump_job( "lpr - before filtering", &prjob );
+		tempfd = Make_temp_fd(&tempfile);
+
+		old_lp_value = safestrdup(Find_str_value( &PC_entry_line_list, "lp", Value_sep ),
+			__FILE__,__LINE__);
+		Set_str_value( &PC_entry_line_list, LP, tempfile );
+		/* Print_job( output_device, status_device, job, timeout, poll_for_status ) */
+		Print_job( tempfd, -1, &prjob, 0, 0, User_filter_JOB );
+		Set_str_value( &PC_entry_line_list, LP, old_lp_value );
+		if( old_lp_value ) free( old_lp_value ); old_lp_value = 0;
+
+		close(tempfd);
+		tempfd = Checkread( tempfile, &statb );
+		if( tempfd < 0 ){
+			Errorcode = JABORT;
+			FATAL(LOG_INFO) _("Cannot open file '%s', %s"), tempfile, Errormsg( errno ) );
+		}
+		close(tempfd);
+		DEBUG2("lpr: jobs size now %0.0f", (double)(statb.st_size));
+		job_size = statb.st_size;
+		Free_listof_line_list(&prjob.datafiles);
+		lp = malloc_or_die(sizeof(lp[0]),__FILE__,__LINE__);
+		memset(lp,0,sizeof(lp[0]));
+		Check_max(&prjob.datafiles,1);
+		prjob.datafiles.list[prjob.datafiles.count++] = (void *)lp;
+		Set_str_value(lp,OPENNAME,tempfile);
+		Set_str_value(lp,"N",_("(lpr_filter)"));
+		Set_flag_value(lp,COPIES,1);
+		Set_double_value(lp,SIZE,job_size);
+		Fix_bq_format( 'f', lp );
+		User_filter_JOB = 0;
+	}
+
+	if(DEBUGL1)Dump_job("lpr - before Fix_control",&prjob);
+	Fix_control( &prjob, Control_filter_DYN, 0 );
+	if(DEBUGL1)Dump_job("lpr - after Fix_control",&prjob);
+
+	if( send_to_pr &&
+		((!strchr(send_to_pr,'@') && strchr(send_to_pr,'%'))
+			|| (send_to_pr[0] == '/') || strchr(send_to_pr,'|')) ){
 		int fd, pid, status_fd, poll_for_status;
 		char *id;
 
-		SETSTATUS(&prjob)"destination '%s'", s );
+		SETSTATUS(&prjob)"destination '%s'", send_to_pr );
 		Errorcode = 0;
 		fd = pid = status_fd = poll_for_status = 0;
-		fd = Printer_open(s, &status_fd, &prjob,
+		fd = Printer_open(send_to_pr, &status_fd, &prjob,
 			Send_try_DYN, Connect_interval_DYN, Max_connect_interval_DYN,
 			Connect_grace_DYN, Connect_timeout_DYN, &pid, &poll_for_status );
 
@@ -236,52 +288,6 @@ int main(int argc, char *argv[], char *envp[])
 		}
 		DEBUG1("lpr: status %s", Server_status(Errorcode) );
 	} else {
-		/* do we flatten job ? */
-		if( Lpr_bounce_DYN || Lpr_bounce_JOB ){
-			int tempfd;
-			struct stat statb;
-			char *tempfile, *old_lp_value;
-			struct line_list *lp;
-
-			if(DEBUGL2) Dump_job( "lpr - before filtering", &prjob );
-			tempfd = Make_temp_fd(&tempfile);
-
-			old_lp_value = safestrdup(Find_str_value( &PC_entry_line_list, "lp", Value_sep ),
-				__FILE__,__LINE__);
-			Set_str_value( &PC_entry_line_list, LP, tempfile );
-			/* Print_job( output_device, status_device, job, timeout, poll_for_status ) */
-			Print_job( tempfd, -1, &prjob, 0, 0, User_filter_JOB );
-			Set_str_value( &PC_entry_line_list, LP, old_lp_value );
-			if( old_lp_value ) free( old_lp_value ); old_lp_value = 0;
-
-			close(tempfd);
-			tempfd = Checkread( tempfile, &statb );
-			if( tempfd < 0 ){
-				Errorcode = JABORT;
-				FATAL(LOG_INFO) _("Cannot open file '%s', %s"), tempfile, Errormsg( errno ) );
-			}
-			close(tempfd);
-			DEBUG2("lpr: jobs size now %0.0f", (double)(statb.st_size));
-			job_size = statb.st_size;
-			Free_listof_line_list(&prjob.datafiles);
-			lp = malloc_or_die(sizeof(lp[0]),__FILE__,__LINE__);
-			memset(lp,0,sizeof(lp[0]));
-			Check_max(&prjob.datafiles,1);
-			prjob.datafiles.list[prjob.datafiles.count++] = (void *)lp;
-			Set_str_value(lp,OPENNAME,tempfile);
-			if( Bounce_queue_format_DYN ){
-				Set_str_value(lp,FORMAT,Bounce_queue_format_DYN);
-			}
-			Set_str_value(lp,"N",_("(lpr_filter)"));
-			Set_flag_value(lp,COPIES,1);
-			Set_double_value(lp,SIZE,job_size);
-			User_filter_JOB = 0;
-		} else if( Direct_JOB || Direct_DYN
-			|| (User_filter_JOB && prjob.datafiles.count > 1) ){
-			Filter_files_in_job( &prjob, -1, User_filter_JOB );
-			User_filter_JOB = 0;
-		}
-		/* Send job to the LPD server for the printer */
 		Errorcode = Send_job( &prjob, &prjob, Connect_timeout_DYN,
 			Connect_interval_DYN,
 			Max_connect_interval_DYN,
@@ -289,6 +295,7 @@ int main(int argc, char *argv[], char *envp[])
 	}
 
   exit:
+	if( send_to_pr ) free(send_to_pr); send_to_pr = 0;
 	if( Errorcode ){
 		Errorcode = 1;
 		if(DEBUGL1)Dump_job("lpr - after error",&prjob);
@@ -494,7 +501,7 @@ void Get_parms(int argc, char *argv[] )
 		case 'D': 	Parse_debug(Optarg,1);
 			break;
 		case 'F':
-		    if( strlen (Optarg) != 1 ){
+		    if( safestrlen (Optarg) != 1 ){
 		        DIEMSG( _("bad -F format string '%s'\n"), Optarg);
 		    }
 		    if( Format_JOB ){
@@ -630,74 +637,75 @@ void Get_parms(int argc, char *argv[] )
 
  char *LPR_msg [] =
 {
-N_("Usage: %s [-Pprinter[@host]] [-A] [-B] [-Cclass] [-Fformat] [-G] [-Jinfo]\n"),
-N_("   [-(K|#)copies] [-Q] [-Raccountname]  [-Ttitle]  [-Uuser[@host]] [-V]\n"),
-N_("   [-Zoptions] [-b] [-m mailaddr] [-h] [-i indent] [-l] [-w num ] [-r]\n"),
-N_("   [-Ddebugopt ] [--] [ filenames ...  ]\n"),
-N_(" -A          - use authentication specified by AUTH environment variable\n"),
-N_(" -B          - filter files and reduce job to single file before sending\n"),
-N_(" -C class    - job class\n"),
-N_(" -D debugopt - debugging flags\n"),
-N_(" -F format   - job format\n"),
-N_("   -b,-l        - binary or literal format\n"),
-N_("    c,d,f,g,l,m,p,t,v are also format options\n"),
-N_(" -G          - filter individual job files before sending\n"),
-N_(" -J info     - banner and job information\n"),
-N_(" -K copies, -# copies   - number of copies\n"),
-N_(" -P printer[@host] - printer on host\n"),
-N_(" -Q          - put 'queuename' in control file\n"),
-N_(" -Raccntname - accounting information\n"),
-N_(" -T title    - title for 'pr' (-p) formatting\n"),
-N_(" -U username - override user name (restricted)\n"),
-N_(" -V          - Verbose information during spooling\n"),
-N_(" -X path     - user specified filter for job files\n"),
-N_(" -Y          - connect and send to TCP/IP port\n"),
-N_(" -Z options  - options to pass to filter\n"),
-N_(" -h          - no header or banner page\n"),
-N_(" -i indent   - indentation\n"),
-N_(" -m mailaddr - mail final status to mailaddr\n"),
-N_(" -r          - remove files after spooling\n"),
-N_(" -w width    - width to use\n"),
-N_(" --          - end of options, files follow\n"),
-N_(" filename '-'  reads from STDIN\n"),
-N_(" PRINTER, LPDEST, NPRINTER, NGPRINTER environment variables set default printer.\n"),
+ N_("Usage: %s [-Pprinter[@host]] [-A] [-B] [-Cclass] [-Fformat] [-G] [-Jinfo]\n"),
+ N_("   [-(K|#)copies] [-Q] [-Raccountname]  [-Ttitle]  [-Uuser[@host]] [-V]\n"),
+ N_("   [-Zoptions] [-b] [-m mailaddr] [-h] [-i indent] [-l] [-w width ] [-r]\n"),
+ N_("   [-Ddebugopt ] [--] [ filenames ...  ]\n"),
+ N_(" -A          - use authentication specified by AUTH environment variable\n"),
+ N_(" -B          - filter files and reduce job to single file before sending\n"),
+ N_(" -C class    - job class\n"),
+ N_(" -D debugopt - debugging flags\n"),
+ N_(" -F format   - job format\n"),
+ N_("   -b,-l        - binary or literal format\n"),
+ N_("    c,d,f,g,l,m,p,t,v are also format options\n"),
+ N_(" -G          - filter individual job files before sending\n"),
+ N_(" -J info     - banner and job information\n"),
+ N_(" -K copies, -# copies   - number of copies\n"),
+ N_(" -P printer[@host] - printer on host\n"),
+ N_(" -Q          - put 'queuename' in control file\n"),
+ N_(" -Raccntname - accounting information\n"),
+ N_(" -T title    - title for 'pr' (-p) formatting\n"),
+ N_(" -U username - override user name (restricted)\n"),
+ N_(" -V          - Verbose information during spooling\n"),
+ N_(" -X path     - user specified filter for job files\n"),
+ N_(" -Y          - connect and send to TCP/IP port (direct mode)\n"),
+ N_(" -Z options  - options to pass to filter\n"),
+ N_(" -h          - no header or banner page\n"),
+ N_(" -i indent   - indentation\n"),
+ N_(" -k          - do not use tempfile when sending to server\n"),
+ N_(" -m mailaddr - mail final status to mailaddr\n"),
+ N_(" -r          - remove files after spooling\n"),
+ N_(" -w width    - width to use\n"),
+ N_(" --          - end of options, files follow\n"),
+ N_(" filename '-'  reads from STDIN\n"),
+ N_(" PRINTER, LPDEST, NPRINTER, NGPRINTER environment variables set default printer.\n"),
  0 };
 
  char *LP_msg [] = {
-N_("Usage: %s [-A] [-B] [-c] [-G] [-m] [-p] [-s] [-w] [-d printer@[host]]\n"),
-N_("  [-f form-name] [-H special-handling]\n"),
-N_("  [-n number] [-o options] [-P page-list]\n"),
-N_("  [-q priority-level] [-S character-set]\n"),
-N_("  [-S print-wheel] [-t title]\n"),
-N_("  [-T content-type [-r]] [-y mode-list]\n"),
-N_("  [-Ddebugopt ] [ filenames ...  ]\n"),
-N_(" lp simulator using LPRng,  functionality may differ slightly\n"),
-N_(" -A          - use authentication specified by AUTH environment variable\n"),
-N_(" -B          - filter files and reduce job to single file before sending\n"),
-N_(" -c          - (make copy before printing - ignored)\n"),
-N_(" -d printer[@host]  - printer on host\n"),
-N_(" -D debugflags  - debugging flags\n"),
-N_(" -f formname - first letter used as job format\n"),
-N_(" -G          - filter individual job files before sending\n"),
-N_(" -H handling - (passed as -Z handling)\n"),
-N_(" -m          - mail sent to $USER on completion\n"),
-N_(" -n copies   - number of copies\n"),
-N_(" -o option     nobanner, width recognized\n"),
-N_("               (others passed as -Z option)\n"),
-N_(" -P pagelist - (print page list - ignored)\n"),
-N_(" -p          - (notification on completion - ignored)\n"),
-N_(" -q          - priority - 0 -> Z (highest), 25 -> A (lowest)\n"),
-N_(" -s          - (suppress messages - ignored)\n"),
-N_(" -S charset  - (passed as -Z charset)\n"),
-N_(" -t title    - job title\n"),
-N_(" -T content  - (passed as -Z content)\n"),
-N_(" -w          - (write message on completion - ignored)\n"),
-N_(" -X path     - user specified filter for job files\n"),
-N_(" -Y          - connect and send to TCP/IP port\n"),
-N_(" -y mode     - (passed as -Z mode)\n"),
-N_(" --          - end of options, files follow\n"),
-N_(" filename '-'  reads from STDIN\n"),
-N_(" PRINTER, LPDEST, NGPRINTER, NPRINTER environment variables set default printer.\n"),
+ N_("Usage: %s [-A] [-B] [-c] [-G] [-m] [-p] [-s] [-w] [-d printer@[host]]\n"),
+ N_("  [-f form-name] [-H special-handling]\n"),
+ N_("  [-n number] [-o options] [-P page-list]\n"),
+ N_("  [-q priority-level] [-S character-set]\n"),
+ N_("  [-S print-wheel] [-t title]\n"),
+ N_("  [-T content-type [-r]] [-y mode-list]\n"),
+ N_("  [-Ddebugopt ] [ filenames ...  ]\n"),
+ N_(" lp simulator using LPRng,  functionality may differ slightly\n"),
+ N_(" -A          - use authentication specified by AUTH environment variable\n"),
+ N_(" -B          - filter files and reduce job to single file before sending\n"),
+ N_(" -c          - (make copy before printing - ignored)\n"),
+ N_(" -d printer[@host]  - printer on host\n"),
+ N_(" -D debugflags  - debugging flags\n"),
+ N_(" -f formname - first letter used as job format\n"),
+ N_(" -G          - filter individual job files before sending\n"),
+ N_(" -H handling - (passed as -Z handling)\n"),
+ N_(" -m          - mail sent to $USER on completion\n"),
+ N_(" -n copies   - number of copies\n"),
+ N_(" -o option     nobanner, width recognized\n"),
+ N_("               (others passed as -Z option)\n"),
+ N_(" -P pagelist - (print page list - ignored)\n"),
+ N_(" -p          - (notification on completion - ignored)\n"),
+ N_(" -q          - priority - 0 -> Z (highest), 25 -> A (lowest)\n"),
+ N_(" -s          - (suppress messages - ignored)\n"),
+ N_(" -S charset  - (passed as -Z charset)\n"),
+ N_(" -t title    - job title\n"),
+ N_(" -T content  - (passed as -Z content)\n"),
+ N_(" -w          - (write message on completion - ignored)\n"),
+ N_(" -X path     - user specified filter for job files\n"),
+ N_(" -Y          - connect and send to TCP/IP port (direct mode)\n"),
+ N_(" -y mode     - (passed as -Z mode)\n"),
+ N_(" --          - end of options, files follow\n"),
+ N_(" filename '-'  reads from STDIN\n"),
+ N_(" PRINTER, LPDEST, NGPRINTER, NPRINTER environment variables set default printer.\n"),
 	0 };
 
 void prmsg( char **msg )
@@ -720,6 +728,7 @@ void usage(void)
 	} else {
 		prmsg( LPR_msg );
 	}
+	Parse_debug("=",-1);
 	FPRINTF( STDERR, "%s\n", Version );
 	exit(1);
 }
@@ -836,12 +845,13 @@ int Make_job( struct job *job )
 			struct line_list user_list;
 			char *str, *t;
 			struct passwd *pw;
-			int found, uid;
+			int found;
+			uid_t uid;
 
 			DEBUG2("Make_job: checking '%s' for -U perms",
 				Allow_user_setting_DYN );
 			Init_line_list(&user_list);
-			Split( &user_list, Allow_user_setting_DYN,File_sep,0,0,0,0,0);
+			Split( &user_list, Allow_user_setting_DYN,File_sep,0,0,0,0,0,0);
 			
 			found = 0;
 			for( i = 0; !found && i < user_list.count; ++i ){
@@ -1053,7 +1063,7 @@ void get_job_number( struct job *job )
 { 0,  STRING_K , &Prtitle_JOB, M_PRTITLE, "T" },
 { 0,  INTEGER_K , &Pwidth_JOB, M_PWIDTH, "W" },
 { 0,  STRING_K , &Zopts_JOB, M_ZOPTS, "Z" },
-{ 0 }
+{ 0,0,0,0,0 }
 } ;
 
 
@@ -1403,13 +1413,3 @@ void Check_dup(int option, int *value)
 {
 	*value = 1;
 }
-
- void Dispatch_input(int *talk, char *input ){}
-
-/*
- * Calls[] = list of dispatch functions 
- */
-
- struct call_list Calls[] = {
-	{0,0}
-};
