@@ -1,20 +1,20 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: lpstat.c,v 5.17 2000/12/25 01:51:12 papowell Exp papowell $";
+"$Id: lpstat.c,v 1.30 2002/05/06 01:06:41 papowell Exp $";
 
 
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
@@ -137,20 +137,9 @@ int main(int argc, char *argv[], char *envp[])
 	}
 
 	if( All_printers ){
-		Merge_line_list( &request_list, &All_line_list,0,1,1);
-	}
-	for( i = 0; i < Printer_list.count; ++i ){
-		s = Printer_list.list[i];
-		if( Find_key_in_list( &All_line_list, s, Value_sep, 0 ) == 0 ){
-			if( !All_printers ){
-				DEBUG1("lpstat: printer in option list '%s'", s);
-				Add_line_list( &request_list, s, 0, 1, 1 );
-			}
-		} else {
-			DEBUG1("lpstat: option in option list '%s'", s);
-			Add_line_list( &options, s, 0, 1, 1 );
-		}
+		Merge_line_list( &request_list, &All_line_list,0,0,0);
 	}
+	Merge_line_list( &request_list, &Printer_list,0,0,0);
 	Check_max(&options,2);
 	if( options.count ){
 		for( i = options.count; i > 0 ; --i ){
@@ -163,12 +152,12 @@ int main(int argc, char *argv[], char *envp[])
 
 	if( Found_flag == 0 ){
 		if( request_list.count == 0 ){
-			Split(&request_list,Printer_DYN,", ",1,0,1,1,0);
+			Split(&request_list,Printer_DYN,", ",1,0,1,1,0,0);
 		}
 		o_flag = 1;
 		flag_count = 1;
 	}
-	if(DEBUGL1)Dump_line_list("lpstat - printers", &request_list);
+	if(DEBUGL1)Dump_line_list("lpstat - printer request list", &request_list);
 	if(DEBUGL1)Dump_line_list("lpstat - options", &options);
 
 	if( r_flag ){
@@ -227,10 +216,8 @@ void Show_status(char **argv, int display_format)
 
 	DEBUG1("Show_status: start");
 	/* set up configuration */
-	Get_printer();
 	Fix_Rm_Rp_info(0,0);
 
-
 	if( Check_for_rg_group( Logname_DYN ) ){
 		SNPRINTF( msg, sizeof(msg))
 			"  Printer: %s - cannot use printer, not in privileged group\n", Printer_DYN );
@@ -240,10 +227,17 @@ void Show_status(char **argv, int display_format)
 	if( A_flag ){
 		Set_DYN(&Auth_DYN, getenv("AUTH"));
 	}
+	if( Direct_DYN && Lp_device_DYN ){
+		SNPRINTF( msg, sizeof(msg))
+			_(" Printer: %s - direct connection to device '%s'\n"),
+			Printer_DYN, Lp_device_DYN );
+		if(  Write_fd_str( 1, msg ) < 0 ) cleanup(0);
+		return;
+	}
 	fd = Send_request( 'Q', Displayformat,
 		0, Connect_timeout_DYN, Send_query_rw_timeout_DYN, 1 );
 	if( fd >= 0 ){
-		shutdown( fd, 1 );
+		/* shutdown( fd, 1 ); */
 		Read_status_info( RemoteHost_DYN, fd,
 			1, Send_query_rw_timeout_DYN, display_format,
 			Status_line_count );
@@ -287,7 +281,7 @@ int Read_status_info( char *host, int sock,
 	do {
 		DEBUG1("Read_status_info: look_for_pr %d, in buffer already '%s'", look_for_pr, buffer );
 		if( DEBUGL2 )Dump_line_list("Read_status_info - starting list", &l );
-		count = strlen(buffer);
+		count = safestrlen(buffer);
 		n = sizeof(buffer)-count-1;
 		status = 1;
 		if( n > 0 ){
@@ -306,8 +300,8 @@ int Read_status_info( char *host, int sock,
 		if( (s = safestrrchr(buffer,'\n')) ){
 			*s++ = 0;
 			/* add the lines */
-			Split(&l,buffer,Line_ends,0,0,0,0,0);
-			memmove(buffer,s,strlen(s)+1);
+			Split(&l,buffer,Line_ends,0,0,0,0,0,0);
+			memmove(buffer,s,safestrlen(s)+1);
 		}
 		if( DEBUGL2 )Dump_line_list("Read_status_info - status after splitting", &l );
 		if( status ){
@@ -329,59 +323,69 @@ int Read_status_info( char *host, int sock,
 				|| Find_exists_value(&Printer_list,t,0) ){
 				++index_list;
 			} else {
+				DEBUG2("Read_status_info: printer found [%d] '%s', total %d", index_list, s, l.count );
 				look_for_pr = 0;
 			}
 		}
 		while( index_list < l.count && (s = l.list[index_list]) ){
 			DEBUG2("Read_status_info: checking [%d] '%s', total %d", index_list, s, l.count );
 			if( s && !isspace(cval(s)) && (t = strstr(s,"Printer:")) ){
+				DEBUG2("Read_status_info: printer heading '%s'", t );
 				if( Find_exists_value(&Printer_list,t,0) ){
+					DEBUG2("Read_status_info: already done '%s'", t );
+					++index_list;
 					look_for_pr = 1;
 					goto again;
-				} else {
-					Add_line_list(&Printer_list,t,0,1,0);
-					/* parse the printer line */
-					if( display_format == 0 ){
-						char msg[SMALLBUFFER];
-						int nospool, noprint; 
-						nospool = (strstr( s, "(spooling disabled)") != 0);
-						noprint = (strstr( s, "(printing disabled)") != 0);
-						/* Write_fd_str( output, "ANALYZE " );
-						if( Write_fd_str( output, s ) < 0
-							|| Write_fd_str( output, "\n" ) < 0 ) return(1);
-						*/
-						if( a_flag ){
-							if( !nospool ){
-								SNPRINTF(msg,sizeof(msg))
-								"%s accepting requests since %s\n",
-								Printer_DYN, Time_str(0,0) );
-							} else {
-								SNPRINTF(msg,sizeof(msg))
-								"%s not accepting requests since %s -\n\tunknown reason\n",
-								Printer_DYN, Pretty_time(0) );
-							}
-							if( Write_fd_str( output, msg ) < 0 ) return(1);
-						}
-						if( p_flag ){
-							SNPRINTF(msg,sizeof(msg))
-							"printer %s unknown state. %s since %s. available\n",
-							Printer_DYN, noprint?"disabled":"enabled",
-							Pretty_time(0));
-							if( Write_fd_str( output, msg ) < 0 ) return(1);
-						}
-						if( p_flag && D_flag ){
-							SNPRINTF(msg,sizeof(msg))
-								"\tDescription: %s@%s\n",
-										RemotePrinter_DYN, RemoteHost_DYN ); 
-							if( Write_fd_str( output, msg ) < 0 ) return(1);
-						}
-					} else {
-						if( Write_fd_str( output, s ) < 0
-							|| Write_fd_str( output, "\n" ) < 0 ) return(1);
-					}
-					++index_list;
-					continue;
 				}
+				Add_line_list(&Printer_list,t,0,1,0);
+				/* parse the printer line */
+				if( (t = strchr(t, ':')) ){
+					++t;
+					while( isspace(cval(t)) ) ++t;
+					Set_DYN(&Printer_DYN,t );
+					for( t = Printer_DYN; !ISNULL(t) && !isspace(cval(t)); ++t );
+					if( isspace(cval(t)) ) *t = 0;
+				}
+				if( display_format == 0 ){
+					char msg[SMALLBUFFER];
+					int nospool, noprint; 
+					nospool = (strstr( s, "(spooling disabled)") != 0);
+					noprint = (strstr( s, "(printing disabled)") != 0);
+					/* Write_fd_str( output, "ANALYZE " );
+					if( Write_fd_str( output, s ) < 0
+						|| Write_fd_str( output, "\n" ) < 0 ) return(1);
+					*/
+					if( a_flag ){
+						if( !nospool ){
+							SNPRINTF(msg,sizeof(msg))
+							"%s accepting requests since %s\n",
+							Printer_DYN, Time_str(0,0) );
+						} else {
+							SNPRINTF(msg,sizeof(msg))
+							"%s not accepting requests since %s -\n\tunknown reason\n",
+							Printer_DYN, Time_str(0,0) );
+						}
+						if( Write_fd_str( output, msg ) < 0 ) return(1);
+					}
+					if( p_flag ){
+						SNPRINTF(msg,sizeof(msg))
+						"printer %s unknown state. %s since %s. available\n",
+						Printer_DYN, noprint?"disabled":"enabled",
+						Pretty_time(0));
+						if( Write_fd_str( output, msg ) < 0 ) return(1);
+					}
+					if( p_flag && D_flag ){
+						SNPRINTF(msg,sizeof(msg))
+							"\tDescription: %s@%s\n",
+									RemotePrinter_DYN, RemoteHost_DYN ); 
+						if( Write_fd_str( output, msg ) < 0 ) return(1);
+					}
+				} else {
+					if( Write_fd_str( output, s ) < 0
+						|| Write_fd_str( output, "\n" ) < 0 ) return(1);
+				}
+				++index_list;
+				continue;
 			}
 			if( display_format == 0 ){
 				++index_list;
@@ -400,7 +404,7 @@ int Read_status_info( char *host, int sock,
 				if( (s = strchr(header,':')) ){
 					*++s = 0;
 				}
-				len = strlen(header);
+				len = safestrlen(header);
 				/* find the last status_line_count lines */
 				same = 1;
 				for( i = index_list+1; i < l.count ; ++i ){
@@ -478,11 +482,11 @@ void Get_parms(int argc, char *argv[] )
 	Name = name;
 
 /*
-SYNOPSIS
+ SYNOPSIS
      lpstat [-A] [ -d ] [ -r ] [ -R ] [ -s ] [ -t ] [ -a [list] ]
           [ -c [list] ] [ -f [list] [ -l ] ] [ -o [list] ]
           [ -p [list] [ -D] [ -l ] ] [ -P ] [ -S [list] [ -l ] ]
-          [ -u [login-ID-list] ] [ -v [list] ] [-n linecount]
+          [ -u [login-ID-list] ] [ -v [list] ] [-n linecount] [-Tdebug]
 */
 	flag_count = 0;
 	for( i = 1; i < argc; ++i ){
@@ -511,7 +515,7 @@ SYNOPSIS
 			case 'u': ++flag_count; u_flag = 1; if( cval(s+2) ) Add_val(&u_val,s+2); else { i += Add_val(&u_val,argv[i+1]); } break;
 			case 'v':               v_flag = 1; if( cval(s+2) ) Add_val(&v_val,s+2); else { i += Add_val(&v_val,argv[i+1]); } break;
 			case 'V': Verbose = 1; break;
-			case 'T': Parse_debug( s+2, 1 ); break;
+			case 'T': if(!cval(s+2)) usage(); Parse_debug( s+2, 1 ); break;
 			default: usage(); break;
 
 			}
@@ -538,7 +542,7 @@ SYNOPSIS
 
 #undef SX
 #define SX(X,Y,Z) \
-	if((X)&&!(Y))Y="all"; Split(&Z,Y,", ",1,0,1,1,0);
+	if((X)&&!(Y))Y="all"; Split(&Z,Y,", ",1,0,1,1,0,0);
 	SX(a_flag,a_val,Printer_list);
 	SX(c_flag,c_val,Printer_list);
 	SX(f_flag,f_val,f_list);
@@ -583,7 +587,7 @@ SYNOPSIS
 "usage: %s [-A] [-d] [-l] [-r] [-R] [-s] [-t] [-a [list]]\n\
   [-c [list]] [-f [list]] [-o [list]]\n\
   [-p [list]] [-P] [-S [list]] [list]\n\
-  [-u [login-ID-list]] [-v [list]] [-V] [-n]\n\
+  [-u [login-ID-list]] [-v [list]] [-V] [-n] [-Tdbgflags]\n\
  list is a list of print queues\n\
  -A        use authentication specified by AUTH environment variable\n\
  -a [list] destination status *\n\
@@ -602,13 +606,15 @@ SYNOPSIS
  -u [joblist] job status information\n\
  -v [list] printer mapping *\n\
  -V        verbose mode \n\
- -Toptions diagnostic flags\n\
+ -Tdbgflags debug flags\n\
     * - long status format produced\n";
 
 
 void usage(void)
 {
 	FPRINTF( STDERR, lpstat_msg, Name );
+	Parse_debug("=",-1);
+	FPRINTF( STDOUT, "%s\n", Version );
 	exit(1);
 }
 
@@ -634,7 +640,7 @@ void usage(void)
 	cmd[1] = 0;
 	SNPRINTF(cmd+1, sizeof(cmd)-1, RemotePrinter_DYN);
 	for( i = 0; options[i]; ++i ){
-		n = strlen(cmd);
+		n = safestrlen(cmd);
 		SNPRINTF(cmd+n,sizeof(cmd)-n," %s",options[i] );
 	}
 	Perm_check.remoteuser = "papowell";
@@ -645,13 +651,3 @@ void usage(void)
 }
 
 #endif
-
- void Dispatch_input(int *talk, char *input ){}
-
-/*
- * Calls[] = list of dispatch functions 
- */
-
- struct call_list Calls[] = {
-	{0,0}
-};
