@@ -1,17 +1,16 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: sendauth.c,v 5.19 2000/12/25 01:51:14 papowell Exp papowell $";
+"$Id: sendauth.c,v 1.30 2002/05/06 01:06:41 papowell Exp $";
 
 #include "lp.h"
-#include "lpd.h"
 #include "sendauth.h"
 #include "sendjob.h"
 #include "globmatch.h"
@@ -147,7 +146,7 @@ int Send_auth_transfer( int *sock, int transfer_timeout,
 	/* send the message */
 	DEBUG3("Send_auth_transfer: sending '%s'", secure );
 	status = Link_send( RemoteHost_DYN, sock, transfer_timeout,
-		secure, strlen(secure), &ack );
+		secure, safestrlen(secure), &ack );
 	DEBUG3("Send_auth_transfer: status '%s'", Link_err_str(status) );
 	if( status ){
 		/* open output file */
@@ -179,9 +178,9 @@ int Send_auth_transfer( int *sock, int transfer_timeout,
 						tempfile );
 					goto error;
 				}
-				memmove(buffer,s,strlen(s)+1);
+				memmove(buffer,s,safestrlen(s)+1);
 			}
-			len = strlen(buffer);
+			len = safestrlen(buffer);
 		}
 		if( buffer[0] ){
 			DEBUG2("Send_auth_transfer: doing '%s'", buffer);
@@ -216,6 +215,7 @@ int Send_auth_transfer( int *sock, int transfer_timeout,
 		if( job ){
 			SETSTATUS(logjob)"Send_auth_transfer: %s", error );
 			Set_str_value(&job->info,ERROR,error);
+			Set_flag_value(&job->info,ERROR_TIME,time(0));
 		}
 		if( (fd = Checkwrite(tempfile,&statb,O_WRONLY|O_TRUNC,1,0)) < 0){
 			Errorcode = JFAIL;
@@ -402,9 +402,9 @@ int Pgp_send( int *sock, int transfer_timeout, char *tempfile,
 					SNPRINTF(error,errlen)"  %s\n", buffer);
 					Write_fd_str(tempfd, error );
 					DEBUG2("Pgp_send: wrote '%s'", error );
-					memmove(buffer,s,strlen(s)+1);
+					memmove(buffer,s,safestrlen(s)+1);
 				}
-				len = strlen(buffer);
+				len = safestrlen(buffer);
 			}
 			DEBUG2("Pgp_send: done" );
 			error[0] = 0;
@@ -432,7 +432,7 @@ int Pgp_send( int *sock, int transfer_timeout, char *tempfile,
 			SNPRINTF(error,errlen)"  %s\n", buffer);
 			Write_fd_str(tempfd, error );
 			DEBUG2("Pgp_send: wrote '%s'", error );
-			memmove(buffer,s,strlen(s)+1);
+			memmove(buffer,s,safestrlen(s)+1);
 		}
 		close( tempfd ); tempfd = -1;
 		error[0] = 0;
@@ -505,7 +505,7 @@ struct security *Fix_send_auth( char *name, struct line_list *info,
 		}
 		Set_str_value(info,FROM,from);
 		if( job ){
-			client = Find_str_value(&job->info,AUTHINFO,Value_sep);
+			client = Find_str_value(&job->info,AUTHUSER,Value_sep);
 			Set_str_value(info,CLIENT,client);
 		} else {
 			client = (char *)Perm_check.authuser;
@@ -590,15 +590,15 @@ void Put_in_auth( int tempfd, const char *key, char *value )
  struct security SendSecuritySupported[] = {
 	/* name,       config_tag, connect,    send,   receive */
 #if defined(KERBEROS)
-	{ "kerberos*", "kerberos", 0,           Krb5_send },
+	{ "kerberos*", "kerberos", 0,           Krb5_send, 0 },
 # if defined(MIT_KERBEROS4)
-	{ "kerberos4", "kerberos", Send_krb4_auth, 0 },
+	{ "kerberos4", "kerberos", Send_krb4_auth, 0, 0 },
 # endif
 #endif
-	{ "pgp",       "pgp",      0,           Pgp_send },
-	{ "none",      "none",     0,           0 },
+	{ "pgp",       "pgp",      0,           Pgp_send,  0 },
+	{ "none",      "none",     0,           0,         0 },
 #if defined(USER_SEND)
  USER_SEND
 #endif
-	{0}
+	{0,0,0,0,0}
 };
