@@ -1,14 +1,14 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: sendmail.c,v 5.12 2000/12/25 01:51:15 papowell Exp papowell $";
+"$Id: sendmail.c,v 1.30 2002/05/06 01:06:41 papowell Exp $";
 
 #include "lp.h"
 #include "errorcodes.h"
@@ -65,7 +65,7 @@ void Sendmail_to_user( int retval, struct job *job )
 		if( Write_fd_str( tempfd, buffer ) < 0 ) goto wr_error;
 	}
 	if( opname ){
-		n = strlen(msg);
+		n = safestrlen(msg);
 		SNPRINTF( msg+n, sizeof(msg)-n) "%s'%s'",n?" and ":"", opname );
 		SNPRINTF(buffer,sizeof(buffer))
 		"%s: %s\n", mailname?"CC":"To", opname );
