@@ -1,14 +1,14 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: sendreq.c,v 5.12 2000/12/25 01:51:15 papowell Exp papowell $";
+"$Id: sendreq.c,v 1.30 2002/05/06 01:06:41 papowell Exp $";
 
 
 #include "lp.h"
@@ -140,17 +140,24 @@ int Send_request(
 		if( status ) goto error;
 	} else {
 		sock = Link_open_list( RemoteHost_DYN,
-			&real_host, 0, connnect_timeout, 0 );
+			&real_host, 0, connnect_timeout, 0, Unix_socket_path_DYN );
 	}
 	err = errno;
 	if( sock < 0 ){
 		char *msg = "";
-		if( !Is_server ){
-			msg = "\nMake sure LPD server is running on the server";
-		}
 		SNPRINTF( errormsg, sizeof(errormsg)-2)
-			"cannot open connection - %s%s",
-			err?Errormsg(err):"bad or missing hostname", msg );
+			"cannot open connection - %s",
+			err?Errormsg(err):"bad or missing hostname" );
+		if( !Is_server ){
+			int v = safestrlen(errormsg);
+			SNPRINTF( errormsg+v, sizeof(errormsg)-v)
+			"\nMake sure the remote host supports the LPD protocol");
+			if( geteuid() && getuid() ){
+				v = safestrlen(errormsg);
+				SNPRINTF( errormsg+v, sizeof(errormsg)-v)
+				"\nand accepts connections from this host and from non-privileged (>1023) ports");
+			}
+		}
 		goto error;
 	}
 
@@ -165,7 +172,7 @@ int Send_request(
 			errormsg, sizeof(errormsg), cmd, security, &info );
 	} else {
 		status = Link_send( RemoteHost_DYN, &sock, transfer_timeout,
-			cmd, strlen(cmd), 0 );
+			cmd, safestrlen(cmd), 0 );
 		if( status ){
 			SNPRINTF(errormsg,sizeof(errormsg))"%s",Link_err_str(status));
 			close(sock); sock = -1;
