@@ -1,14 +1,14 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: stty.c,v 5.7 2000/12/25 01:51:15 papowell Exp papowell $";
+"$Id: stty.c,v 1.30 2002/05/06 01:06:42 papowell Exp $";
 
 
 #include "lp.h"
@@ -110,9 +110,9 @@
  *  such as "pass8" and "parity".
  */
 
-struct tchars termctrl;
-struct ltchars linectrl;
-struct sgttyb mode;
+ struct tchars termctrl;
+ struct ltchars linectrl;
+ struct sgttyb mode;
 
  static struct {
     char *string;
@@ -196,7 +196,7 @@ struct sgttyb mode;
 
     { "stop", &termctrl.t_stopc, CSTOP },
     { "start", &termctrl.t_startc, CSTART },
-    { 0 }
+    { 0, 0, 0 }
 };
 
 void Do_stty( int fd )
@@ -208,7 +208,7 @@ void Do_stty( int fd )
 	struct line_list l;
 
 	Init_line_list(&l);
-	Split(&l,Stty_command_DYN,Whitespace,0,0,0,0,0);
+	Split(&l,Stty_command_DYN,Whitespace,0,0,0,0,0,0);
 	Check_max(&l,1);
 	l.list[l.count] = 0;
 	DEBUG3("Do_stty: using SGTTYB, fd %d", fd );
@@ -347,7 +347,7 @@ void Do_stty( int fd )
     { (char *) 0, 0, 0 }
 };
 
-struct termio tio;
+ struct termio tio;
  static struct {
     char *string;
     int iset;
@@ -554,7 +554,7 @@ void Do_stty( int fd )
 	struct line_list l;
 
 	Init_line_list(&l);
-	Split(&l,Stty_command_DYN,Whitespace,0,0,0,0,0);
+	Split(&l,Stty_command_DYN,Whitespace,0,0,0,0,0,0);
 	Check_max(&l,1);
 	l.list[l.count] = 0;
 
@@ -677,7 +677,7 @@ void Do_stty( int fd )
 	{ (char *) 0, 0, 0 }
 };
 
-struct s_term_dat {
+ struct s_term_dat {
 	char *name;
 	uint or_dat;
 	uint and_dat;
@@ -901,10 +901,10 @@ struct s_term_dat {
 	{"-CTSXON", 0, CTSXON}
 };
 
-struct termiox tx_dat;
+ struct termiox tx_dat;
 #endif /* USE_TERMIOX */
 
-struct termios t_dat;
+ struct termios t_dat;
 
  static struct special {
 	char *name;
@@ -913,7 +913,7 @@ struct termios t_dat;
 
 	{ "stop", (char *)&(t_dat.c_cc[VSTOP]) },
 	{ "start", (char *)&(t_dat.c_cc[VSTART]) },
-	{ 0 }
+	{ 0, 0 }
 };
 
 void Do_stty( int fd )
@@ -926,7 +926,7 @@ void Do_stty( int fd )
 	struct line_list l;
 
 	Init_line_list(&l);
-	Split(&l,Stty_command_DYN,Whitespace,0,0,0,0,0);
+	Split(&l,Stty_command_DYN,Whitespace,0,0,0,0,0,0);
 	Check_max(&l,1);
 	l.list[l.count] = 0;
 
