@@ -1,14 +1,14 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: utilities.c,v 5.18 2000/12/25 01:51:16 papowell Exp papowell $";
+"$Id: utilities.c,v 1.30 2002/05/06 01:06:42 papowell Exp $";
 
 #include "lp.h"
 
@@ -162,13 +162,17 @@ int Write_fd_len( int fd, const char *msg, int len )
 int Write_fd_len_timeout( int timeout, int fd, const char *msg, int len )
 {
 	int i;
-	if( Set_timeout() ){
-		Set_timeout_alarm( timeout  );
-		i = Write_fd_len( fd, msg, len );
+	if( timeout > 0 ){
+		if( Set_timeout() ){
+			Set_timeout_alarm( timeout  );
+			i = Write_fd_len( fd, msg, len );
+		} else {
+			i = -1;
+		}
+		Clear_timeout();
 	} else {
-		i = -1;
+		i = Write_fd_len( fd, msg, len );
 	}
-	Clear_timeout();
 	return( i < 0 ? -1 : 0 );
 }
 
@@ -182,7 +186,7 @@ int Write_fd_len_timeout( int timeout, int fd, const char *msg, int len )
 int Write_fd_str( int fd, const char *msg )
 {
 	if( msg && *msg ){
-		return( Write_fd_len( fd, msg, strlen(msg) ));
+		return( Write_fd_len( fd, msg, safestrlen(msg) ));
 	}
 	return( 0 );
 }
@@ -198,7 +202,7 @@ int Write_fd_str( int fd, const char *msg )
 int Write_fd_str_timeout( int timeout, int fd, const char *msg )
 {
 	if( msg && *msg ){
-		return( Write_fd_len_timeout( timeout, fd, msg, strlen(msg) ) );
+		return( Write_fd_len_timeout( timeout, fd, msg, safestrlen(msg) ) );
 	}
 	return( 0 );
 }
@@ -215,13 +219,17 @@ int Write_fd_str_timeout( int timeout, int fd, const char *msg )
 int Read_fd_len_timeout( int timeout, int fd, char *msg, int len )
 {
 	int i;
-	if( Set_timeout() ){
-		Set_timeout_alarm( timeout  );
-		i = read( fd, msg, len );
+	if( timeout > 0 ){
+		if( Set_timeout() ){
+			Set_timeout_alarm( timeout  );
+			i = read( fd, msg, len );
+		} else {
+			i = -1;
+		}
+		Clear_timeout();
 	} else {
-		i = -1;
+		i = read( fd, msg, len );
 	}
-	Clear_timeout();
 	return( i );
 }
 
@@ -243,11 +251,13 @@ int Read_fd_len_timeout( int timeout, int fd, char *msg, int len )
  */
 
 /* plp_signal will set flags so that signal handlers will continue
- * note that in Solaris,  you MUST reinstall the
+ * Note that in Solaris,  you MUST reinstall the
  * signal hanlders in the signal handler!  The default action is
  * to try to restart the system call - note that the code should
- * be written so that you check for error returns, and continue
- * so this is merely a convenience.
+ * be written so that you check for error returns from a system call
+ * and continue if no error.
+ * WARNING: read/write may terminate early? who knows...
+ * See plp_signal_break.
  */
 
 plp_sigfunc_t plp_signal (int signo, plp_sigfunc_t func)
@@ -275,6 +285,9 @@ plp_sigfunc_t plp_signal (int signo, plp_sigfunc_t func)
  * TERMINATION of a system call if possible.  This allows
  * you to force a signal to cause termination of a system
  * wait or other action.
+ * WARNING: read/write may terminate early? who knows... so
+ * beware if you are expecting this and don't believe that
+ * you got an entire buffer read/written.
  */
 
 plp_sigfunc_t plp_signal_break (int signo, plp_sigfunc_t func)
@@ -428,6 +441,14 @@ int safestrcmp( const char *s1, const char *s2 )
 }
 
 
+/* perform safe comparison, even with null pointers */
+int safestrlen( const char *s1 )
+{
+	if( s1 ) return(strlen(s1));
+	return(0);
+}
+
+
 /* perform safe comparison, even with null pointers */
 int safestrncmp( const char *s1, const char *s2, int len )
 {
@@ -605,7 +626,7 @@ char *mystrncat( char *s1, const char *s2, int len )
 {
 	int size;
 	s1[len-1] = 0;
-	size = strlen( s1 );
+	size = safestrlen( s1 );
 	if( s2 && len - size > 0  ){
 		strncpy( s1+size, s2, len - size );
 	}
@@ -682,7 +703,7 @@ int Set_block_io( int fd )
  *     write data from to this fd from this buffer before this timeout
  *  int timeout
  *       > 0  - wait total of this long
- *       0    - wait indefinately
+ *       0    - wait indefinitely
  *      -1    - do not wait
  *  Returns:
  *   **outbuffer, *outlen updated
@@ -926,7 +947,7 @@ void Clear_timeout( void )
  * 
  * To_ruid( user );	-- set ruid to user, euid to root
  * To_euid( user );	-- set euid to user, ruid to root
- * To_root();	-- set euid to root, ruid to root
+ * To_euid_root();	-- set euid to root, ruid to root
  * To_daemon();	-- set euid to daemon, ruid to root
  * To_user();	-- set euid to user, ruid to root
  * Full_daemon_perms() -- set both UID and EUID, one way, no return
@@ -1015,47 +1036,53 @@ void Clear_timeout( void )
 #endif
 
 /***************************************************************************
- * setup_info()
- * 1. checks for the correct daemon uid
- * 2. checks to see if called (only needs to do this once)
- * 3. if UID 0 or EUID 0 forces both UID and EUID to 0 (test)
- * 4. Sets UID_root flag to indicate that we can change
+ * Setup_uid()
+ * 1. gets the original EUID, RUID, EGID, RGID
+ * 2. if UID 0 or EUID 0 forces both UID and EUID to 0 (test)
+ * 3. Sets the EUID to the original RUID
+ *    This leaves the UID (EUID) 
  ***************************************************************************/
 
- static void setup_info(void)
+void Setup_uid(void)
 {
 	int err = errno;
 	static int SetRootUID;	/* did we set UID to root yet? */
 
-	DaemonUID = Getdaemon(); /* do this each time in case we change it */
 	if( SetRootUID == 0 ){
 		OriginalEUID = geteuid();	
 		OriginalRUID = getuid();	
-		DEBUG1("setup_info: OriginalEUD %d, OriginalRUID %d",
+		OriginalEGID = getegid();	
+		OriginalRGID = getgid();	
+		DEBUG1("Setup_uid: OriginalEUID %d, OriginalRUID %d",
 			OriginalEUID, OriginalRUID );
+		DEBUG1("Setup_uid: OriginalEGID %d, OriginalRGID %d",
+			OriginalEGID, OriginalRGID );
 		/* we now make sure that we are able to use setuid() */
 		/* notice that setuid() will work if EUID or RUID is 0 */
-		if( OriginalEUID == 0 || OriginalRUID == 0 ){
+		if( OriginalEUID == ROOTUID || OriginalRUID == ROOTUID ){
 			/* set RUID/EUID to ROOT - possible if EUID or UID is 0 */
 			if(
 #				ifdef HAVE_SETEUID
-					setuid( (uid_t)0 ) || seteuid( (uid_t)0 )
+					setuid( (uid_t)ROOTUID ) || seteuid( OriginalRUID )
 #				else
-					setuid( (uid_t)0 ) || setreuid( 0, 0 )
+					setuid( (uid_t)ROOTUID ) || setreuid( ROOTUID, OriginalRUID )
 #				endif
 				){
 				FATAL(LOG_ERR)
-					"setup_info: RUID/EUID Start %d/%d seteuid failed",
+					"Setup_uid: RUID/EUID Start %d/%d seteuid failed",
 					OriginalRUID, OriginalEUID);
 			}
-			if( getuid() || geteuid() ){
+			if( getuid() != ROOTUID ){
 				FATAL(LOG_ERR)
-				"setup_info: IMPOSSIBLE! RUID/EUID Start %d/%d, now %d/%d",
+				"Setup_uid: IMPOSSIBLE! RUID/EUID Start %d/%d, now %d/%d",
 					OriginalRUID, OriginalEUID, 
 					getuid(), geteuid() );
 			}
 			UID_root = 1;
 		}
+		DEBUG1( "Setup_uid: Original RUID/EUID %d/%d, RUID/EUID %d/%d",
+					OriginalRUID, OriginalEUID, 
+					getuid(), geteuid() );
 		SetRootUID = 1;
 	}
 	errno = err;
@@ -1067,7 +1094,7 @@ void Clear_timeout( void )
  * 2. check to see if you need to do anything
  * 3. check to make sure you can
  ***************************************************************************/
- static int seteuid_wrapper( int to )
+ static int seteuid_wrapper( uid_t to )
 {
 	int err = errno;
 	uid_t euid;
@@ -1078,7 +1105,7 @@ void Clear_timeout( void )
 		OriginalRUID, OriginalEUID, DaemonUID, UID_root );
 	if( UID_root ){
 		/* be brutal: set both to root */
-		if( setuid( 0 ) ){
+		if( setuid( ROOTUID ) ){
 			LOGERR_DIE(LOG_ERR)
 			"seteuid_wrapper: setuid() failed!!");
 		}
@@ -1088,7 +1115,7 @@ void Clear_timeout( void )
 			"seteuid_wrapper: seteuid() failed!!");
 		}
 #else
-		if( setreuid( 0, to) ){
+		if( setreuid( ROOTUID, to) ){
 			LOGERR_DIE(LOG_ERR)
 			"seteuid_wrapper: setreuid() failed!!");
 		}
@@ -1107,7 +1134,7 @@ void Clear_timeout( void )
  * 2. check to see if you need to do anything
  * 3. check to make sure you can
  ***************************************************************************/
- static int setruid_wrapper( int to )
+ static int setruid_wrapper( uid_t to )
 {
 	int err = errno;
 	uid_t ruid;
@@ -1118,7 +1145,7 @@ void Clear_timeout( void )
 		OriginalRUID, OriginalEUID, DaemonUID, UID_root );
 	if( UID_root ){
 		/* be brutal: set both to root */
-		if( setuid( 0 ) ){
+		if( setuid( ROOTUID ) ){
 			LOGERR_DIE(LOG_ERR)
 			"setruid_wrapper: setuid() failed!!");
 		}
@@ -1128,10 +1155,15 @@ void Clear_timeout( void )
 			"setruid_wrapper: setruid() failed!!");
 		}
 #elif defined(HAVE_SETREUID)
-		if( setreuid( to, 0) ){
+		if( setreuid( to, ROOTUID) ){
 			LOGERR_DIE(LOG_ERR)
 			"setruid_wrapper: setreuid() failed!!");
 		}
+#elif defined(__CYGWIN__)
+		if( seteuid( to ) ){
+			LOGERR_DIE(LOG_ERR)
+			"setruid_wrapper: seteuid() failed!!");
+		}
 #else
 # error - you do not have a way to set ruid
 #endif
@@ -1146,27 +1178,42 @@ void Clear_timeout( void )
 /*
  * Superhero functions - change the EUID to the requested one
  *  - these are really idiot level,  as all of the tough work is done
- * in setup_info() and seteuid_wrapper() 
+ * in Setup_uid() and seteuid_wrapper() 
+ *  We also change the groups, just to be nasty as well, except for
+ *  To_ruid and To_uid,  which only does the RUID and EUID
+ *    Sigh...  To every rule there is an exception.
  */
-int To_root(void)
+int To_euid_root(void)
 {
-	setup_info(); return( seteuid_wrapper( 0 )	);
+	return( seteuid_wrapper( ROOTUID )	);
 }
+
+ static int To_daemon_called;
+
 int To_daemon(void)
 {
-	setup_info(); return( seteuid_wrapper( DaemonUID )	);
+	Set_full_group( DaemonUID, DaemonGID );
+	To_daemon_called = 1;
+	return( seteuid_wrapper( DaemonUID )	);
 }
+
 int To_user(void)
 {
-	setup_info(); return( seteuid_wrapper( OriginalRUID )	);
+	if( To_daemon_called ){
+		Errorcode = JABORT;
+		LOGMSG(LOG_ERR) "To_user: LOGIC ERROR! To_daemon has been called");
+		abort();
+	}
+	/* Set_full_group( OriginalRUID, OriginalRGID ); */
+	return( seteuid_wrapper( OriginalRUID )	);
 }
-int To_ruid(int uid)
+int To_ruid(int ruid)
 {
-	setup_info(); return( setruid_wrapper( uid )	);
+	return( setruid_wrapper( ruid )	);
 }
-int To_uid( int uid )
+int To_euid( int euid )
 {
-	setup_info(); return( seteuid_wrapper( uid ) );
+	return( seteuid_wrapper( euid ) );
 }
 
 /*
@@ -1174,13 +1221,13 @@ int To_uid( int uid )
  * This is unrecoverable!
  */
 
-int setuid_wrapper(int to)
+int setuid_wrapper(uid_t to)
 {
 	int err = errno;
 	if( UID_root ){
 		/* Note: you MUST use setuid() to force saved_setuid correctly */
-		if( setuid( (uid_t)0 ) ){
-			LOGERR_DIE(LOG_ERR) "setuid_wrapper: setuid(0) failed!!");
+		if( setuid( (uid_t)ROOTUID ) ){
+			LOGERR_DIE(LOG_ERR) "setuid_wrapper: setuid(ROOTUID) failed!!");
 		}
 		if( setuid( (uid_t)to ) ){
 			LOGERR_DIE(LOG_ERR) "setuid_wrapper: setuid(%d) failed!!", to);
@@ -1194,15 +1241,23 @@ int setuid_wrapper(int to)
 
 int Full_daemon_perms(void)
 {
-	setup_info(); return(setuid_wrapper(DaemonUID));
+	Setup_uid();
+	Set_full_group( DaemonUID, DaemonGID );
+	return(setuid_wrapper(DaemonUID));
 }
+
 int Full_root_perms(void)
 {
-	setup_info(); return(setuid_wrapper( 0 ));
+	Setup_uid();
+	Set_full_group( ROOTUID, ROOTUID );
+	return(setuid_wrapper( ROOTUID ));
 }
+
 int Full_user_perms(void)
 {
-	setup_info(); return(setuid_wrapper(OriginalRUID));
+	Setup_uid();
+	Set_full_group( OriginalRUID, OriginalRGID );
+	return(setuid_wrapper(OriginalRUID));
 }
 
 
@@ -1232,7 +1287,7 @@ int Getdaemon(void)
 		}
 	}
 	DEBUG4( "Getdaemon: uid '%d'", uid );
-	if( uid == 0 ) uid = getuid();
+	if( uid == ROOTUID ) uid = getuid();
 	DEBUG4( "Getdaemon: final uid '%d'", uid );
 	return( uid );
 }
@@ -1276,32 +1331,59 @@ int Getdaemon_group(void)
  * 3. set the RGID/EGID
  ***************************************************************************/
 
-int Setdaemon_group(void)
+int Set_full_group( int euid, int gid )
 {
-	uid_t euid;
-	int status;
+	int status=0;
 	int err;
+	struct passwd *pw = 0;
 
-	DaemonGID = Getdaemon_group();
-	DEBUG4( "Setdaemon_group: set '%d'", DaemonGID );
+	DEBUG4( "Set_full_group: euid '%d'", euid );
+
+	/* get the user we want to set the groups for */
+	pw = getpwuid(euid);
 	if( UID_root ){
-		euid = geteuid();
-		To_root();	/* set RUID/EUID to root */
-		status = setgid( DaemonGID );
-		err = errno;
-		if( To_uid( euid ) ){
+		setuid(ROOTUID);	/* set RUID/EUID to root */
+#if defined(HAVE_INITGROUPS)
+		if( pw ){
+			/* froth froth... initgroups() uses the same buffer as
+			 * getpwuid... SHRIEK  so we need to copy the user name
+			 */
+			char user[256];
+			safestrncpy(user,pw->pw_name);
+			if( safestrlen(user) != safestrlen(pw->pw_name) ){
+				FATAL(LOG_ERR) "Set_full_group: CONFIGURATION BOTCH! safestrlen of user name '%s' = %d larger than buffer size %d",
+					pw->pw_name, safestrlen(pw->pw_name), sizeof(user) );
+			}
+			if( initgroups(user, pw->pw_gid ) == -1 ){
+				err = errno;
+				LOGERR_DIE(LOG_ERR) "Set_full_group: initgroups failed '%s'",
+					Errormsg( err ) );
+			}
+		} else
+#endif
+#if defined(HAVE_SETGROUPS)
+			if( setgroups(0,0) == -1 ){
+				err = errno;
+				LOGERR_DIE(LOG_ERR) "Set_full_group: setgroups failed '%s'",
+					Errormsg( err ) );
+			}
+#endif
+		status = setgid( gid );
+		if( status < 0 ){
 			err = errno;
-			LOGERR_DIE(LOG_ERR) "setdaemon_group: To_uid '%d' failed '%s'",
-				euid, Errormsg( err ) );
-		}
-		if( status < 0 || DaemonGID != getegid() ){
-			LOGERR_DIE(LOG_ERR) "setdaemon_group: setgid '%d' failed '%s'",
-			DaemonGID, Errormsg( err ) );
+			LOGERR_DIE(LOG_ERR) "Set_full_group: setgid '%d' failed '%s'",
+				gid, Errormsg( err ) );
 		}
 	}
 	return( 0 );
 }
 
+int Setdaemon_group(void)
+{
+	Set_full_group( DaemonUID, DaemonGID );
+	return( 0 );
+}
+
 
 /*
  * Testing magic:
@@ -1319,14 +1401,14 @@ void Reset_daemonuid(void)
 {
 	uid_t uid;
     uid = Getdaemon();  /* get the config file daemon id */
+    DaemonGID = Getdaemon_group();  /* get the config file daemon id */
     if( uid != DaemonUID ){
-        if( uid == 0 ){
+        if( uid == ROOTUID ){
             DaemonUID = OriginalRUID;   /* special case for testing */
         } else {
             DaemonUID = uid;
         }
     }
-	To_daemon();        /* now we are running with desired UID */
     DEBUG4( "DaemonUID %d", DaemonUID );
 }
 
