@@ -1,10 +1,10 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
- * $Id: getqueue.h,v 5.19 2000/12/25 01:51:18 papowell Exp papowell $
+ * $Id: getqueue.h,v 1.30 2002/05/06 01:06:43 papowell Exp $
  ***************************************************************************/
 
 
@@ -22,8 +22,6 @@ EXTERN const char * ALL					DEFINE( = "all" );
 EXTERN const char * ATTEMPT				DEFINE( = "attempt" );
 EXTERN const char * AUTH				DEFINE( = "auth" );
 EXTERN const char * AUTHFROM			DEFINE( = "authfrom" );
-EXTERN const char * AUTHFROM_ID			DEFINE( = "authfrom_id" );
-EXTERN const char * AUTHINFO			DEFINE( = "_" );
 EXTERN const char * AUTHTYPE			DEFINE( = "authtype" );
 EXTERN const char * AUTHUSER			DEFINE( = "authuser" );
 EXTERN const char * AUTOHOLD			DEFINE( = "autohold" );
@@ -48,9 +46,11 @@ EXTERN const char * DF_NAME				DEFINE( = "df_name" );
 EXTERN const char * DMALLOC_OPTIONS		DEFINE( = "DMALLOC_OPTIONS" );
 EXTERN const char * DMALLOC_OUTFILE		DEFINE( = "dmalloc_outfile" );
 EXTERN const char * DONE_TIME			DEFINE( = "done_time" );
+EXTERN const char * DONE_REMOVE			DEFINE( = "done_remove" );
 EXTERN const char * DUMP				DEFINE( = "dump" );
 EXTERN const char * END					DEFINE( = "end" );
 EXTERN const char * ERROR				DEFINE( = "error" );
+EXTERN const char * ERROR_TIME			DEFINE( = "error_time" );
 EXTERN const char * ESC_ID				DEFINE( = "esc_id" );
 EXTERN const char * FILENAMES			DEFINE( = "filenames" );
 EXTERN const char * FILE_HOSTNAME		DEFINE( = "file_hostname" );
@@ -72,6 +72,7 @@ EXTERN const char * HOST				DEFINE( = "host" );
 EXTERN const char * HPFORMAT			DEFINE( = "hpformat" );
 EXTERN const char * ID					DEFINE( = "id" );
 EXTERN const char * IDENTIFIER			DEFINE( = "A" );
+EXTERN const char * INCOMING_TIME		DEFINE( = "incoming_time" );
 EXTERN const char * INPUT				DEFINE( = "input" );
 EXTERN const char * JOBNAME				DEFINE( = "J" );
 EXTERN const char * JOBSEQ				DEFINE( = "jobseq" );
@@ -86,7 +87,6 @@ EXTERN const char * LOGNAME				DEFINE( = "P" );
 EXTERN const char * LP					DEFINE( = "lp" );
 EXTERN const char * LPC					DEFINE( = "lpc" );
 EXTERN const char * LPD					DEFINE( = "lpd" );
-EXTERN const char * LPD_ACK_FD			DEFINE( = "lpd_ack_fd" );
 EXTERN const char * LPD_CONF			DEFINE( = "LPD_CONF" );
 EXTERN const char * LPD_PORT			DEFINE( = "lpd_port" );
 EXTERN const char * LPD_REQUEST			DEFINE( = "lpd_request" );
@@ -116,6 +116,8 @@ EXTERN const char * QUEUENAME			DEFINE( = "Q" );
 EXTERN const char * QUEUE_CONTROL_FILE	DEFINE( = "queue_control_file" );
 EXTERN const char * QUEUE_STATUS_FILE	DEFINE( = "queue_status_file" );
 EXTERN const char * REDIRECT			DEFINE( = "redirect" );
+EXTERN const char * REMOTEHOST			DEFINE( = "remotehost" );
+EXTERN const char * REMOTEPORT			DEFINE( = "remoteport" );
 EXTERN const char * REMOTEUSER			DEFINE( = "remoteuser" );
 EXTERN const char * REMOVE_TIME			DEFINE( = "remove_time" );
 EXTERN const char * SD					DEFINE( = "sd" );
@@ -134,6 +136,8 @@ EXTERN const char * STATUS_FD			DEFINE( = "status_fd" );
 EXTERN const char * SUBSERVER			DEFINE( = "subserver" );
 EXTERN const char * TRACE				DEFINE( = "trace" );
 EXTERN const char * TRANSFERNAME		DEFINE( = "transfername" );
+EXTERN const char * OTRANSFERNAME		DEFINE( = "otransfername" );
+EXTERN const char * UNIXSOCKET			DEFINE( = "unixsocket" );
 EXTERN const char * UPDATE				DEFINE( = "update" );
 EXTERN const char * UPDATE_TIME			DEFINE( = "update_time" );
 EXTERN const char * USER				DEFINE( = "user" );
@@ -142,7 +146,7 @@ EXTERN const char * VALUE				DEFINE( = "value" );
 /* PROTOTYPES */
 int Scan_queue( struct line_list *spool_control,
 	struct line_list *sort_order, int *pprintable, int *pheld, int *pmove,
-		int only_pr_and_move, int create_db, int write_db );
+		int only_queue_process, int *perr, int *pdone );
 char *Get_fd_image( int fd, int maxsize );
 char *Get_file_image( const char *file, int maxsize );
 int Get_fd_image_and_split( int fd,
@@ -156,28 +160,27 @@ int Get_file_image_and_split( const char *file,
 	int sort, const char *keysep, int uniq, int trim, int nocomments,
 	char **return_image );
 void Check_for_hold( struct job *job, struct line_list *spool_control );
-void Setup_job( struct job *job, struct line_list *spool_control, const char *dir,
+void Setup_job( struct job *job, struct line_list *spool_control,
 	const char *cf_name, const char *hf_name, int check_for_existence );
 int Get_hold_class( struct line_list *info, struct line_list *sq );
 void Append_Z_value( struct job *job, char *s );
 int Setup_cf_info( struct job *job, int check_for_existence );
 char *Make_hf_image( struct job *job );
-int Set_hold_file( struct job *job, struct line_list *perm_check );
+int Set_hold_file( struct job *job, struct line_list *perm_check, int fd );
 void Get_hold_file( struct job *job, char *hf_name );
-void Remove_from_db( char *openname );
 void Get_spool_control( const char *file, struct line_list *info );
 void Set_spool_control( struct line_list *perm_check, const char *file,
 	struct line_list *info );
-char *intval( const char *key, struct line_list *list, char *cmpstr );
-char * revintval( const char *key, struct line_list *list, char *cmpstr );
-char * strzval( const char *key, struct line_list *list, char *cmpstr );
-char * strnzval( const char *key, struct line_list *list, char *cmpstr );
-char * strval( const char *key, struct line_list *list, char *cmpstr,
+void intval( const char *key, struct line_list *list, struct job *job );
+void revintval( const char *key, struct line_list *list, struct job *job );
+void strzval( const char *key, struct line_list *list, struct job *job );
+void strnzval( const char *key, struct line_list *list, struct job *job );
+void strval( const char *key, struct line_list *list, struct job *job,
 	int reverse );
-char *Make_sort_key( struct job *job );
+void Make_sort_key( struct job *job );
 int Setup_printer( char *prname, char *error, int errlen, int subserver );
 int Read_pid( int fd, char *str, int len );
-void Write_pid( int fd, int pid, char *str );
+int Write_pid( int fd, int pid, char *str );
 int Patselect( struct line_list *token, struct line_list *cf, int starting );
 int Check_format( int type, const char *name, struct job *job );
 char *Find_start(char *str, const char *key );
@@ -196,7 +199,7 @@ char *Fix_job_number( struct job *job, int n );
 char *Make_identifier( struct job *job );
 void Dump_job( char *title, struct job *job );
 void Job_printable( struct job *job, struct line_list *spool_control,
-	int *pprintable, int *pheld, int *pmove );
+	int *pprintable, int *pheld, int *pmove, int *perr, int *pdone );
 int Server_active( char *file );
 void Update_destination( struct job *job );
 int Get_destination( struct job *job, int n );
@@ -206,13 +209,11 @@ char *Fix_datafile_info( struct job *job, char *number, char *suffix,
 	char *xlate_format );
 int ordercomp(  const void *left, const void *right, const void *orderp);
 void Fix_control( struct job *job, char *filter, char *xlate_format );
-int Create_control( struct job *job, char *error, int errlen, char *auth_id,
+int Create_control( struct job *job, char *error, int errlen,
 	char *xlate_format );
 void Init_buf(char **buf, int *max, int *len);
 void Put_buf_len( const char *s, int cnt, char **buf, int *max, int *len );
 void Put_buf_str( const char *s, char **buf, int *max, int *len );
 void Free_buf(char **buf, int *max, int *len);
-int Open_gdbm( int db_create );
-void Close_gdbm(void);
 
 #endif
