@@ -1,10 +1,10 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
- * $Id: lp.h,v 5.27 2000/12/25 01:51:19 papowell Exp papowell $
+ * $Id: lp.h,v 1.30 2002/05/06 01:06:43 papowell Exp $
  ***************************************************************************/
 
 
@@ -87,11 +87,15 @@ struct security {
  * while in principle we could segragate these, it is not worth it
  * as the number is relatively small
  *****************************************************************/
+EXTERN int Optind;			/* next argv to process */
+EXTERN int Opterr DEFINE(=1); /* Zero disables errors msgs */
+EXTERN char *Optarg;		/* Pointer to option argument */
+EXTERN char *Name;			/* Name of program */
 
 extern char *Copyright[];	/* Copyright info */
-extern char *Name;			/* Program name */
 #define Version	Copyright[0]
 EXTERN int Is_server;		/* LPD sets to non-zero */
+EXTERN int Server_pid;		/* PID of server */
 EXTERN int Is_lpr;			/* LPR sets to non-zero */
 EXTERN int Logger_fd;		/* for logger */
 EXTERN int Mail_fd;			/* for mail */
@@ -104,6 +108,7 @@ EXTERN int Status_fd;		/* Status file descriptor for spool queue */
 EXTERN char *Outbuf, *Inbuf;	/* buffer */
 EXTERN int Outlen, Outmax, Inlen, Inmax;	/* max and current len of buffer */
 EXTERN uid_t OriginalEUID, OriginalRUID;   /* original EUID, RUID values */
+EXTERN uid_t OriginalEGID, OriginalRGID;   /* original EGID, RGID values */
 EXTERN uid_t DaemonUID;    /* Daemon UID */
 EXTERN uid_t UID_root;     /* UID is root */
 EXTERN gid_t DaemonGID;    /* Daemon GID */
@@ -111,12 +116,16 @@ EXTERN int Max_fd DEFINE(=4);	   /* Maximum FD opened */
 
 extern void Max_open(int fd);
 
+
+#if 0
+typepdef void (*WorkerProc)( struct line_list *args );
 struct call_list{
-	const char * *id;
+	const char *id;
 	void (*p)( struct line_list *args );
 };
-
 extern struct call_list Calls[];
+#endif
+
 
 
 #ifdef HAVE_STDARGS
@@ -308,6 +317,8 @@ EXTERN char* Default_priority_DYN;	/* default priority */
 EXTERN char* Default_remote_host_DYN;
 EXTERN char* Default_tmp_dir_DYN;	/* default temporary file directory */
 EXTERN char* Destinations_DYN; /* printers that a route filter may return and we should query */
+EXTERN int   Done_jobs_DYN;        /* keep the last NN done jobs */
+EXTERN int   Done_jobs_max_age_DYN; /* keep the done jobs for at least max age seconds */
 EXTERN int Direct_DYN;		/* allow LPR to send jobs to a socket */
 EXTERN int Exit_linger_timeout_DYN;	/* we set this timeout on all of the sockets */
 EXTERN int FF_on_close_DYN; /* print a form feed when device is closed */
@@ -320,6 +331,7 @@ EXTERN char* Filter_path_DYN;
 EXTERN int Fake_large_file_DYN; 	/* fake large file size if you cannot use 0 */
 EXTERN int Filter_poll_interval_DYN; /* intervals at which to check filter */
 EXTERN int Force_FQDN_hostname_DYN; /* force FQDN Host name in control file */
+EXTERN int Force_IPADDR_hostname_DYN; /* force IPADDR for Host name in control file */
 EXTERN int Force_localhost_DYN;	/* force localhost for client job transfer */
 EXTERN char* Force_lpq_status_DYN;	/* force lpq status format */
 EXTERN int Force_poll_DYN; /* force polling job queues */
@@ -327,6 +339,7 @@ EXTERN char* Force_queuename_DYN; /* force the use of this queue name */
 EXTERN char* Form_feed_DYN; /* string to send for a form feed */
 EXTERN char* Formats_allowed_DYN; /* valid output filter formats */
 EXTERN int Full_time_DYN; /* full or complete time format in messages */
+EXTERN int Half_close_DYN; /* do shutdown on socket when sending job */
 EXTERN int Generate_banner_DYN; /* generate a banner when not a bounce queue */
 EXTERN char* IF_Filter_DYN; /* filter command, run on a per-file basis */
 EXTERN int IPV6Protocol_DYN;	/* IPV4 or IPV6 protocol */
@@ -356,8 +369,9 @@ EXTERN int Long_number_DYN; /* long job number (6 digits) */
 EXTERN int Max_datafiles_DYN; /* maximum datafiles */
 EXTERN int Lpd_bounce_DYN; /* force LPD to do bounce queue filtering */
 EXTERN char* Lp_device_DYN; /* device name or lp-pipe command to send output to */
+EXTERN char* Lpd_listen_port_DYN; /* lpd listens on this port, "off" does not open port */
 EXTERN char* Lpd_path_DYN; /* LPD path for server use */
-EXTERN char* Lpd_port_DYN;
+EXTERN char* Lpd_port_DYN;	/* client/lpd connect to remote (non-local) lpd servers on this port */
 EXTERN char* Lpd_printcap_path_DYN;
 EXTERN int Lpr_bounce_DYN; /* allow LPR to do bounce queue filtering */
 EXTERN char* Lpq_status_file_DYN; /* cached lpq status */
@@ -369,10 +383,12 @@ EXTERN char* Mail_operator_on_error_DYN;
 EXTERN int Max_connect_interval_DYN;	/* maximum connect interval */
 EXTERN int Max_copies_DYN; /* maximum copies allowed */
 EXTERN int Max_job_size_DYN; /* maximum job size (1Kb blocks, 0 = unlimited) */
+EXTERN int Max_accounting_file_size_DYN;	/* maximum accounting file size */
 EXTERN int Max_log_file_size_DYN;	/* maximum log file size */
 EXTERN int Max_servers_active_DYN;	/* maximum number of servers active */
 EXTERN int Max_status_line_DYN; /* maximum status line size */
 EXTERN int Max_status_size_DYN;
+EXTERN int Min_accounting_file_size_DYN;	/* minimum accounting file size */
 EXTERN int Min_log_file_size_DYN;	/* minimum log file size */
 EXTERN int Min_printable_count_DYN; /* minimum printable characters for printable check */
 EXTERN int Min_status_size_DYN;
@@ -394,6 +410,8 @@ EXTERN int Page_x_DYN; /* page width in pixels (horizontal) */
 EXTERN int Page_y_DYN; /* page length in pixels (vertical) */
 EXTERN char* Pass_env_DYN;		/* pass these environment variables */
 EXTERN int Poll_time_DYN; /* force polling job queues */
+EXTERN int Poll_start_interval_DYN; /* interval between trying to start servers */
+EXTERN int Poll_servers_started_DYN; /* maximum servers to start at one time */
 EXTERN char* Pr_program_DYN; /* pr program for p format */
 EXTERN char* Prefix_Z_DYN; /* prefix -Z options on outgoing or filter*/
 EXTERN char* Prefix_option_to_option_DYN; /* prefix option to option, ie, "z,o" */
@@ -406,7 +424,6 @@ EXTERN char* Queue_control_file_DYN; /* Queue control file name */
 EXTERN char* Queue_lock_file_DYN; /* Queue lock file name */
 EXTERN char* Queue_status_file_DYN; /* Queue status file name */
 EXTERN char* Queue_unspooler_file_DYN; /* Unspooler PID status file name */
-EXTERN char* Queue_db_file_DYN; /* Queue DB file name */
 EXTERN int Read_write_DYN; /* open the printer for reading and writing */
 EXTERN char* RemoteHost_DYN; /* remote-queue machine (hostname) (with rm) */
 EXTERN char* RemotePrinter_DYN; /* remote-queue printer name (with rp) */
@@ -414,6 +431,8 @@ EXTERN char* Remote_support_DYN; /* Operations allowed to remote system */
 EXTERN char* Remove_Z_DYN; /* remove -Z options on outgoing or filter*/
 EXTERN char* Report_server_as_DYN; /* report server name as this value */
 EXTERN int Require_configfiles_DYN; /* require lpd.conf, printcap, lpd.perms files */
+EXTERN int Require_explicit_Q_DYN; /* require default queue to be explicitly defined */
+EXTERN char* RestrictToGroupMembers_DYN;	/* restrict to group members */
 EXTERN int Retry_ECONNREFUSED_DYN; /* retry on ECONNREFUSED  */
 EXTERN int Retry_NOLINK_DYN; /* retry on link connection failure */
 EXTERN char* Return_short_status_DYN;	/* return short status */
@@ -451,8 +470,7 @@ EXTERN int Suppress_header_DYN; /* suppress headers and/or banner page */
 EXTERN int Suspend_OF_filter_DYN; /* suspend OF filter */
 EXTERN char* Syslog_device_DYN;	/* default syslog() facility */
 EXTERN char* Trailer_on_close_DYN; /* trailer string to print when queue empties */
-EXTERN int Use_date_DYN;		/* put date in control file */
-EXTERN int Use_identifier_DYN;	/* put identifier in control file */
+EXTERN char* Unix_socket_path_DYN;	/* UNIX socket pathname */
 EXTERN int Use_info_cache_DYN;
 EXTERN int Use_queuename_DYN;	/* put queuename in control file */
 EXTERN int Use_queuename_flag_DYN;	/* Specified with the -Q option */
