@@ -1,10 +1,10 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
- * $Id: lpd_jobs.h,v 5.11 2000/12/25 01:51:20 papowell Exp papowell $
+ * $Id: lpd_jobs.h,v 1.30 2002/05/06 01:06:43 papowell Exp $
  ***************************************************************************/
 
 
@@ -14,8 +14,8 @@
 
 /* PROTOTYPES */
 void Update_spool_info( struct line_list *sp );
-int cmp_server( const void *left, const void *right, const void *arg );
-int Get_subserver_pc( char *printer, struct line_list *subserver_info, int done_time );
+int cmp_server( const void *left, const void *right, const void *p );
+void Get_subserver_pc( char *printer, struct line_list *subserver_info, int done_time );
 void Dump_subserver_info( char *title, struct line_list *l);
 void Get_subserver_info( struct line_list *order,
 	char *list, char *old_order);
@@ -36,6 +36,9 @@ int Printer_open( char *lp_device, int *status_fd, struct job *job,
 	int max_attempts, int interval, int max_interval, int grace,
 	int connect_tmout, int *filterpid, int *poll_for_status );
 void Add_banner_to_job( struct job *job );
+void Fix_bq_format( int format, struct line_list *datafile );
 void Filter_files_in_job( struct job *job, int outfd, char *user_filter );
+void Service_queue( struct line_list *args );
+int Remove_done_jobs( void );
 
 #endif
