@@ -1,10 +1,10 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
- * $Id: lpr.h,v 5.9 2000/12/25 01:51:21 papowell Exp papowell $
+ * $Id: lpr.h,v 1.30 2002/05/06 01:06:43 papowell Exp $
  ***************************************************************************/
 
 
@@ -47,10 +47,11 @@ EXTERN int Job_number;
 /* PROTOTYPES */
 int main(int argc, char *argv[], char *envp[]);
 void Get_parms(int argc, char *argv[] );
+void prmsg( char **msg );
 void usage(void);
 int Make_job( struct job *job );
 void get_job_number( struct job *job );
-double Copy_stdin( struct job *job );
+double Copy_STDIN( struct job *job );
 double Check_files( struct job *job );
 int Check_lpr_printable(char *file, int fd, struct stat *statb, int format );
 int is_exec( char *buf, int n);
@@ -59,6 +60,5 @@ void Dienoarg(int option);
 void Check_int_dup (int option, int *value, char *arg, int maxvalue);
 void Check_str_dup(int option, char **value, char *arg, int maxlen );
 void Check_dup(int option, int *value);
-int Start_worker( struct line_list *l, int fd );
 
 #endif
