@@ -1,10 +1,10 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
- * $Id: lprm.h,v 5.6 2000/12/25 01:51:21 papowell Exp papowell $
+ * $Id: lprm.h,v 1.30 2002/05/06 01:06:43 papowell Exp $
  ***************************************************************************/
 
 
@@ -21,7 +21,7 @@ EXTERN int LP_mode;    /* show all printers */
 int main(int argc, char *argv[], char *envp[]);
 void Do_removal(char **argv);
 void Get_parms(int argc, char *argv[] );
+void pr_msg( char **msg );
 void usage(void);
-int Start_worker( struct line_list *l, int fd );
 
 #endif
