@@ -1,10 +1,10 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
- * $Id: portable.h,v 5.17 2000/12/25 01:51:22 papowell Exp papowell $
+ * $Id: portable.h,v 1.30 2002/05/06 01:06:43 papowell Exp $
  ***************************************************************************/
 
 #ifndef _PLP_PORTABLE_H
@@ -96,6 +96,23 @@ LPRng requires ANSI Standard C compiler
 #if defined(sun)
 #endif
 
+/*************************************************************************
+ * SCO OpenServer 5.0.5
+ *************************************************************************/
+/* normal include files do not define MAXPATHLEN - rather PATHSIZE in 
+   sys/param.h */
+#ifdef sco
+#ifndef MAXPATHLEN
+#define MAXPATHLEN	PATHSIZE
+#endif
+/* SCO doesn't define the S_ISSOCK POSIX macro to use in testing the 
+   stat.st_mode structure member  - it appears as though a socket has
+   st_mode = 0020000 (same as character special) */
+#ifndef S_ISSOCK
+#define S_IFSOCK	0020000
+#define S_ISSOCK(m)	(((m) & S_IFMT) == S_IFSOCK)
+#endif
+#endif
 
 /*************************************************************************
  * Cray
@@ -268,6 +285,7 @@ LPRng requires ANSI Standard C compiler
 #include <netinet/in.h>
 #include <netdb.h>
 #include <arpa/inet.h>
+#include <sys/un.h>
 
 #include <sys/stat.h>
 #include <pwd.h>
@@ -337,10 +355,10 @@ typedef struct dirent plp_dir_t;
  * Patrick Powell Thu Apr  6 07:47:54 PDT 1995
  *********************************************************************/
 
-#ifdef HAVE_MALLOC_H
-# include <malloc.h>
-#else
-# if !defined(HAVE_STDLIB_H)
+#if !defined(HAVE_STDLIB_H)
+# ifdef HAVE_MALLOC_H
+#   include <malloc.h>
+# else
    void *malloc(size_t);
    void free(void *);
 # endif
@@ -619,7 +637,7 @@ XX ** NO VARARGS ** XX
  */
 
 #define NONBLOCK (O_NDELAY|O_NONBLOCK)
-#if defined(HPUX) && HPUX<1100
+#if defined(HPUX) && HPUX<110
 #  undef NONBLOCK
 #  define NONBLOCK (O_NONBLOCK)
 #  undef FD_SET_FIX
@@ -692,6 +710,19 @@ const char *inet_ntop( int family, const void *addr, char *strptr, size_t len );
 #endif
 
 
+/**********************************************************************
+ *  Cygwin Definitions
+ **********************************************************************/
+#ifdef __CYGWIN__
+#define ROOTUID 18
+#else
+#define ROOTUID 0
+#endif
+
+#ifndef HAVE_FLOCK_DEF
+extern int flock( int fd, int operation );
+#endif
+
 /**********************************************************************
  *  SUNOS Definitions
  **********************************************************************/
