@@ -1,10 +1,10 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
- * $Id: utilities.h,v 5.10 2000/12/25 01:51:23 papowell Exp papowell $
+ * $Id: utilities.h,v 1.30 2002/05/06 01:06:43 papowell Exp $
  ***************************************************************************/
 
 
@@ -14,8 +14,8 @@
 
 #include <setjmp.h>
 
-EXTERN int Alarm_timed_out;                                     /* flag */
-EXTERN int Timeout_pending;
+EXTERN volatile int Alarm_timed_out;      /* flag */
+EXTERN volatile int Timeout_pending;
 
 #if defined(HAVE_SIGLONGJMP)
 EXTERN sigjmp_buf Timeout_env;
@@ -54,6 +54,7 @@ void plp_sigpause( void );
 int safestrcasecmp (const char *s1, const char *s2);
 int safestrncasecmp (const char *s1, const char *s2, int len );
 int safestrcmp( const char *s1, const char *s2 );
+int safestrlen( const char *s1 );
 int safestrncmp( const char *s1, const char *s2, int len );
 char *safestrchr( const char *s1, int c );
 char *safestrrchr( const char *s1, int c );
@@ -75,19 +76,28 @@ void Set_timeout_signal_handler( int timeout, plp_sigfunc_t handler );
 void Set_timeout_alarm( int timeout );
 void Set_timeout_break( int timeout );
 void Clear_timeout( void );
-int To_root(void);
+void Setup_uid(void);
+int To_euid_root(void);
 int To_daemon(void);
 int To_user(void);
-int To_ruid(int uid);
-int To_uid( int uid );
-int setuid_wrapper(int to);
+int To_ruid(int ruid);
+int To_euid( int euid );
+int setuid_wrapper(uid_t to);
 int Full_daemon_perms(void);
 int Full_root_perms(void);
 int Full_user_perms(void);
 int Getdaemon(void);
 int Getdaemon_group(void);
+int Set_full_group( int euid, int gid );
 int Setdaemon_group(void);
 void Reset_daemonuid(void);
 double Space_avail( char *pathname );
+/* VARARGS2 */
+#ifdef HAVE_STDARGS
+ void safefprintf (int fd, char *format,...)
+#else
+ void safefprintf (va_alist) va_dcl
+#endif
+;
 
 #endif
